@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 #
 #    LinOTP - the open source solution for two factor authentication
-#    Copyright (C) 2010 - 2016 LSE Leading Security Experts GmbH
+#    Copyright (C) 2010 - 2019 KeyIdentity GmbH
 #
 #    This file is part of LinOTP server.
 #
@@ -19,9 +19,9 @@
 #    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #
 #
-#    E-mail: linotp@lsexperts.de
+#    E-mail: linotp@keyidentity.com
 #    Contact: www.linotp.org
-#    Support: www.lsexperts.de
+#    Support: www.keyidentity.com
 #
 
 
@@ -40,7 +40,7 @@ is achieved by its new, totally modular architecture.
 Tokenclasses
 ------------
 
-    LinOTP already comes with several tokenclasses defined in linotp.lib.tokens
+    LinOTP already comes with several tokenclasses defined in linotp.tokens
     But you can simply define your own tokenclass object. Take a look at
     the base class in tokenclass.py
 
@@ -48,7 +48,7 @@ UserIdResolvers
 ---------------
 
     LinOTP can use arbitrary methods to look up your user base - the userid
-    resolvers. With LinOPT comes a flatfile (passwd), the ldap/active directory
+    resolvers. With LinOTP comes a flatfile (passwd), the ldap/active directory
     resolver and a sql resolver.
 
 
@@ -62,9 +62,10 @@ the simple webui which come with linotp
 # indirectly) import any module that might not yet be installed when installing
 # LinOTP.
 
-__copyright__ = "2010 - 2016 LSE Leading Security Experts GmbH"
+__copyright__ = "2010 - 2019 KeyIdentity GmbH"
 __product__ = "LinOTP"
 __license__ = "Gnu AGPLv3"
 __contact__ = "www.linotp.org"
-__email__ = "linotp@lsexperts.de"
-__version__ = '2.8.1'
+__email__ = "linotp@keyidentity.com"
+__version__ = '2.11.1'
+__api__ = "2.0802"
