@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 #
 #    LinOTP - the open source solution for two factor authentication
-#    Copyright (C) 2010 - 2016 LSE Leading Security Experts GmbH
+#    Copyright (C) 2010 - 2019 KeyIdentity GmbH
 #
 #    This file is part of LinOTP server.
 #
@@ -19,9 +19,9 @@
 #    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #
 #
-#    E-mail: linotp@lsexperts.de
+#    E-mail: linotp@keyidentity.com
 #    Contact: www.linotp.org
-#    Support: www.lsexperts.de
+#    Support: www.keyidentity.com
 #
 """
 auth controller - to do authentication tests
@@ -41,7 +41,7 @@ from linotp.lib.reply import sendError
 from pylons import response
 from linotp.model.meta import Session
 
-import traceback
+from linotp.lib.config import getLinotpConfig
 
 log = logging.getLogger(__name__)
 
@@ -51,10 +51,7 @@ required = False
 
 class AuthController(BaseController):
 
-
-    def __before__(self, action, **params):
-
-        log.debug("[__before__]")
+    def __before__(self, action,):
 
         try:
 
@@ -67,9 +64,6 @@ class AuthController(BaseController):
             Session.close()
             return sendError(response, exx, context='before')
 
-        finally:
-            log.debug("[__before__::%r] done" % (action))
-
     def index(self):
         '''
         This is the method for testing authentication
@@ -80,7 +74,6 @@ class AuthController(BaseController):
         log.debug("[index] index, authenticating user")
         return render("/auth.mako")
 
-
     def index3(self):
         '''
         This is the method for testing authentication
@@ -91,6 +84,38 @@ class AuthController(BaseController):
         log.debug("[index3] index, authenticating user")
         return render("/auth3.mako")
 
+    def challenge_response(self):
+        '''
+        This is the method for testing challenge-response
+        authentication
+
+        Call it directly in your browser like this
+            http(s)://server/auth/challenge-response
+        '''
+        log.debug("[challenge-response] index, authenticating user")
+        return render("/auth-challenge-response.mako")
+
+    def qrtoken(self):
+        '''
+        This is the method for testing authentication
+        using your KeyIdentity QR Token
+
+        Call it directly in your browser like this
+            http(s)://server/auth/qrtoken
+        '''
+        log.debug("[qrtoken] authenticating user")
+        return render("/auth-qrtoken.mako")
+
+    def pushtoken(self):
+        '''
+        This is the method for testing authentication
+        using your KeyIdentity Push Token
+
+        Call it directly in your browser like this
+            http(s)://server/auth/pushtoken
+        '''
+        log.debug("[pushtoken] authenticating user")
+        return render("/auth-push.mako")
 
     def ocra(self):
         '''
@@ -102,7 +127,6 @@ class AuthController(BaseController):
         log.debug("[ocra] authenticating user")
         return render("/auth-ocra.mako")
 
-
     def ocra2(self):
         '''
         This is the method for testing ocra2 tokens
@@ -113,6 +137,4 @@ class AuthController(BaseController):
         log.debug("[ocra2] authenticating user")
         return render("/auth-ocra2.mako")
 
-
 #eof##########################################################################
-
