@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 #
 #    LinOTP - the open source solution for two factor authentication
-#    Copyright (C) 2010 - 2016 LSE Leading Security Experts GmbH
+#    Copyright (C) 2010 - 2019 KeyIdentity GmbH
 #
 #    This file is part of LinOTP server.
 #
@@ -19,9 +19,9 @@
 #    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #
 #
-#    E-mail: linotp@lsexperts.de
+#    E-mail: linotp@keyidentity.com
 #    Contact: www.linotp.org
-#    Support: www.lsexperts.de
+#    Support: www.keyidentity.com
 #
 """
 error controller - to display errors
@@ -31,14 +31,16 @@ error controller - to display errors
 
 import cgi
 
-from paste.urlparser    import PkgResourcesParser
-from pylons             import request
+from pylons import request
 from pylons.middleware  import error_document_template
-
 from pylons.controllers.util import forward
+
+from paste.urlparser import PkgResourcesParser
+
 from webhelpers.html.builder import literal
 
-from linotp.lib.base    import BaseController
+from linotp.lib.base import BaseController
+from linotp.lib.util import str2unicode
 
 
 class ErrorController(BaseController):
@@ -47,13 +49,20 @@ class ErrorController(BaseController):
         """Render the error document"""
         resp = request.environ.get('pylons.original_response')
         if resp is not None:
-            content = literal(resp.body)
+            unicode_body = str2unicode(resp.body)
+            content = literal(unicode_body)
         else:
-            content = cgi.escape(request.GET.get('message', ''))
+            message = request.GET.get('message',
+                                      request.POST.get('message', ''))
+            content = cgi.escape(message)
+
+        code = request.GET.get('code',
+                               request.POST.get('code',
+                                               unicode(resp.status_int)))
 
         page = error_document_template % \
             dict(prefix=request.environ.get('SCRIPT_NAME', ''),
-                 code=cgi.escape(request.GET.get('code', unicode(resp.status_int))),
+                 code=cgi.escape(code),
                  message=content)
         return page
 
