@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 #
 #    LinOTP - the open source solution for two factor authentication
-#    Copyright (C) 2010 - 2016 LSE Leading Security Experts GmbH
+#    Copyright (C) 2010 - 2019 KeyIdentity GmbH
 #
 #    This file is part of LinOTP server.
 #
@@ -19,17 +19,14 @@
 #    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #
 #
-#    E-mail: linotp@lsexperts.de
+#    E-mail: linotp@keyidentity.com
 #    Contact: www.linotp.org
-#    Support: www.lsexperts.de
+#    Support: www.keyidentity.com
 #
 """
 gettoken controller - to retrieve OTP values
 """
 
-
-
-
 import logging
 
 from pylons import tmpl_context as c
@@ -37,22 +34,27 @@ from pylons import request, response
 from pylons import config
 from pylons.templating import render_mako as render
 
-
+import linotp.model
 from linotp.lib.base import BaseController
 
-from linotp.lib.util import  getParam, check_session
-from linotp.lib.util import  get_client
-from linotp.lib.user import  getUserFromParam
-from linotp.lib.user import  getDefaultRealm
-from linotp.lib.user import  getUserFromRequest
+from linotp.lib.util import getParam, check_session
+from linotp.lib.util import get_client
+from linotp.lib.user import getUserFromParam
+from linotp.lib.user import getDefaultRealm
+from linotp.lib.user import getUserFromRequest
 
 
-from linotp.lib.token import getTokenType, getOtp, get_multi_otp, getTokens4UserOrSerial
+from linotp.lib.token import getTokenType
+from linotp.lib.token import getOtp
+from linotp.lib.token import get_multi_otp
+from linotp.lib.token import getTokens4UserOrSerial
+
 from linotp.lib.policy import checkPolicyPre, PolicyException
 from linotp.lib.reply import sendResult, sendError
 
-from linotp.model.meta import Session
-import traceback
+from linotp.lib.context import request_context
+
+Session = linotp.model.Session
 
 audit = config.get('audit')
 
@@ -65,11 +67,12 @@ log = logging.getLogger(__name__)
 class GettokenController(BaseController):
 
     '''
-    The linotp.controllers are the implementation of the web-API to talk to the LinOTP server.
-    The ValidateController is used to validate the username with its given OTP value.
+    The linotp.controllers are the implementation of the web-API to talk to
+    the LinOTP server. The ValidateController is used to validate the username
+    with its given OTP value.
 
-    The Tagespasswort Token uses this controller to retrieve the current OTP value of
-    the Token and be able to set it in the application
+    The Tagespasswort Token uses this controller to retrieve the current
+    OTP value of the Token and be able to set it in the application
     The functions of the GettokenController are invoked like this
 
         https://server/gettoken/<functionname>
@@ -77,36 +80,32 @@ class GettokenController(BaseController):
     The functions are described below in more detail.
     '''
 
-    def __before__(self, action, **params):
-
-        log.debug("[__before__]")
+    def __before__(self, action):
 
         try:
-            audit.initialize()
-            c.audit['client'] = get_client()
-            check_session()
+            c.audit = request_context['audit']
+            c.audit['client'] = get_client(request)
+            check_session(request)
 
         except Exception as exx:
-            log.exception("[__before__::%r] exception %r" % (action, exx))
+            log.exception("[__before__::%r] exception %r", action, exx)
             Session.rollback()
             Session.close()
             return sendError(response, exx, context='before')
 
-        finally:
-            log.debug("[__before__::%r] done" % (action))
-
     def __after__(self):
         c.audit['administrator'] = getUserFromRequest(request).get("login")
-        if request.params.has_key('serial'):
-                c.audit['serial'] = request.params['serial']
-                c.audit['token_type'] = getTokenType(request.params['serial'])
+        if 'serial' in self.request_params:
+            serial = self.request_params['serial']
+            c.audit['serial'] = serial
+            c.audit['token_type'] = getTokenType(serial)
         audit.log(c.audit)
 
-
     def getmultiotp(self):
         '''
-        This function is used to retrieve multiple otp values for a given user or a given serial
-        If the user has more than one token, the list of the tokens is returend.
+        This function is used to retrieve multiple otp values for a given user
+        or a given serial. If the user has more than one token, the list of
+        the tokens is returend.
 
         method:
             gettoken/getmultiotp
@@ -124,7 +123,7 @@ class GettokenController(BaseController):
         if "True" != getotp_active:
             return sendError(response, "getotp is not activated.", 0)
 
-        param = request.params
+        param = self.request_params
         ret = {}
 
         try:
@@ -134,14 +133,15 @@ class GettokenController(BaseController):
             view = getParam(param, "view", optional)
 
             r1 = checkPolicyPre('admin', 'getotp', param)
-            log.debug("[getmultiotp] admin-getotp returned %s" % r1)
+            log.debug("[getmultiotp] admin-getotp policy: %s", r1)
 
             max_count = checkPolicyPre('gettoken', 'max_count', param)
-            log.debug("[getmultiotp] checkpolicypre returned %s" % max_count)
+            log.debug("[getmultiotp] maxcount policy: %s", max_count)
             if count > max_count:
                 count = max_count
 
-            log.debug("[getmultiotp] retrieving OTP value for token %s" % serial)
+            log.debug("[getmultiotp] retrieving OTP value for token %s",
+                      serial)
             ret = get_multi_otp(serial, count=int(count), curTime=curTime)
             ret["serial"] = serial
 
@@ -152,28 +152,27 @@ class GettokenController(BaseController):
                 c.ret = ret
                 return render('/manage/multiotp_view.mako')
             else:
-                return sendResult(response, ret , 0)
+                return sendResult(response, ret, 0)
 
         except PolicyException as pe:
-            log.exception("[getotp] gettoken/getotp policy failed: %r" % pe)
+            log.exception("[getotp] gettoken/getotp policy failed: %r", pe)
             Session.rollback()
             return sendError(response, unicode(pe), 1)
 
-        except Exception as e:
-            log.exception("[getmultiotp] gettoken/getmultiotp failed: %r" % e)
+        except Exception as exx:
+            log.exception("[getmultiotp] gettoken/getmultiotp failed: %r", exx)
             Session.rollback()
-            return sendError(response, "gettoken/getmultiotp failed: %s"
-                             % unicode(e), 0)
+            return sendError(response, "gettoken/getmultiotp failed: %r" % exx,
+                             0)
 
         finally:
             Session.close()
-            log.debug("[getmultiotp] done")
-
 
     def getotp(self):
         '''
-        This function is used to retrieve the current otp value for a given user or a given serial
-        If the user has more than one token, the list of the tokens is returend.
+        This function is used to retrieve the current otp value for a given
+        user or a given serial. If the user has more than one token, the list
+        of the tokens is returend.
 
         method:
             gettoken/getotp
@@ -182,7 +181,7 @@ class GettokenController(BaseController):
             user    - username / loginname
             realm   - additional realm to match the user to a useridresolver
             serial  - the serial number of the token
-            curTime - used ONY for internal testing: datetime.datetime object
+            curTime - used ONLY for internal testing: datetime.datetime object
 
         returns:
             JSON response
@@ -192,7 +191,7 @@ class GettokenController(BaseController):
         if "True" != getotp_active:
             return sendError(response, "getotp is not activated.", 0)
 
-        param = request.params
+        param = self.request_params
         ret = {}
         res = -1
         otpval = ""
@@ -202,7 +201,7 @@ class GettokenController(BaseController):
         try:
 
             serial = getParam(param, "serial", optional)
-            user = getUserFromParam(param, optional)
+            user = getUserFromParam(param)
             curTime = getParam(param, "curTime", optional)
 
             c.audit['user'] = user.login
@@ -210,32 +209,42 @@ class GettokenController(BaseController):
                 c.audit['realm'] = user.realm or getDefaultRealm()
 
             if serial:
-                log.debug("[getotp] retrieving OTP value for token %s" % serial)
+                log.debug("[getotp] retrieving OTP value for token %s", serial)
+
             elif user.login:
-                log.debug("[getotp] retrieving OTP value for token for user %s@%s" % (user.login, user.realm))
+                log.debug("[getotp] retrieving OTP value for token for user "
+                          "%s@%s", user.login, user.realm)
+
                 toks = getTokens4UserOrSerial(user, serial)
                 tokennum = len(toks)
+
                 if tokennum > 1:
-                    log.debug("[getotp] The user has more than one token. Returning the list of serials")
+                    log.debug("[getotp] The user has more than one token."
+                              "Returning the list of serials")
                     res = -3
                     for token in toks:
                         serials.append(token.getSerial())
                 elif 1 == tokennum:
                     serial = toks[0].getSerial()
-                    log.debug("[getotp] retrieving OTP for token %s for user %s@%s" %
-                                (serial, user.login, user.realm))
+                    log.debug("[getotp] retrieving OTP for token %s for user"
+                              " %s@%s", serial, user.login, user.realm)
                 else:
-                    log.debug("[getotp] no token found for user %s@%s" % (user.login, user.realm))
+                    log.debug("[getotp] no token found for user %s@%s",
+                              user.login, user.realm)
                     res = -4
             else:
                 res = -5
 
-            # if a serial was given or a unique serial could be received from the given user.
+            # if a serial was given or a unique serial could be
+            # received from the given user.
+
             if serial:
                 max_count = checkPolicyPre('gettoken', 'max_count', param)
-                log.debug("[getmultiotp] checkpolicypre returned %s" % max_count)
+                log.debug("[getmultiotp] max_count policy: %s", max_count)
                 if max_count <= 0:
-                    return sendError(response, "The policy forbids receiving OTP values for the token %s in this realm" % serial , 1)
+                    return sendError(response, "The policy forbids receiving"
+                                     " OTP values for the token %s in "
+                                     "this realm" % serial , 1)
 
                 (res, pin, otpval, passw) = getOtp(serial, curTime=curTime)
 
@@ -246,14 +255,16 @@ class GettokenController(BaseController):
                 if -1 == otpval:
                     ret['description'] = "No Token with this serial number"
                 if -2 == otpval:
-                    ret['description'] = "This Token does not support the getOtp function"
+                    ret['description'] = ("This Token does not support the"
+                                          " getOtp function")
                 if -3 == otpval:
                     ret['description'] = "The user has more than one token"
                     ret['serials'] = serials
                 if -4 == otpval:
                     ret['description'] = "No Token found for this user"
                 if -5 == otpval:
-                    ret['description'] = "you need to provide a user or a serial"
+                    ret['description'] = ("you need to provide a user or "
+                                          "a serial")
             else:
                 ret['result'] = True
                 ret['otpval'] = otpval
@@ -261,22 +272,21 @@ class GettokenController(BaseController):
                 ret['pass'] = passw
 
             Session.commit()
-            return sendResult(response, ret , 0)
+            return sendResult(response, ret, 0)
 
         except PolicyException as pe:
-            log.exception("[getotp] gettoken/getotp policy failed: %r" % pe)
+            log.exception("[getotp] gettoken/getotp policy failed: %r", pe)
             Session.rollback()
             return sendError(response, unicode(pe), 1)
 
-        except Exception as e:
-            log.exception("[getotp] gettoken/getotp failed: %r" % e)
+        except Exception as exx:
+            log.exception("[getotp] gettoken/getotp failed: %r", exx)
             Session.rollback()
-            return sendError(response, "gettoken/getotp failed: %s" % unicode(e), 0)
+            return sendError(response, "gettoken/getotp failed: %s" %
+                             unicode(exx), 0)
 
         finally:
             Session.close()
-            log.debug('[getotp] done')
 
 
 #eof###########################################################################
-
