@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 #
 #    LinOTP - the open source solution for two factor authentication
-#    Copyright (C) 2010 - 2016 LSE Leading Security Experts GmbH
+#    Copyright (C) 2010 - 2019 KeyIdentity GmbH
 #
 #    This file is part of LinOTP server.
 #
@@ -19,83 +19,101 @@
 #    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #
 #
-#    E-mail: linotp@lsexperts.de
+#    E-mail: linotp@keyidentity.com
 #    Contact: www.linotp.org
-#    Support: www.lsexperts.de
+#    Support: www.keyidentity.com
 #
 
 """
 manage controller - In provides the web gui management interface
 """
-
+import base64
 import os
-
-try:
-    import json
-except ImportError:
-    import simplejson as json
+import logging
+import json
 
 from pylons import request, response, config, tmpl_context as c
-from linotp.lib.base import BaseController
 from pylons.templating import render_mako as render
-from mako.exceptions import CompileException
-
+from pylons.i18n.translation import _
 from paste.deploy.converters import asbool
 
+from mako.exceptions import CompileException
+
+from linotp.lib.base import BaseController
+from linotp.lib.error import ParameterError
+
 # Our Token stuff
-from linotp.lib.token   import TokenIterator
-from linotp.lib.token   import getTokenType
-from linotp.lib.token   import newToken
+from linotp.lib.tokeniterator import TokenIterator
+from linotp.lib.token import getTokenType
+from linotp.tokens import tokenclass_registry
 
 
-from linotp.lib.user    import getUserFromParam, getUserFromRequest
-from linotp.lib.user    import getUserList, User
+from linotp.lib.user import getUserFromParam, getUserFromRequest
+from linotp.lib.user import getUserList, User
 
-from linotp.lib.util    import getParam
-from linotp.lib.util    import check_session
-from linotp.lib.util    import get_version
-from linotp.lib.util    import get_copyright_info
-from linotp.lib.reply   import sendError
+from linotp.lib.util import check_session
+from linotp.lib.util import get_version
+from linotp.lib.util import get_copyright_info
+from linotp.lib.type_utils import boolean
 
-from linotp.lib.util    import remove_empty_lines
+from linotp.lib.reply import sendError
+from linotp.lib.reply import sendResult
+
+from linotp.lib.util import remove_empty_lines
 from linotp.lib.util import get_client
 from linotp.lib.util import unicode_compare
-from linotp.model.meta import Session
+from linotp.lib.realm import getRealms
 
-from linotp.lib.policy import checkPolicyPre, PolicyException, getAdminPolicies, getPolicyDefinitions
+from linotp.lib.policy import checkPolicyPre
+from linotp.lib.policy import PolicyException
+from linotp.lib.policy import getAdminPolicies
+from linotp.lib.policy.definitions import getPolicyDefinitions
 
-from pylons.i18n.translation import _
-
-audit = config.get('audit')
-
-import logging
-
-log = logging.getLogger(__name__)
+from linotp.lib.context import request_context
 
 from linotp.lib.ImportOTP import getKnownTypes, getImportText
+import linotp.model.meta
+
+Session = linotp.model.meta.Session
+
+audit = config.get('audit')
+log = logging.getLogger(__name__)
+
 KNOWN_TYPES = getKnownTypes()
 IMPORT_TEXT = getImportText()
+
 log.info("importing linotp.lib. Known import types: %s" % IMPORT_TEXT)
 
 
-optional = True
-required = False
-
 class ManageController(BaseController):
 
     def __before__(self, action, **params):
 
-        log.debug("[__before__::%r] %r" % (action, params))
-
         try:
-            audit.initialize()
+            c.audit = request_context['audit']
             c.audit['success'] = False
-            c.audit['client'] = get_client()
+            c.audit['client'] = get_client(request)
 
             c.version = get_version()
+            c.version_ref = base64.encodestring(c.version)[:6]
+
             c.licenseinfo = get_copyright_info()
             c.polDefs = getPolicyDefinitions()
 
+            c.display_provider = boolean(
+                    request_context['Config'].get('display_provider', True))
+
+            # -------------------------------------------------------------- --
+
+            # check for support of setting admin password
+
+            c.admin_can_change_password = False
+            if ('linotpadmin.user' in config and
+                'linotpadmin.password' in config):
+                c.admin_can_change_password = True
+
+            # -------------------------------------------------------------- --
+
             # Session handling for the functions, that show data:
             # Also exclude custom-style.css, since the CSRF check
             # will always fail and return a HTTP 401 anyway.
@@ -110,7 +128,7 @@ class ManageController(BaseController):
                                         '/manage/custom-style.css']:
                 pass
             else:
-                check_session()
+                check_session(request)
 
         except Exception as exx:
             log.exception("[__before__::%r] exception %r" % (action, exx))
@@ -122,17 +140,16 @@ class ManageController(BaseController):
             log.debug("[__before__::%r] done" % (action))
 
     def __after__(self):
-
-        if c.audit['action'] in [ 'manage/tokenview_flexi',
-                                'manage/userview_flexi' ]:
+        if c.audit['action'] in ['manage/tokenview_flexi',
+                                 'manage/userview_flexi' ]:
             c.audit['administrator'] = getUserFromRequest(request).get("login")
-            if request.params.has_key('serial'):
-                    c.audit['serial'] = request.params['serial']
-                    c.audit['token_type'] = getTokenType(request.params['serial'])
+            if 'serial' in self.request_params:
+                serial = self.request_params['serial']
+                c.audit['serial'] = serial
+                c.audit['token_type'] = getTokenType(serial)
 
             audit.log(c.audit)
 
-
     def index(self):
         '''
         This is the main function of the management web UI
@@ -142,6 +159,7 @@ class ManageController(BaseController):
             c.debug = asbool(config.get('debug', False))
             c.title = "LinOTP Management"
             admin_user = getUserFromRequest(request)
+
             if admin_user.has_key('login'):
                 c.admin = admin_user['login']
 
@@ -149,7 +167,18 @@ class ManageController(BaseController):
             c.importers = IMPORT_TEXT
             c.help_url = config.get('help_url')
 
-            ## add render info for token type config
+            # -------------------------------------------------------------- --
+
+            # check for support of setting admin password
+
+            c.admin_can_change_password = False
+            if ('linotpadmin.user' in config and
+                'linotpadmin.password' in config):
+                c.admin_can_change_password = True
+
+            # -------------------------------------------------------------- --
+
+            # add render info for token type config
             confs = _getTokenTypeConfig('config')
             token_config_tab = {}
             token_config_div = {}
@@ -157,12 +186,12 @@ class ManageController(BaseController):
                 tab = ''
                 div = ''
                 try:
-                    #loc = conf +'_token_settings'
+                    # loc = conf +'_token_settings'
                     tab = confs.get(conf).get('title')
-                    #tab = '<li ><a href=#'+loc+'>'+tab+'</a></li>'
+                    # tab = '<li ><a href=#'+loc+'>'+tab+'</a></li>'
 
                     div = confs.get(conf).get('html')
-                    #div = +div+'</div>'
+                    # div = +div+'</div>'
                 except Exception as e:
                     log.debug('[index] no config info for token type %s  (%r)' % (conf, e))
 
@@ -173,9 +202,9 @@ class ManageController(BaseController):
             c.token_config_tab = token_config_tab
             c.token_config_div = token_config_div
 
-            ##  add the enrollment fragments from the token definition
-            ##  tab: <option value="ocra">${_("OCRA - challenge/response Token")}</option>
-            ##  div: "<div id='"+ tt + "'>"+enroll+"</div>"
+            #  add the enrollment fragments from the token definition
+            #  tab: <option value="ocra">${_("OCRA - challenge/response Token")}</option>
+            #  div: "<div id='"+ tt + "'>"+enroll+"</div>"
             enrolls = _getTokenTypeConfig('init')
 
             token_enroll_tab = {}
@@ -198,7 +227,11 @@ class ManageController(BaseController):
 
             c.tokentypes = _getTokenTypes()
 
-            http_host = request.environ.get("HTTP_HOST")
+            # Use HTTP_X_FORWARDED_HOST in preference to HTTP_HOST
+            # in case we're running behind a reverse proxy
+            http_host = request.environ.get("HTTP_X_FORWARDED_HOST", '')
+            if not http_host:
+                http_host = request.environ.get("HTTP_HOST")
             url_scheme = request.environ.get("wsgi.url_scheme")
             c.logout_url = "%s://log-me-out:fake@%s/manage/logout" % (url_scheme, http_host)
 
@@ -218,20 +251,16 @@ class ManageController(BaseController):
 
         finally:
             Session.close()
-            log.debug('[index] done')
 
 
     def tokentype(self):
         '''
         '''
         c.title = 'TokenTypeInfo'
-        g = config['pylons.app_globals']
-        tokens = g.tokenclasses
         ttinfo = []
-        ttinfo.extend(tokens.keys())
-        for tok in tokens:
-            tclass = tokens.get(tok)
-            tclass_object = newToken(tclass)
+        ttinfo.extend(tokenclass_registry.keys())
+        for tok in tokenclass_registry:
+            tclass_object = tokenclass_registry.get(tok)
             if hasattr(tclass_object, 'getClassType'):
                 ii = tclass_object.getClassType()
                 ttinfo.append(ii)
@@ -264,6 +293,7 @@ class ManageController(BaseController):
         '''
         c.title = "LinOTP Management"
         c.tokenArray = []
+        c.getotp_active = config.get("linotpGetotp.active", "False") == "True"
         return render('/manage/tokenview.mako')
 
 
@@ -277,7 +307,7 @@ class ManageController(BaseController):
 
     def custom_style(self):
         '''
-        If this action was called, the user hasn't created a custom-style.css yet. To avoid hitting
+        If this action was called, the user hasn't created a custom css yet. To avoid hitting
         the debug console over and over, we serve an empty file.
         '''
         response.headers['Content-type'] = 'text/css'
@@ -300,30 +330,42 @@ class ManageController(BaseController):
         Unlike the complex /admin/show function, it only returns a
         simple array of the tokens.
         '''
-        param = request.params
+        param = self.request_params
 
         try:
-            #serial  = getParam(param,"serial",optional)
-            c.page = getParam(param, "page", optional)
-            c.filter = getParam(param, "query", optional)
-            c.qtype = getParam(param, "qtype", optional)
-            c.sort = getParam(param, "sortname", optional)
-            c.dir = getParam(param, "sortorder", optional)
-            c.psize = getParam(param, "rp", optional)
+            c.page = param.get("page")
+            c.filter = param.get("query")
+            c.qtype = param.get("qtype")
+            c.sort = param.get("sortname")
+            c.dir = param.get("sortorder")
+            c.psize = param.get("rp")
 
             filter_all = None
             filter_realm = None
             user = User()
 
             if c.qtype == "loginname":
-                if "@" in c.filter:
-                    (login, realm) = c.filter.split("@")
-                    user = User(login, realm)
-                else:
-                    user = User(c.filter)
+
+                # we take by default the given expression as a loginname,
+                # especially if it contains a "*" wildcard.
+                # it only might be more, a user and a realm, if there
+                # is an '@' sign in the loginname and the part after the
+                # last '@' sign is matching an existing realm
+
+                user = User(login=c.filter)
+
+                if "*" not in c.filter and "@" in c.filter:
+
+                    login, _ , realm = c.filter.rpartition("@")
+
+                    if realm.lower() in getRealms():
+                        user = User(login, realm)
+                        if not user.exists():
+                            user = User(login=c.filter)
 
             elif c.qtype == "all":
                 filter_all = c.filter
+
             elif c.qtype == "realm":
                 filter_realm = c.filter
 
@@ -359,8 +401,8 @@ class ManageController(BaseController):
             lines = []
             for tok in c.tokenArray:
                 lines.append(
-                    { 'id' : tok['LinOtp.TokenSerialnumber'],
-                        'cell': [
+                    {'id' : tok['LinOtp.TokenSerialnumber'],
+                     'cell': [
                             tok['LinOtp.TokenSerialnumber'],
                             tok['LinOtp.Isactive'],
                             tok['User.username'],
@@ -373,12 +415,12 @@ class ManageController(BaseController):
                             tok['LinOtp.CountWindow'],
                             tok['LinOtp.SyncWindow'],
                             tok['LinOtp.Userid'],
-                            tok['LinOtp.IdResClass'], ]
+                            tok['LinOtp.IdResClass'].split('.')[-1],
+                            ]
                     }
                     )
 
             # We need to return 'page', 'total', 'rows'
-            response.content_type = 'application/json'
             res = { "page": int(c.page),
                 "total": c.resultset['tokens'],
                 "rows": lines }
@@ -386,7 +428,8 @@ class ManageController(BaseController):
             c.audit['success'] = True
 
             Session.commit()
-            return json.dumps(res, indent=3)
+            # The flexi handler should support std LinOTP output
+            return sendResult(response, res)
 
         except PolicyException as pe:
             log.exception("[tokenview_flexi] Error during checking policies: %r" % pe)
@@ -400,8 +443,6 @@ class ManageController(BaseController):
 
         finally:
             Session.close()
-            log.debug("[tokenview_flexi] done")
-
 
     def userview_flexi(self):
         '''
@@ -409,23 +450,23 @@ class ManageController(BaseController):
         Unlike the complex /admin/userlist function, it only returns a
         simple array of the tokens.
         '''
-        param = request.params
+        param = self.request_params
 
         try:
-            #serial  = getParam(param,"serial",optional)
-            c.page = getParam(param, "page", optional)
-            c.filter = getParam(param, "query", optional)
-            qtype = getParam(param, "qtype", optional)
-            c.sort = getParam(param, "sortname", optional)
-            c.dir = getParam(param, "sortorder", optional)
-            c.psize = getParam(param, "rp", optional)
-            c.realm = getParam(param, "realm", optional)
 
-            user = getUserFromParam(param, optional)
+            c.page = param.get("page")
+            c.filter = param.get("query")
+            qtype = param.get("qtype")
+            c.sort = param.get("sortname")
+            c.dir = param.get("sortorder")
+            c.psize = param.get("rp")
+            c.realm = param.get("realm")
+
+            user = getUserFromParam(param)
             # check admin authorization
             # check if we got a realm or resolver, that is ok!
-            checkPolicyPre('admin', 'userlist', { 'user': user.login,
-                                                 'realm' : c.realm })
+            checkPolicyPre('admin', 'userlist', {'user': user.login,
+                                                 'realm': c.realm})
 
             if c.filter == "":
                 c.filter = "*"
@@ -497,7 +538,7 @@ class ManageController(BaseController):
             c.audit['success'] = True
 
             Session.commit()
-            return json.dumps(res, indent=3)
+            return sendResult(response, res)
 
         except PolicyException as pe:
             log.exception("[userview_flexi] Error during checking policies: %r" % pe)
@@ -511,40 +552,41 @@ class ManageController(BaseController):
 
         finally:
             Session.close()
-            log.debug('[userview_flexi] done')
-
 
     def tokeninfo(self):
         '''
-        this returns the contents of /admin/show?serial=xyz in a html format
+        this returns the contents of /admin/show?serial=xyz in an html format
         '''
-        param = request.params
+        param = self.request_params
 
         try:
-            serial = getParam(param, 'serial', required)
+            try:
+                serial = param['serial']
+            except KeyError:
+                raise ParameterError("Missing parameter: 'serial'")
 
             filterRealm = ""
             # check admin authorization
             res = checkPolicyPre('admin', 'show', param)
 
-            filterRealm = res['realms']
             # check if policies are active at all
             # If they are not active, we are allowed to SHOW any tokens.
-            pol = getAdminPolicies("show")
-            if not pol['active']:
-                filterRealm = ["*"]
+            filterRealm = ["*"]
+            if res['active'] and res['realms']:
+                filterRealm = res['realms']
 
-            log.info("[tokeninfo] admin >%s< may display the following realms: %s" % (res['admin'], filterRealm))
+            log.info("[tokeninfo] admin >%s< may display the following realms:"
+                     " %s" % (res['admin'], filterRealm))
             log.info("[tokeninfo] displaying tokens: serial: %s", serial)
 
-            toks = TokenIterator(User("", "", ""), serial, filterRealm=filterRealm)
+            toks = TokenIterator(User("", "", ""), serial,
+                                 filterRealm=filterRealm)
 
-            ### now row by row
+            # now row by row
             lines = []
             for tok in toks:
                 lines.append(tok)
             if len(lines) > 0:
-
                 c.tokeninfo = lines[0]
             else:
                 c.tokeninfo = {}
@@ -553,7 +595,8 @@ class ManageController(BaseController):
                 if "LinOtp.TokenInfo" == k:
                     try:
                         # Try to convert string to Dictionary
-                        c.tokeninfo['LinOtp.TokenInfo'] = json.loads(c.tokeninfo['LinOtp.TokenInfo'])
+                        c.tokeninfo['LinOtp.TokenInfo'] = json.loads(
+                                            c.tokeninfo['LinOtp.TokenInfo'])
                     except:
                         pass
 
@@ -571,7 +614,6 @@ class ManageController(BaseController):
 
         finally:
             Session.close()
-            log.debug('[tokeninfo] done')
 
 
     def logout(self):
@@ -619,12 +661,9 @@ class ManageController(BaseController):
 
         finally:
             Session.close()
-            log.debug("[help] done")
 
+# ###########################################################
 
-
-
-############################################################
 def _getTokenTypes():
     '''
         _getTokenTypes - retrieve the list of dynamic tokens and their title section
@@ -633,20 +672,12 @@ def _getTokenTypes():
         :rtype:  dict
     '''
 
-    glo = config['pylons.app_globals']
-    tokenclasses = glo.tokenclasses
-
-    tokens = []
-    tokens.extend(tokenclasses.keys())
-
     tinfo = {}
-    for tok in tokens:
-        if tok in tokenclasses.keys():
-            tclass = tokenclasses.get(tok)
-            tclass_object = newToken(tclass)
-            if hasattr(tclass_object, 'getClassInfo'):
-                ii = tclass_object.getClassInfo('title') or tok
-                tinfo[tok] = _(ii)
+
+    for tclass_object in set(tokenclass_registry.values()):
+        tok = tclass_object.getClassType()
+        if hasattr(tclass_object, 'getClassInfo'):
+            tinfo[tok] = _(tclass_object.getClassInfo('title') or tok)
 
     return tinfo
 
@@ -664,18 +695,16 @@ def _getTokenTypeConfig(section='config'):
     '''
 
     res = {}
-    g = config['pylons.app_globals']
-    tokenclasses = g.tokenclasses
 
-    for tok in tokenclasses.keys():
-        tclass = tokenclasses.get(tok)
-        tclass_object = newToken(tclass)
+    for tclass_object in set(tokenclass_registry.values()):
+        tok = tclass_object.getClassType()
+
         if hasattr(tclass_object, 'getClassInfo'):
 
             conf = tclass_object.getClassInfo(section, ret={})
 
-            ## set globale render scope, so that the mako
-            ## renderer will return only a subsection from the template
+            # set globale render scope, so that the mako
+            # renderer will return only a subsection from the template
             p_html = ''
             t_html = ''
             try:
@@ -684,14 +713,14 @@ def _getTokenTypeConfig(section='config'):
                 p_html = render(os.path.sep + page.get('html'))
                 p_html = remove_empty_lines(p_html)
 
-
                 tab = conf.get('title')
                 c.scope = tab.get('scope')
                 t_html = render(os.path.sep + tab.get('html'))
                 t_html = remove_empty_lines(t_html)
 
             except CompileException as ex:
-                log.exception("[_getTokenTypeConfig] compile error while processing %r.%r:" % (tok, section))
+                log.exception("[_getTokenTypeConfig] compile error while "
+                              "processing %r.%r:" % (tok, section))
                 log.error("[_getTokenTypeConfig] %r" % ex)
                 raise Exception(ex)
 
@@ -705,4 +734,3 @@ def _getTokenTypeConfig(section='config'):
     return res
 
 ############################################################
-
