@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 #
 #    LinOTP - the open source solution for two factor authentication
-#    Copyright (C) 2010 - 2016 LSE Leading Security Experts GmbH
+#    Copyright (C) 2010 - 2019 KeyIdentity GmbH
 #
 #    This file is part of LinOTP server.
 #
@@ -19,60 +19,62 @@
 #    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #
 #
-#    E-mail: linotp@lsexperts.de
+#    E-mail: linotp@keyidentity.com
 #    Contact: www.linotp.org
-#    Support: www.lsexperts.de
+#    Support: www.keyidentity.com
 #
 """
 ocra controller - Interface for the Challenge Response Token (OCRA)
 """
-
-import traceback
 import logging
 from urllib import urlencode
 
-
+import webob
 from pylons import request, response, config, tmpl_context as c
-from linotp.model.meta import Session
 
+
+from linotp.lib.auth.validate import ValidationHandler
 from linotp.lib.base import BaseController
-from linotp.lib.util  import check_session
-from linotp.lib.util import get_client
-from linotp.lib.error   import ParameterError
+from linotp.lib.error import ParameterError
 
-from linotp.lib.util    import getParam, getLowerParams
-from linotp.lib.reply   import sendResult, sendError
-from linotp.lib.reply   import sendQRImageResult
+from linotp.lib.policy import PolicyException
+from linotp.lib.policy import checkPolicyPre
+from linotp.lib.realm import getDefaultRealm
 
-
-from linotp.lib.realm   import getDefaultRealm
-from linotp.lib.user    import getUserFromRequest
-from linotp.lib.user    import getUserInfo
-from linotp.lib.user    import User
-
-from linotp.lib.policy  import checkPolicyPre
-from linotp.lib.policy  import PolicyException
+from linotp.lib.reply import sendQRImageResult
+from linotp.lib.reply import sendResult, sendError
 
 from linotp.lib.token import getTokens4UserOrSerial
-from linotp.lib.tokenclass import OcraTokenClass
+from linotp.tokens.ocra.ocratoken import OcraTokenClass
 
-from linotp.lib.token import checkSerialPass
-from linotp.lib.user import  getUserFromParam
-import webob
+from linotp.lib.user import User
+from linotp.lib.user import getUserFromParam
+from linotp.lib.user import getUserFromRequest
+from linotp.lib.user import getUserInfo
 
+from linotp.lib.util import check_session
+from linotp.lib.util import getLowerParams
+from linotp.lib.util import get_client
+
+from linotp.lib.context import request_context
+import linotp.model
+
+from linotp.lib.error import UserError
+
+import linotp.model.meta
+
+Session = linotp.model.meta.Session
 
 
 audit = config.get('audit')
 
 log = logging.getLogger(__name__)
 
-optional = True
-required = False
-
 
 class OcraController(BaseController):
     '''
-    The OcraController implements challenges/response tokens according to RFC 6287
+    The OcraController implements challenges/response tokens
+    according to RFC 6287
     '''
 
     def __before__(self, action, **params):
@@ -80,21 +82,24 @@ class OcraController(BaseController):
         Here we see, what action is to be called and check the authorization
         '''
 
-        log.debug("[__before__::%r] %r" % (action, params))
-
         try:
 
-            audit.initialize()
+            c.audit = request_context['audit']
             c.audit['success'] = False
-            c.audit['client'] = get_client()
+            c.audit['client'] = get_client(request)
             if action != "check_t":
-                check_session()
+                check_session(request)
 
+            request_context['Audit'] = audit
             return response
 
         except webob.exc.HTTPUnauthorized as acc:
-            ## the exception, when an abort() is called if forwarded
-            log.exception("[__before__::%r] webob.exception %r" % (action, acc))
+
+            # the exception, when an abort() is called if forwarded
+
+            log.exception("[__before__::%r] webob.exception %r"
+                          % (action, acc))
+
             Session.rollback()
             Session.close()
             raise acc
@@ -105,8 +110,6 @@ class OcraController(BaseController):
             Session.close()
             return sendError(response, exx, context='before')
 
-        finally:
-            log.debug("[__before__::%r] done" % (action))
 
     def __after__(self):
         c.audit['administrator'] = getUserFromRequest(request).get("login")
@@ -135,7 +138,8 @@ class OcraController(BaseController):
               (either serial or user is required)
 
             * data: (required - String: URLendoced)
-              These are the display data, that can be used to generate the challenge
+              These are the display data, that can be used to generate the
+              challenge
 
         remark:
             the app will report a wrong qrcode, if the policy::
@@ -170,70 +174,79 @@ class OcraController(BaseController):
               QR-Code to be displayed to the QRTAN App
         """
         res = {}
-        description = 'ocra/request: request a challenge for a given user or token (serial). You must either provide a parameter "user" or a parameter "serial".'
+        description = ('ocra/request: request a challenge for a given user or'
+                       ' token (serial). You must either provide a parameter '
+                       '"user" or a parameter "serial".')
         dataobj = ""
 
         try:
-            param = getLowerParams(request.params)
+            param = getLowerParams(self.request_params)
             log.info("[request] saving default configuration: %r" % param)
 
             checkPolicyPre('ocra', "request")
 
-            serial = getParam(param, 'serial', optional)
-            user = getUserFromParam(param, optional)
+            serial = param.get('serial')
+            user = getUserFromParam(param)
+
+            if user.is_empty and serial is None:
 
-            if user.isEmpty() and serial is None:
-                ## raise exception
                 log.exception("[request] user or serial is required")
                 raise ParameterError("Usage: %s" % description, id=77)
 
-            message = getParam(param, 'data'  , optional)
+            if not serial:
+                if not user.exists():
+                    raise UserError("getUserId failed: no user >%s< found!"
+                                    % user.login, id=1205)
+
+            message = param.get('data')
             if message is None:
                 message = ''
 
-            ## ocra token
+            # ocra token
+
             tokens = getTokens4UserOrSerial(user, serial)
 
-            if len(tokens) > 1 :
-                error = ('More than one token found: unable to create challenge '
-                        'for (u:%r,s:%r)!' % (user, serial))
-                log.error(error)
+            if len(tokens) > 1:
+                error = ('More than one token found: unable to create '
+                         'challenge for (u:%r,s:%r)!' % (user, serial))
                 raise Exception(error)
 
             if len(tokens) == 0:
                 error = ('No token found: unable to create challenge for'
-                          ' (u:%r,s:%r)!' % (user, serial))
-                log.error(error)
+                         ' (u:%r,s:%r)!' % (user, serial))
                 raise Exception(error)
 
             ocra = tokens[0]
             (transId, challenge, res, url) = ocra.challenge(message)
 
-            u = urlencode({'u':str(url.encode("utf-8"))})
+            u = urlencode({'u': str(url.encode("utf-8"))})
 
-            uInfo = {'tr': transId, 'ch' : challenge,
-                      'me': str(message.encode("utf-8")), 'u': u[2:]}
-            detail = {"transactionid"   : transId,
-                      'challenge'       : challenge,
-                      'message'         : str(message.encode("utf-8")),
-                      'url'             : str(url.encode("utf-8")),
-                     }
+            uInfo = {'tr': transId,
+                     'ch': challenge,
+                     'me': str(message.encode("utf-8")),
+                     'u': u[2:]}
+
+            detail = {"transactionid": transId,
+                      'challenge': challenge,
+                      'message': str(message.encode("utf-8")),
+                      'url': str(url.encode("utf-8")), }
+
+            # create the app_url from the data
 
-            ## create the app_url from the data'''
             dataobj = 'lseqr://req?%s' % (str(urlencode(uInfo)))
 
-            ## append the signature to the url '''
-            signature = {'si' : ocra.signData(dataobj)}
+            # append the signature to the url
+
+            signature = {'si': ocra.signData(dataobj)}
             uInfo['si'] = signature
             dataobj = '%s&%s' % (dataobj, str(urlencode(signature)))
 
             detail["data"] = dataobj
 
             c.audit['success'] = res
-            #c.audit['info'] += "%s=%s, " % (k, value)
 
             Session.commit()
-            qr = getParam(param, 'qr', optional)
+            qr = param.get('qr')
             if qr is not None:
                 param['alt'] = detail
                 return sendQRImageResult(response, dataobj, param)
@@ -252,11 +265,9 @@ class OcraController(BaseController):
 
         finally:
             Session.close()
-            log.debug("[request] done")
 
+    # https://linotpserver/ocra/check_t?transactionid=TRANSACTIONID&pass=TAN
 
-
-    ## https://linotpserver/ocra/check_t?transactionid=TRANSACTIONID&pass=TAN
     def check_t(self):
         """
         method:
@@ -267,10 +278,12 @@ class OcraController(BaseController):
 
         arguments:
             * transactionid:  (required - string)
-                    Dies ist eine Transaktions-ID, die bei der Challenge ausgegeben wurde.
+                    Dies ist eine Transaktions-ID, die bei der Challenge
+                    ausgegeben wurde.
 
             * pass:   (required - string)
-                    die response, die der OCRA Token auf Grund der Challenge berechnet hat
+                    die response, die der OCRA Token auf Grund der Challenge
+                    berechnet hat
 
         returns:
 
@@ -296,24 +309,28 @@ class OcraController(BaseController):
         description = 'ocra/check_t: validate a token request.'
 
         try:
-            param = getLowerParams(request.params)
+            param = getLowerParams(self.request_params)
             log.info("[check_t] check OCRA token: %r" % param)
 
-            #checkPolicyPre('ocra', "check_t" )
+            # TODO: checkPolicyPre('ocra', "check_t")
 
-            passw = getParam(param, 'pass'  , optional)
+            passw = param.get('pass')
             if passw is None:
-                ## raise exception'''
+                # raise exception'''
                 log.exception("[check_t] missing pass ")
-                raise ParameterError("Usage: %s Missing parameter 'pass'." % description, id=77)
+                raise ParameterError("Usage: %s Missing parameter "
+                                     "'pass'." % description, id=77)
 
-            transid = getParam(param, 'transactionid', optional)
-            if transid is None:
-                ## raise exception'''
-                log.exception("[check_t] missing transactionid, user or serial number of token")
-                raise ParameterError("Usage: %s Missing parameter 'transactionid'." % description, id=77)
+            transid = param.get('transactionid')
+            if not transid:
+                # raise exception
+                log.exception("[check_t] missing transactionid, user or "
+                              "serial number of token")
+                raise ParameterError("Usage: %s Missing parameter "
+                                     "'transactionid'." % description, id=77)
+
+            # if we have a transaction, get serial from this challenge
 
-            ## if we have a transaction, get serial from this challenge
             value = {}
             ocraChallenge = OcraTokenClass.getTransaction(transid)
             if ocraChallenge is not None:
@@ -333,55 +350,64 @@ class OcraController(BaseController):
                 elif len(realms) > 0:
                     realm = realms[0]
 
-                userInfo = getUserInfo(tok.LinOtpUserid, tok.LinOtpIdResolver, tok.LinOtpIdResClass)
+                userInfo = getUserInfo(tok.LinOtpUserid,
+                                       tok.LinOtpIdResolver,
+                                       tok.LinOtpIdResClass)
+
                 user = User(login=userInfo.get('username'), realm=realm)
 
-                (ok, opt) = checkSerialPass(serial, passw, user=user,
-                                     options={'transactionid':transid})
+                vh = ValidationHandler()
+                (ok, _opt) = vh.checkSerialPass(
+                                       serial,
+                                       passw,
+                                       user=user,
+                                       options={'transactionid': transid})
 
                 failcount = theToken.getFailCount()
                 value['result'] = ok
                 value['failcount'] = int(failcount)
 
             else:
-                ## no challenge found for this transid
+
+                # no challenge found for this transid
+
                 value['result'] = False
-                value['failure'] = 'No challenge for transaction %r found'\
-                                    % transid
+                value['failure'] = ('No challenge for transaction %r found'
+                                    % transid)
 
             c.audit['success'] = res
-            #c.audit['info'] += "%s=%s, " % (k, value)
 
             Session.commit()
             return sendResult(response, value, 1)
 
-        except Exception as e :
-            log.exception("[check_t] failed: %r" % e)
+        except Exception as exx:
+            log.exception("[check_t] failed: %r", exx)
             Session.rollback()
-            return sendResult(response, unicode(e), 0)
+            return sendResult(response, unicode(exx), 0)
 
         finally:
             Session.close()
-            log.debug("[check_t] done")
 
+    #
+    #    https://linotpserver/ocra/checkstatus?transactionid=TRANSACTIONID
+    #    https://linotpserver/ocra/checkstatus?serial=SERIENNUMMER
+    #    https://linotpserver/ocra/checkstatus?user=BENUTZER
+    #
 
-    '''
-        https://linotpserver/ocra/checkstatus?transactionid=TRANSACTIONID
-        https://linotpserver/ocra/checkstatus?serial=SERIENNUMMER
-        https://linotpserver/ocra/checkstatus?user=BENUTZER
-    '''
     def checkstatus(self):
         """
         method:
             ocra/checkstatus
 
         description:
-            Methode zur assynchronen Ueberpruefungen eines Challenge Response Valiadation requests
+            Methode zur assynchronen Ueberpruefungen eines Challenge
+            Response Valiadation requests
 
         arguments:
 
             * transactionid:  (required one of  - string - (hex))
-                    Dies ist eine Transaktions-ID, die bei der Challenge ausgegeben wurde.
+                    Dies ist eine Transaktions-ID, die bei der Challenge
+                    ausgegeben wurde.
 
             * serial: (required one of  - string)
                     die Serien Nummer des OCRA Token
@@ -431,21 +457,23 @@ class OcraController(BaseController):
 
         """
         res = {}
-        description = 'ocra/checkstatus: check the token status - for assynchronous verification. Missing parameter: You need to provide one of the parameters "transactionid", "user" or "serial"'
+        description = ('ocra/checkstatus: check the token status - '
+                       'for assynchronous verification. Missing parameter: '
+                       'You need to provide one of the parameters '
+                       '"transactionid", "user" or "serial"')
 
         try:
-            param = getLowerParams(request.params)
+            param = getLowerParams(self.request_params)
             log.debug("[checkstatus] check OCRA token status: %r" % param)
 
             checkPolicyPre('ocra', "status")
 
-            transid = getParam(param, 'transactionid'   , optional)
-            user = getUserFromParam(param, optional)
-            #user   = getParam(param, 'user'          ,optional)
-            serial = getParam(param, 'serial'          , optional)
+            transid = param.get('transactionid')
+            user = getUserFromParam(param)
+            serial = param.get('serial')
 
-            if transid is None and user.isEmpty() and serial is None:
-                ## raise exception
+            if transid is None and user.is_empty and serial is None:
+                # raise exception
                 log.exception("[ocra/checkstatus] : missing transactionid, user or serial number for token")
                 raise ParameterError("Usage: %s" % description, id=77)
 
@@ -456,8 +484,9 @@ class OcraController(BaseController):
             if serial is not None:
                 serials.add(serial)
 
-            ## if we have a transaction, get serial from this challenge
-            if transid is not None :
+            # if we have a transaction, get serial from this challenge
+
+            if transid is not None:
                 ocraChallenge = None
                 try:
                     ocraChallenge = OcraTokenClass.getTransaction(transid)
@@ -466,13 +495,15 @@ class OcraController(BaseController):
                 if ocraChallenge is not None:
                     serials.add(ocraChallenge.tokenserial)
 
-            ## if we have a serial number of  token
+            # if we have a serial number of  token
+
             if len(serials) > 0:
                 for serial in serials:
                     tokens.extend(getTokens4UserOrSerial(serial=serial))
 
-            ## if we have a user
-            if user.isEmpty() == False:
+            # if we have a user
+
+            if not user.is_empty:
                 try:
                     tokens.extend(getTokens4UserOrSerial(user=user))
                 except:
@@ -483,9 +514,11 @@ class OcraController(BaseController):
                     challenges = []
                     if transid is None:
                         serial = token.getSerial()
-                        challenges = OcraTokenClass.getTransactions4serial(serial)
+                        challenges = OcraTokenClass.getTransactions4serial(
+                                                                        serial)
                     else:
-                        challenges.append(OcraTokenClass.getTransaction(transid))
+                        challenges.append(OcraTokenClass.getTransaction(
+                                                                    transid))
 
                     for challenge in challenges:
                         stat = token.getStatus(challenge.transid)
@@ -510,8 +543,6 @@ class OcraController(BaseController):
 
         finally:
             Session.close()
-            log.debug('[ocra/checkstatus] done')
-
 
     def getActivationCode(self):
         '''
@@ -528,20 +559,21 @@ class OcraController(BaseController):
             JSON with     "activationcode": "JZXW4ZI=2A"
         '''
 
-        from linotp.lib.crypt import createActivationCode
+        from linotp.lib.crypto import createActivationCode
 
         res = {}
-        #description = 'ocra/getActivationCode'
+
+        # description = 'ocra/getActivationCode'
 
         try:
-            params = getLowerParams(request.params)
+            params = getLowerParams(self.request_params)
             log.debug("[getActivationCode]: %r" % params)
 
             checkPolicyPre('ocra', "activationcode")
 
             ac = str(params.get('activationcode'))
             activationCode = createActivationCode(acode=ac)
-            res = {'activationcode':activationCode}
+            res = {'activationcode': activationCode}
 
             Session.commit()
             return sendResult(response, res, 1)
@@ -558,25 +590,25 @@ class OcraController(BaseController):
 
         finally:
             Session.close()
-            log.debug('[getActivationCode] done')
-
 
     def calculateOtp(self):
         '''
 
         '''
-        from linotp.lib.crypt import kdf2
-        from linotp.lib.ocra import OcraSuite
+        from linotp.lib.crypto import kdf2
+        from linotp.tokens.ocra import OcraSuite
         from datetime import datetime
 
         from urlparse import urlparse
         from urlparse import parse_qs
 
         res = {}
-        #description = 'ocra/calculateOtp: calculate the first otp from the given init2 response '
+
+        # description = 'ocra/calculateOtp: calculate the first
+        # otp from the given init2 response '
 
         try:
-            params = getLowerParams(request.params)
+            params = getLowerParams(self.request_params)
             log.debug("[calculateOtp]: %r" % params)
 
             checkPolicyPre('ocra', "calcOTP")
@@ -591,7 +623,6 @@ class OcraController(BaseController):
 
             nonce3 = params.get('no')
             ocrasuite3 = params.get('os')
-            #serial3         = params.get('se')
 
             challenge = params.get('challenge')
             counter = params.get('counter')
@@ -599,9 +630,10 @@ class OcraController(BaseController):
             init1 = params.get('init1')
             init2 = params.get('init2')
 
-            ## parse init1 '''
             if init1 is not None:
-                ## now parse the appurl for the ocrasuite '''
+
+                # now parse the appurl for the ocrasuite
+
                 uri = urlparse(init1.replace('lseqr://', 'http://'))
                 qs = uri.query
                 qdict = parse_qs(qs)
@@ -620,9 +652,12 @@ class OcraController(BaseController):
                 if sharedsecret is None:
                     sharedsecret = sharedsecret2
 
-            ## parse init1
+            # parse init1
+
             if init2 is not None:
-                ## now parse the appurl for the ocrasuite
+
+                # now parse the appurl for the ocrasuite
+
                 uri = urlparse(init2.replace('lseqr://', 'http://'))
                 qs = uri.query
                 qdict = parse_qs(qs)
@@ -650,8 +685,9 @@ class OcraController(BaseController):
             if ocrasuite3 is not None:
                 ocrasuite = unicode(ocrasuite3)
 
-            ##  now we have all in place for the key derivation to create the new key
-            ##     sharedsecret, activationcode and nonce
+            #  now we have all in place for the key derivation to create
+            # the new key sharedsecret, activationcode and nonce
+
             key_len = 20
             if ocrasuite.find('-SHA256'):
                 key_len = 32
@@ -666,7 +702,6 @@ class OcraController(BaseController):
                 activationcode = unicode(activationcode)
 
             newkey = kdf2(sharedsecret, nonce, activationcode, len=key_len)
-            ## hnewkey = binascii.hexlify(newkey)
             ocra = OcraSuite(ocrasuite)
 
             param = {}
@@ -674,9 +709,13 @@ class OcraController(BaseController):
             param['Q'] = unicode(challenge)
             param['P'] = unicode(ocrapin)
             param['S'] = ''
+
             if ocra.T is not None:
-                ## Default value for G is 1M, i.e., time-step size is one minute and the
-                ##  T represents the number of minutes since epoch time [UT].
+
+                # Default value for G is 1M, i.e., time-step size is
+                # one minute and the T represents the number of minutes
+                # since epoch time [UT].
+
                 now = datetime.now()
                 stime = now.strftime("%s")
                 itime = int(stime)
@@ -685,7 +724,7 @@ class OcraController(BaseController):
             data = ocra.combineData(**param)
             otp = ocra.compute(data, newkey)
 
-            res = {'otp':otp}
+            res = {'otp': otp}
 
             Session.commit()
             return sendResult(response, res, 1)
@@ -702,8 +741,6 @@ class OcraController(BaseController):
 
         finally:
             Session.close()
-            log.debug('[ocra/calculateOtp] done')
 
 
-#eof###########################################################################
-
+# eof
