@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 #
 #    LinOTP - the open source solution for two factor authentication
-#    Copyright (C) 2010 - 2016 LSE Leading Security Experts GmbH
+#    Copyright (C) 2010 - 2019 KeyIdentity GmbH
 #
 #    This file is part of LinOTP server.
 #
@@ -19,24 +19,22 @@
 #    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #
 #
-#    E-mail: linotp@lsexperts.de
+#    E-mail: linotp@keyidentity.com
 #    Contact: www.linotp.org
-#    Support: www.lsexperts.de
+#    Support: www.keyidentity.com
 #
 """
 openid controller - This is the controller for the openid service
 """
 
-ASSOC_EXPIRES_IN = 3600
-COOKIE_NAME = "linotp_openid"
-
-
 import logging
-
 import webob
-import traceback
+from urllib import urlencode
 
+import linotp.model
 
+from linotp.lib.base import BaseController
+from linotp.lib.auth.validate import ValidationHandler
 
 from pylons import tmpl_context as c
 from pylons import request, response
@@ -44,26 +42,20 @@ from pylons.controllers.util import redirect
 from pylons import config
 from pylons import url as url
 
-from linotp.lib.base import BaseController
+from linotp.lib.error import ParameterError
 
-from linotp.lib.util import  getParam
 from linotp.lib.util import get_client
-from linotp.lib.util import  is_valid_fqdn
-from linotp.lib.user import  getUserFromParam
-from linotp.lib.realm import  getDefaultRealm
-from linotp.lib.realm import getRealms
-from linotp.lib.user     import getRealmBox
-from linotp.lib.util    import get_version
-from linotp.lib.util    import get_copyright_info
+from linotp.lib.util import is_valid_fqdn
+from linotp.lib.user import getUserFromParam
+from linotp.lib.realm import getDefaultRealm
+from linotp.lib.util import get_version
+from linotp.lib.util import get_copyright_info
 
 from linotp.lib.policy import PolicyException
 from pylons.templating import render_mako as render
 from webob.exc import HTTPBadRequest
 
-
-from linotp.lib.token import checkUserPass
 from linotp.lib.reply import sendError
-from linotp.model.meta import Session
 
 from linotp.lib.openid import IdResMessage
 from linotp.lib.openid import create_association, check_authentication
@@ -71,15 +63,18 @@ from linotp.lib.openid import create_association, check_authentication
 from linotp.lib.openid import OPENID_2_0_TYPE
 from linotp.lib.openid import OPENID_1_0_TYPE
 
-from urllib import urlencode
+from linotp.lib.context import request_context
+
+Session = linotp.model.Session
+
+ASSOC_EXPIRES_IN = 3600
+COOKIE_NAME = "linotp_openid"
 
 audit = config.get('audit')
 
-optional = True
-required = False
-
 log = logging.getLogger(__name__)
 
+
 class OpenidController(BaseController):
 
     '''
@@ -93,15 +88,12 @@ class OpenidController(BaseController):
 
     def __before__(self, action, **params):
 
-        log.debug("[__before__::%r]" % (action))
         valid_request = False
         try:
-            for key, value in params.items():
-                log.debug("[__before__::%r:%r]" % (key, value))
 
-
-            audit.initialize()
-            c.audit['client'] = get_client()
+            c.audit = request_context[audit]
+            c.audit['client'] = get_client(request)
+            request_context['Audit'] = audit
 
             self.storage = config.get('openid_sql')
 
@@ -117,7 +109,8 @@ class OpenidController(BaseController):
             c.licenseinfo = get_copyright_info()
 
             http_host = request.environ.get("HTTP_HOST")
-            log.debug("[__before__] Doing openid request from host %s" % http_host)
+            log.debug("[__before__] Doing openid request from host %s",
+                       http_host)
             if not is_valid_fqdn(http_host, split_port=True):
                 err = "Bad hostname: %s" % http_host
                 audit.log(c.audit)
@@ -133,9 +126,9 @@ class OpenidController(BaseController):
             if login:
                 c.logged_in = True
 
-            ## default return for the __before__ and __after__
-            log.debug("[__before__::%r]" % (response))
+            # default return for the __before__ and __after__
             valid_request = True
+
             return response
 
         except PolicyException as pex:
@@ -156,8 +149,6 @@ class OpenidController(BaseController):
                 self.storage.session.rollback()
                 self.storage.session.close()
 
-            log.debug("[__before__::%r] done" % (action))
-
 
     def __after__(self):
         try:
@@ -172,7 +163,6 @@ class OpenidController(BaseController):
             return sendError(response, exx, context='after')
 
         finally:
-            log.debug("[__after__] done")
             self.storage.session.close()
 
 
@@ -243,15 +233,7 @@ class OpenidController(BaseController):
         and returns to the given "openid." either directly or after authenticating the user
         openid.claimed_id.
         '''
-
-        req_method = request.environ.get('REQUEST_METHOD')
-
-        if "POST" == req_method:
-            params = request.environ.get('webob._parsed_post_vars')
-            params = params[0]
-        else:
-            # we got a GET request
-            params = request.params
+        params = self.request_params
 
         # distpatching the request depending on the mode
         if 'openid.mode' not in params:
@@ -349,10 +331,14 @@ class OpenidController(BaseController):
         '''
         This is called when the user accepts - hit the submit button - that he will login to the consumer
         '''
-        param = request.params
-        log.debug("[checkid_submit] params: %s" % param)
-        redirect_token = getParam(param, "redirect_token", required)
-        verify_always = getParam(param, "verify_always", optional)
+        log.debug("[checkid_submit] params: %s" % self.request_params)
+
+        try:
+            redirect_token = self.request_params["redirect_token"]
+        except KeyError:
+            raise ParameterError("Missing parameter: 'redirect_token'", id=905)
+
+        verify_always = self.request_params.get("verify_always")
         r_url, site, handle = self.storage.get_redirect(redirect_token)
         self.storage.add_site(site, handle)
 
@@ -373,11 +359,9 @@ class OpenidController(BaseController):
         redirect(r_url)
 
     def check_authentication(self, params):
-        #log.debug("[check_authentication] entering function")
         res = check_authentication(**params)
         response.status = 200
         response.content_type = "text/plain"
-        #log.debug("[check_authentication] %s" % res)
         return res
 
 
@@ -411,19 +395,14 @@ class OpenidController(BaseController):
 
         response.delete_cookie(COOKIE_NAME)
 
-        params = {}
-        params.update(request.params)
-        p = {}
-
         ## are we are called during an openid auth request?
-        if "openid.return_to" in params:
+        if "openid.return_to" in self.request_params:
             redirect_to = "/openid/login"
-            p.update(params)
-            do_redirect = url(str("%s?%s" % (redirect_to, urlencode(p))))
+            do_redirect = url(str("%s?%s" % (redirect_to, urlencode(self.request_params))))
 
         else:
             redirect_to = "/openid/status"
-            do_redirect = url(str("%s?%s" % (redirect_to, urlencode(p))))
+            do_redirect = url(redirect_to)
 
         redirect(do_redirect)
 
@@ -431,7 +410,7 @@ class OpenidController(BaseController):
         '''
         This is the redirect of the first template
         '''
-        param = request.params
+        param = self.request_params
 
         c.defaultRealm = getDefaultRealm()
         c.p = {}
@@ -464,15 +443,12 @@ class OpenidController(BaseController):
         '''
         This shows the login status.
         '''
-        param = {}
-        param.update(request.params)
-
         cookie = request.cookies.get(COOKIE_NAME)
         if cookie is not None:
             c.login, token = cookie.split(":")
 
-        if "message" in param:
-            c.message = param.get("message")
+        if "message" in self.request_params:
+            c.message = self.request_params.get("message")
 
         return render("/openid/status.mako")
 
@@ -493,20 +469,18 @@ class OpenidController(BaseController):
             JSON response
         '''
         ok = False
-        param = {}
+        param = self.request_params
         do_redirect = None
         message = None
 
         try:
-            param.update(request.params)
-
             same_user = True
-            passw = getParam(param, "pass", optional)
+            passw = param.get("pass")
 
             ## getUserFromParam will return default realm if no realm is
             ## provided via @ append or extra parameter realm
             ## if the provided realm does not exist, the realm is left empty
-            user = getUserFromParam(param, optional)
+            user = getUserFromParam(param)
 
             ## if the requested user has a realm specified (via @realm append)
             ## and this is not the same as the user from getUserFromParam
@@ -519,8 +493,9 @@ class OpenidController(BaseController):
             c.audit['user'] = user.login
             c.audit['realm'] = user.realm or getDefaultRealm()
 
+            vh = ValidationHandler()
             if same_user is True:
-                (ok, opt) = checkUserPass(user, passw)
+                (ok, opt) = vh.checkUserPass(user, passw)
 
             c.audit['success'] = ok
 
@@ -543,9 +518,8 @@ class OpenidController(BaseController):
             # status page
 
             p = {}
-            redirect_to = getParam(param, "redirect_to", optional)
+            redirect_to = param.get("redirect_to")
             if redirect_to and ok:
-                p = {}
                 for k in  [ 'openid.return_to', "openid.realm", "openid.ns", "openid.claimed_id", "openid.mode",
                             "openid.identity" ]:
                     p[k] = param[k]
@@ -578,7 +552,6 @@ class OpenidController(BaseController):
         log.debug("[compare_users] %s == %s?" % (u1, u2))
         return u1 == u2
 
-
     def _split_cookie(self):
         login = ""
         token = ""
@@ -587,4 +560,12 @@ class OpenidController(BaseController):
             login, token = cookie.split(":")
         return login, token
 
-#eof###########################################################################
+    def custom_style(self):
+        '''
+        If this action was called, the user hasn't created a custom css yet. To avoid hitting
+        the debug console over and over, we serve an empty file.
+        '''
+        response.headers['Content-type'] = 'text/css'
+        return ''
+
+# eof #
