@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 #
 #    LinOTP - the open source solution for two factor authentication
-#    Copyright (C) 2010 - 2016 LSE Leading Security Experts GmbH
+#    Copyright (C) 2010 - 2019 KeyIdentity GmbH
 #
 #    This file is part of LinOTP server.
 #
@@ -19,9 +19,9 @@
 #    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #
 #
-#    E-mail: linotp@lsexperts.de
+#    E-mail: linotp@keyidentity.com
 #    Contact: www.linotp.org
-#    Support: www.lsexperts.de
+#    Support: www.keyidentity.com
 #
 """
 selfservice controller - This is the controller for the self service interface,
@@ -33,119 +33,70 @@ selfservice controller - This is the controller for the self service interface,
 
 """
 import os
-
-try:
-    import json
-except ImportError:
-    import simplejson as json
-
+import json
 import webob
-import base64
 
-from pylons import request, response, config, tmpl_context as c
+from paste.httpexceptions import HTTPFound
+
+from pylons import request
+from pylons import response
+from pylons import config
+from pylons import tmpl_context as c
+from pylons import url
+
 from pylons.controllers.util import abort
+from pylons.controllers.util import redirect
+from pylons.templating import render_mako as render
+from pylons.i18n.translation import _
+
 from mako.exceptions import CompileException
 
+import linotp.model
 from linotp.lib.base import BaseController
-from pylons.templating import render_mako as render
-from linotp.lib.token import genSerial
-
-
-from linotp.lib.token import enableToken, assignToken, unassignToken
-from linotp.lib.token import initToken, removeToken
-from linotp.lib.token import setPin
-from linotp.lib.token import resyncToken, resetToken, setPinUser
-from linotp.lib.token import TokenIterator, isTokenOwner
-from linotp.lib.token import hasOwner, getTokenType
-from linotp.lib.token import getTokenRealms
-from linotp.lib.token import get_multi_otp
-from linotp.lib.token import get_serial_by_otp
-
-from linotp.lib.token import get_tokenserial_of_transaction
+from linotp.lib.error import ParameterError
 
+from linotp.lib.token import getTokenType
 from linotp.lib.token import getTokens4UserOrSerial
 
-from linotp.lib.token import checkSerialPass
-from linotp.lib.tokenclass import OcraTokenClass
-from linotp.lib.audit.base import search as audit_search
+from linotp.lib.policy import getSelfserviceActions
+from linotp.lib.policy import _get_auth_PinPolicy
 
-from linotp.lib.policy import checkOTPPINPolicy, getRandomOTPPINLength
-from linotp.lib.policy import getRandomPin, getPolicy
-
-from linotp.lib.policy import getSelfserviceActions, checkPolicyPre
-from linotp.lib.policy import PolicyException, getOTPPINEncrypt
-
-from linotp.lib.util import getParam
-from linotp.lib.util import getLowerParams
-from linotp.lib.util import check_selfservice_session
-from linotp.lib.util import generate_otpkey
 from linotp.lib.util import remove_empty_lines
 
-from linotp.lib.reply import sendResult, sendError
+from linotp.lib.reply import sendError
 
-from linotp.lib.audit.base import logTokenNum
-from linotp.lib.util    import get_version
-from linotp.lib.util    import get_copyright_info
-from linotp.lib.util import get_client
+from linotp.lib.realm import getRealms
 from linotp.lib.realm import getDefaultRealm
 
-from linotp.model.meta import Session
+from linotp.lib.user import getRealmBox
 
-from linotp.lib.reply import sendQRImageResult
-from linotp.lib.reply import create_img
+from linotp.lib.util import get_version
+from linotp.lib.util import get_copyright_info
+from linotp.lib.util import get_client
 
-from linotp.lib.userservice import (add_dynamic_selfservice_enrollment,
-                                    add_dynamic_selfservice_policies
-                                    )
 
+from linotp.lib.userservice import add_dynamic_selfservice_enrollment
+from linotp.lib.userservice import add_dynamic_selfservice_policies
+from linotp.lib.userservice import get_pre_context
+from linotp.lib.userservice import remove_auth_cookie
+from linotp.lib.userservice import check_session
 
 from linotp.lib.selfservice import get_imprint
-from linotp.lib.user import getUserInfo, User
-
-from linotp.lib.error import SelfserviceException
-
-import traceback
-# import datetime, random
-import copy
 
 from linotp.lib.selftest import isSelfTest
 from linotp.controllers.userservice import get_auth_user
+from linotp.controllers.userservice import getTokenForUser
 
-from linotp.lib.token import newToken
-
-from pylons.i18n.translation import _
-
+from linotp.tokens import tokenclass_registry
+from linotp.lib.context import request_context
 
 import logging
-log = logging.getLogger(__name__)
 
-audit = config.get('audit')
+Session = linotp.model.Session
 
 ENCODING = "utf-8"
-
-optional = True
-required = False
-
-def getTokenForUser(user):
-    """
-    should be moved to token.py
-    """
-    tokenArray = []
-
-    log.debug("[getTokenForUser] iterating tokens for user...")
-    log.debug("[getTokenForUser] ...user %s in realm %s." %
-              (user.login, user.realm))
-    tokens = getTokens4UserOrSerial(user=user, serial=None, _class=False)
-
-    for token in tokens:
-        tok = token.get_vars()
-        if tok.get('LinOtp.TokenInfo', None):
-            token_info = json.loads(tok.get('LinOtp.TokenInfo'))
-            tok['LinOtp.TokenInfo'] = token_info
-        tokenArray.append(tok)
-
-    log.debug("[getTokenForUser] found tokenarray: %r" % tokenArray)
-    return tokenArray
+log = logging.getLogger(__name__)
+audit = config.get('audit')
 
 
 class SelfserviceController(BaseController):
@@ -155,24 +106,25 @@ class SelfserviceController(BaseController):
     # the following actions don't require a session parameter
     # as they are only callbacks to render a form
     form_access_methods = [
-            "activateqrtoken",
-            "assign",
-            "custom_style",
-            "delete",
-            "disable",
-            "enable",
-            "getotp",
-            "history",
-            "index",
-            "load_form",
-            "reset",
-            "resync",
-            "setmpin",
-            "setpin",
-            "unassign",
-            "webprovisiongoogletoken",
-            "webprovisionoathtoken"
-            ]
+        "activateocratoken",
+        "assign",
+        "custom_style",
+        "delete",
+        "disable",
+        "enable",
+        "getotp",
+        "history",
+        "index",
+        "landing",
+        "load_form",
+        "reset",
+        "resync",
+        "setmpin",
+        "setpin",
+        "unassign",
+        "webprovisiongoogletoken",
+        "webprovisionoathtoken"
+    ]
 
     def __before__(self, action):
         '''
@@ -181,34 +133,101 @@ class SelfserviceController(BaseController):
         _before_ is executed before any other function in this controller.
         '''
 
-        try:
-            audit.initialize()
-            c.audit['success'] = False
-            c.audit['client'] = get_client()
+        self.redirect = None
 
+        try:
             c.version = get_version()
             c.licenseinfo = get_copyright_info()
 
-            (auth_type, auth_user) = get_auth_user(request)
+            c.audit = request_context['audit']
+            c.audit['success'] = False
+            self.client = get_client(request)
+            c.audit['client'] = self.client
 
-            if not auth_user or auth_type not in ['repoze', 'selftest']:
-                abort(401, "You are not authenticated")
+            request_context['Audit'] = audit
 
-            (c.user, _foo, c.realm) = auth_user.rpartition('@')
-            self.authUser = User(c.user, c.realm, '')
+            # -------------------------------------------------------------- --
+
+            # handle requests which dont require authetication
+
+            if action in ['logout', 'custom_style']:
+                return
+
+            # -------------------------------------------------------------- --
+
+            # get the authenticated user
+
+            auth_type, auth_user, auth_state = get_auth_user(request)
+
+            # -------------------------------------------------------------- --
+
+            # handle not authenticated requests
+
+            if not auth_user or auth_type not in ['user_selfservice']:
+
+                if action in ['login']:
+                    return
+
+                if action in ['index']:
+                    self.redirect = True
+                    redirect(url(controller='selfservice', action='login'))
+
+                else:
+                    abort(403, "No valid session")
+
+            # -------------------------------------------------------------- --
+
+            # handle authenticated requests
+
+            # there is only one special case, which is the login that
+            # could be forwarded to the index page
+
+            if action in ['login']:
+                if auth_state != 'authenticated':
+                    return
+
+                self.redirect = True
+                redirect(url(controller='selfservice', action='index'))
+
+            # -------------------------------------------------------------- --
+
+            # in case of user_selfservice, an unauthenticated request should always go to login
+            if auth_user and auth_type is 'user_selfservice' \
+                    and auth_state is not 'authenticated':
+                self.redirect = True
+                redirect(url(controller='selfservice', action='login'))
+
+
+            # futher processing with the authenticated user
+
+            if auth_state != 'authenticated':
+                abort(403, "No valid session")
+
+            c.user = auth_user.login
+            c.realm = auth_user.realm
+            self.authUser = auth_user
+
+            # -------------------------------------------------------------- --
+
+            # authenticated session verification
+
+            if auth_type == 'user_selfservice':
 
-            if auth_type == "repoze":
                 # checking the session only for not_form_access actions
                 if action not in self.form_access_methods:
-                    call_url = "selfservice/%s" % action
-                    valid_session = check_selfservice_session(url=call_url,
-                                                    cookies=request.cookies,
-                                                    params=request.params)
+
+                    valid_session = check_session(request,
+                                                  auth_user,
+                                                  self.client)
+
                     if not valid_session:
                         c.audit['action'] = request.path[1:]
                         c.audit['info'] = "session expired"
                         audit.log(c.audit)
-                        abort(401, "No valid session")
+
+                        abort(403, "No valid session")
+
+            # -------------------------------------------------------------- --
 
             c.imprint = get_imprint(c.realm)
 
@@ -216,10 +235,9 @@ class SelfserviceController(BaseController):
 
             c.user = self.authUser.login
             c.realm = self.authUser.realm
-            c.tokenArray = getTokenForUser(self.authUser)
 
             # only the defined actions should be displayed
-            # - remark: the generic actions like enrollTT are already approved
+            # - remark: the generic actions like enrollTT are allready approved
             #   to have a rendering section and included
             actions = getSelfserviceActions(self.authUser)
             c.actions = actions
@@ -234,54 +252,60 @@ class SelfserviceController(BaseController):
                         # w.r.t. javascript evaluation
                         try:
                             nval = int(val)
-                        except:
+                        except ValueError:
                             nval = val
                         c.__setattr__(name.strip(), nval)
 
-            c.dynamic_actions = add_dynamic_selfservice_enrollment(config, c.actions)
+            c.dynamic_actions = add_dynamic_selfservice_enrollment(config,
+                                                                   c.actions)
 
-            # we require to establish all token local defined policies to be initialiezd
-            additional_policies = add_dynamic_selfservice_policies(config, actions)
+            # we require to establish all token local defined
+            # policies to be initialiezd
+            additional_policies = add_dynamic_selfservice_policies(config,
+                                                                   actions)
             for policy in additional_policies:
                 c.__setattr__(policy, -1)
 
             c.otplen = -1
             c.totp_len = -1
 
+            c.pin_policy = _get_auth_PinPolicy(user=self.authUser)
+
             return response
 
-        except webob.exc.HTTPUnauthorized as acc:
+        except (webob.exc.HTTPUnauthorized, webob.exc.HTTPForbidden) as acc:
             # the exception, when an abort() is called if forwarded
             log.info("[__before__::%r] webob.exception %r" % (action, acc))
-            log.info("[__before__] %s" % traceback.format_exc())
             Session.rollback()
             Session.close()
             raise acc
 
+        except HTTPFound as exx:
+            raise exx
+
         except Exception as e:
             log.exception("[__before__] failed with error: %r" % e)
             Session.rollback()
             Session.close()
             return sendError(response, e, context='before')
 
-        finally:
-            log.debug('[__after__] done')
-
-
-
-    def __after__(self, action, **params):
+    def __after__(self, action,):
         '''
 
         '''
-        param = request.params
+        if self.redirect:
+            return
+
+        param = self.request_params
 
         try:
             if c.audit['action'] in ['selfservice/index']:
                 if isSelfTest():
                     log.debug("[__after__] Doing selftest!")
-                    suser = getParam(param, "selftest_user", True)
-                    if suser is not None:
-                        (c.user, _foo, c.realm) = getParam(param, "selftest_user", True).rpartition('@')
+
+                    if "selftest_user" in param:
+                        (c.user, _foo, c.realm) = param[
+                            "selftest_user"].rpartition('@')
                     else:
                         c.realm = ""
                         c.user = "--ua--"
@@ -290,13 +314,14 @@ class SelfserviceController(BaseController):
                         if uuser is not None:
                             (c.user, _foo, c.realm) = uuser.rpartition('@')
 
-                log.debug("[__after__] authenticating as %s in realm %s!" % (c.user, c.realm))
+                log.debug("[__after__] authenticating as %s in realm %s!"
+                          % (c.user, c.realm))
 
                 c.audit['user'] = c.user
                 c.audit['realm'] = c.realm
                 c.audit['success'] = True
 
-                if param.has_key('serial'):
+                if 'serial' in param:
                     c.audit['serial'] = param['serial']
                     c.audit['token_type'] = getTokenType(param['serial'])
 
@@ -317,16 +342,73 @@ class SelfserviceController(BaseController):
             Session.close()
             return sendError(response, e, context='after')
 
-        finally:
-            log.debug('[__after__] done')
-
     def index(self):
         '''
         This is the redirect to the first template
         '''
-        c.title = "LinOTP Self Service"
-        ren = render('/selfservice/base.mako')
-        return ren
+
+        c.title = _("LinOTP Self Service")
+        return render('selfservice/base.mako')
+
+    def logout(self):
+        """
+        handle the logout
+
+        we delete the cookies from the server and the client and
+        redirect to the login page
+        """
+
+        cookie = request.cookies.get('user_selfservice')
+        if cookie:
+            remove_auth_cookie(cookie)
+            response.delete_cookie('user_selfservice')
+
+        self.redirect = True
+        redirect(url(controller='selfservice', action='login'))
+
+    def login(self):
+        '''
+        render the selfservice login page
+        '''
+
+        cookie = request.cookies.get('user_selfservice')
+        if cookie:
+            remove_auth_cookie(cookie)
+            response.delete_cookie('user_selfservice')
+
+        c.title = _("LinOTP Self Service Login")
+
+        # ------------------------------------------------------------------ --
+
+        # prepare the realms and put the default realm on the top
+
+        defaultRealm = getDefaultRealm()
+        realmArray = [defaultRealm]
+
+        for realm in getRealms():
+            if realm != defaultRealm:
+                realmArray.append(realm)
+
+        # ------------------------------------------------------------------ --
+
+        # prepare the global context c for the rendering context
+
+        c.defaultRealm = defaultRealm
+        c.realmArray = realmArray
+
+        c.realmbox = getRealmBox()
+
+        context = get_pre_context(c.audit['client'])
+
+        mfa_login = context['mfa_login']
+        mfa_3_fields = context['mfa_3_fields']
+
+        c.otp = False
+        c.mfa_3_fields = False
+        if mfa_login and mfa_3_fields:
+            c.mfa_3_fields = True
+
+        return render('/selfservice/login.mako')
 
     def load_form(self):
         '''
@@ -340,13 +422,13 @@ class SelfserviceController(BaseController):
         :return: rendered html of the requested token
         '''
         res = ''
-        param = {}
 
         try:
+            try:
+                act = self.request_params["type"]
+            except KeyError:
+                raise ParameterError("Missing parameter: 'type'", id=905)
 
-            param.update(request.params)
-
-            act = getParam(param, "type", required)
             try:
                 (tok, section, scope) = act.split('.')
             except Exception:
@@ -355,12 +437,8 @@ class SelfserviceController(BaseController):
             if section != 'selfservice':
                 return res
 
-            g = config['pylons.app_globals']
-            tokenclasses = copy.deepcopy(g.tokenclasses)
-
-            if tok in tokenclasses:
-                tclass = tokenclasses.get(tok)
-                tclt = newToken(tclass)
+            if tok in tokenclass_registry:
+                tclt = tokenclass_registry.get(tok)
                 if hasattr(tclt, 'getClassInfo'):
                     sections = tclt.getClassInfo(section, {})
                     if scope in sections.keys():
@@ -376,22 +454,20 @@ class SelfserviceController(BaseController):
             return res
 
         except CompileException as exx:
-            log.exception("[load_form] compile error while processing %r.%r:" %
-                                                                (tok, scope))
-            log.error("[load_form] %r" % exx)
+            log.exception("[load_form] compile error while processing %r.%r:"
+                          "Exeption was %r" % (tok, scope, exx))
             Session.rollback()
-            raise Exception(exx)
+            raise exx
 
         except Exception as exx:
             Session.rollback()
             error = ('error (%r) accessing form data for: tok:%r, scope:%r'
-                                ', section:%r' % (exx, tok, scope, section))
+                     ', section:%r' % (exx, tok, scope, section))
             log.exception(error)
             return '<pre>%s</pre>' % error
 
         finally:
             Session.close()
-            log.debug('[load_form] done')
 
     def custom_style(self):
         '''
@@ -477,17 +553,24 @@ class SelfserviceController(BaseController):
         '''
         return render('/selfservice/history.mako')
 
+    def landing(self):
+        '''
+        This is the landing page for selfservice
+        '''
+        c.tokenArray = getTokenForUser(self.authUser)
+        return render('/selfservice/landing.mako')
+
     def webprovisionoathtoken(self):
         '''
         This is the form for an oathtoken to do web provisioning.
         '''
         return render('/selfservice/webprovisionoath.mako')
 
-    def activateqrtoken(self):
+    def activateocratoken(self):
         '''
-        return the form for an qr token activation
+        return the form for an ocra token activation
         '''
-        return render('/selfservice/activateqr.mako')
+        return render('/selfservice/activateocra.mako')
 
     def webprovisiongoogletoken(self):
         '''
@@ -498,23 +581,17 @@ class SelfserviceController(BaseController):
             return render('/selfservice/webprovisiongoogle.mako')
 
         except Exception as exx:
-            log.exception("[webprovisiongoogletoken] failed with error: %r" % exx)
+            log.exception(
+                "[webprovisiongoogletoken] failed with error: %r" % exx)
             return sendError(response, exx)
 
-        finally:
-            log.debug('[webprovisiongoogletoken] done')
-
-
-
     def usertokenlist(self):
         '''
         This returns a tokenlist as html output
         '''
+        c.tokenArray = getTokenForUser(self.authUser)
         res = render('/selfservice/tokenlist.mako')
         return res
 
 
-
-
-#eof##########################################################################
-
+# eof #
