@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 #
 #    LinOTP - the open source solution for two factor authentication
-#    Copyright (C) 2010 - 2016 LSE Leading Security Experts GmbH
+#    Copyright (C) 2010 - 2019 KeyIdentity GmbH
 #
 #    This file is part of LinOTP server.
 #
@@ -19,70 +19,65 @@
 #    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #
 #
-#    E-mail: linotp@lsexperts.de
+#    E-mail: linotp@keyidentity.com
 #    Contact: www.linotp.org
-#    Support: www.lsexperts.de
+#    Support: www.keyidentity.com
 #
 
 """
 validate controller - to check the authentication request
 """
 
-
-
-
-
 import logging
+
 import webob
-
-from pylons import tmpl_context as c
 from pylons import request, response, config
+from pylons import tmpl_context as c
 from pylons.controllers.util import abort
+from pylons.i18n.translation import _
+
+from linotp.lib.auth.validate import ValidationHandler
 from linotp.lib.base import BaseController
-
-from linotp.lib.util import get_client
-from linotp.lib.util import  getParam
-from linotp.lib.user import  getUserFromParam
-from linotp.lib.realm import  getDefaultRealm
-from linotp.lib.user import  getUserInfo
-from linotp.lib.user import  getUserId
-from linotp.lib.user    import User
-
-from linotp.lib.config  import getFromConfig
-
-from linotp.lib.token import checkUserPass, checkSerialPass
-from linotp.lib.token import get_tokenserial_of_transaction
-
-from linotp.lib.reply import sendResult, sendError
-from linotp.lib.reply import sendQRImageResult
-
-from linotp.model.meta import Session
-from linotp.lib.selftest import isSelfTest
-from linotp.lib.policy import check_user_authorization
-from linotp.lib.policy import check_auth_tokentype
-from linotp.lib.policy import check_auth_serial
-from linotp.lib.policy import AuthorizeException
-from linotp.lib.policy import set_realm
-from linotp.lib.policy import is_auth_return
-
-from linotp.lib.token import checkYubikeyPass
-from linotp.lib.token import getTokens4UserOrSerial
-
+from linotp.lib.config import getFromConfig
 from linotp.lib.error import ParameterError
 
-import traceback
+from linotp.lib.policy import AuthorizeException
+from linotp.lib.policy import check_auth_serial
+from linotp.lib.policy import check_auth_tokentype
+from linotp.lib.policy import check_user_authorization
+from linotp.lib.policy import is_auth_return
+from linotp.lib.policy import set_realm
+
+from linotp.lib.realm import getDefaultRealm
+from linotp.lib.reply import sendQRImageResult
+from linotp.lib.reply import sendResult, sendError
+from linotp.lib.selftest import isSelfTest
+from linotp.lib.token import getTokens4UserOrSerial
+from linotp.lib.token import get_tokenserial_of_transaction
+from linotp.tokens.base import TokenClass
+
+from linotp.lib.user import User
+from linotp.lib.user import getUserFromParam
+from linotp.lib.user import getUserId
+from linotp.lib.user import getUserInfo
+from linotp.lib.util import get_client
+
+from linotp.lib.context import request_context
+from linotp.lib.error import ValidateError
+from linotp.lib.pairing import decrypt_pairing_response
+
+import linotp.model
+
+
+Session = linotp.model.Session
+
+CONTENT_TYPE_PAIRING = 1
 
 audit = config.get('audit')
 
-optional = True
-required = False
-
 log = logging.getLogger(__name__)
 
 
-#from paste.debug.profile import profile_decorator
-
-
 class ValidateController(BaseController):
 
     '''
@@ -98,11 +93,10 @@ class ValidateController(BaseController):
 
     def __before__(self, action, **params):
 
-        log.debug("[__before__::%r] %r" % (action, params))
-
         try:
-            audit.initialize()
-            c.audit['client'] = get_client()
+            c.audit = request_context['audit']
+            c.audit['client'] = get_client(request)
+            request_context['Audit'] = audit
             return response
 
         except Exception as exx:
@@ -111,9 +105,6 @@ class ValidateController(BaseController):
             Session.close()
             return sendError(response, exx, context='before')
 
-        finally:
-            log.debug("[__before__::%r] done" % (action))
-
 
     def __after__(self, action, **params):
         audit.log(c.audit)
@@ -135,17 +126,17 @@ class ValidateController(BaseController):
 
         options = {}
 
-        ## put everythin in the options but the user, pass, init
+        # put everything in the options but the user, pass, init
         options.update(param)
         for para in ["pass", "user", "init"]:
             if options.has_key(para):
                 del options[para]
 
-        passw = getParam(param, "pass", optional)
-        user = getUserFromParam(param, optional)
+        passw = param.get("pass")
+        user = getUserFromParam(param)
 
         # support for ocra application challenge verification
-        challenge = getParam(param, "challenge", optional)
+        challenge = param.get("challenge")
         if challenge is not None:
             options = {}
             options['challenge'] = challenge
@@ -155,19 +146,21 @@ class ValidateController(BaseController):
         c.audit['realm'] = realm
 
         # AUTHORIZATION Pre Check
-        # we need to overwrite the user.realm in case the user does not exist in the original realm (setrealm-policy)
+        # we need to overwrite the user.realm in case the
+        # user does not exist in the original realm (setrealm-policy)
         user.realm = set_realm(user.login, realm, exception=True)
         check_user_authorization(user.login, user.realm, exception=True)
 
-        if isSelfTest() == True:
-            initTime = getParam(param, "init", optional)
+        if isSelfTest() is True:
+            initTime = param.get("init")
             if initTime is not None:
                 if options is None:
                     options = {}
                 options['initTime'] = initTime
+        vh = ValidationHandler()
+        (ok, opt) = vh.checkUserPass(user, passw, options=options)
 
-        (ok, opt) = checkUserPass(user, passw, options=options)
-
+        c.audit.update(request_context.get('audit'))
         c.audit['success'] = ok
 
         if ok:
@@ -177,7 +170,7 @@ class ValidateController(BaseController):
 
         # add additional details
         if is_auth_return(ok, user=user):
-            if opt == None:
+            if opt is None:
                 opt = {}
             if ok:
                 opt['realm'] = c.audit.get('realm')
@@ -227,13 +220,11 @@ class ValidateController(BaseController):
             If ``value`` is ``true`` the user was authenticated successfully.
         '''
 
-        param = {}
+        param = self.request_params.copy()
         ok = False
         opt = None
 
         try:
-            param.update(request.params)
-
             # prevent the detection if a user exist
             # by sending a request w.o. pass parameter
             try:
@@ -245,7 +236,7 @@ class ValidateController(BaseController):
                 c.audit['info'] = unicode(exx)
                 ok = False
                 if is_auth_return(ok):
-                    if opt == None:
+                    if opt is None:
                         opt = {}
                     opt['error'] = c.audit.get('info')
 
@@ -268,13 +259,64 @@ class ValidateController(BaseController):
         except Exception as exx:
             log.exception("[check] validate/check failed: %r" % exx)
             # If an internal error occurs or the SMS gateway did not send the SMS, we write this to the detail info.
+            c.audit['info'] = "%r" % exx
+            Session.rollback()
+            return sendResult(response, False, 0)
+
+        finally:
+            Session.close()
+
+    def check_status(self):
+        """
+        check the status of a transaction - for polling support
+        """
+
+        try:
+
+            param = self.request_params
+
+            #
+            # we require either state or transactionid as parameter
+
+            transid = param.get('state', param.get('transactionid', None))
+            if not transid:
+                raise ParameterError(_('Missing required parameter "state" or '
+                                     '"transactionid"!'))
+
+            #
+            # serial is an optional parameter
+
+            serial = param.get('serial', None)
+
+            # user is an optional parameter:
+            # if no 'user' in the parameters, the User object will be empty
+            user = getUserFromParam(param)
+
+            passw = param.get('pass')
+            if passw is None:
+                raise ParameterError(_('Missing required parameter "pass"!'))
+
+            use_offline = param.get('use_offline', False)
+
+            va = ValidationHandler()
+            ok, opt = va.check_status(transid=transid, user=user,
+                                      serial=serial, password=passw,
+                                      use_offline=use_offline)
+
+            c.audit['success'] = ok
+            c.audit['info'] = unicode(opt)
+
+            Session.commit()
+            return sendResult(response, ok, 0, opt=opt)
+
+        except Exception as exx:
+            log.exception("check_status failed: %r" % exx)
             c.audit['info'] = unicode(exx)
             Session.rollback()
             return sendResult(response, False, 0)
 
         finally:
             Session.close()
-            log.debug('[check] done')
 
 
     def check_yubikey(self):
@@ -307,13 +349,17 @@ class ValidateController(BaseController):
                 }
         '''
 
-        param = request.params
-        passw = getParam(param, "pass", required)
         try:
 
+            try:
+                passw = self.request_params['pass']
+            except KeyError:
+                raise ParameterError("Missing parameter: 'pass'")
+
             ok = False
             try:
-                ok, opt = checkYubikeyPass(passw)
+                vh = ValidationHandler()
+                ok, opt = vh.checkYubikeyPass(passw)
                 c.audit['success'] = ok
 
             except AuthorizeException as exx:
@@ -334,17 +380,14 @@ class ValidateController(BaseController):
 
         finally:
             Session.close()
-            log.debug('[check_yubikey] done')
 
     def check_url(self):
         '''
         This function works with pam_url.
         '''
         ok = False
-        param = {}
+        param = self.request_params
         try:
-            param.update(request.params)
-
             try:
                 (ok, opt) = self._check(param)
             except AuthorizeException as acc:
@@ -375,7 +418,6 @@ class ValidateController(BaseController):
 
         finally:
             Session.close()
-            log.debug("[check_url] done")
 
 
     def samlcheck(self):
@@ -399,7 +441,7 @@ class ValidateController(BaseController):
 
         try:
             opt = None
-            param = request.params
+            param = self.request_params
             (ok, opt) = self._check(param)
             attributes = {}
 
@@ -408,15 +450,14 @@ class ValidateController(BaseController):
                 try:
                     allowSAML = getFromConfig("allowSamlAttributes")
                 except:
-                    log.warning("[samlcheck] Calling controller samlcheck. But allowSamlAttributes == False.")
+                    log.warning("[samlcheck] Calling controller samlcheck. But allowSamlAttributes is False.")
                 if "True" == allowSAML:
                     ## Now we get the attributes of the user
-                    user = getUserFromParam(param, optional)
+                    user = getUserFromParam(param)
                     (uid, resId, resIdC) = getUserId(user)
                     userInfo = getUserInfo(uid, resId, resIdC)
-                    #users   = getUserList({ 'username':user.getUser()} , user)
                     log.debug("[samlcheck] getting attributes for: %s@%s"
-                              % (user.getUser(), user.getRealm()))
+                              % (user.login, user.realm))
 
                     res = userInfo
                     for key in ['username',
@@ -438,21 +479,22 @@ class ValidateController(BaseController):
 
         finally:
             Session.close()
-            log.debug('[samlcheck] done')
 
     def check_t(self):
 
-        param = {}
+        param = self.request_params.copy()
         value = {}
         ok = False
-        opt = None
+        opt = {}
 
         try:
-            param.update(request.params)
-            passw = getParam(param, "pass", required)
+            if 'pass' not in param:
+                raise ParameterError("Missing parameter: 'pass'")
+
+            passw = param['pass']
 
             transid = param.get('state', None)
-            if transid is not  None:
+            if transid is not None:
                 param['transactionid'] = transid
                 del param['state']
 
@@ -462,41 +504,15 @@ class ValidateController(BaseController):
             if transid is None:
                 raise Exception("missing parameter: state or transactionid!")
 
-            serial = get_tokenserial_of_transaction(transId=transid)
-            if serial is None:
-                value['value'] = False
-                value['failure'] = 'No challenge for transaction %r found'\
-                                    % transid
+            vh = ValidationHandler()
+            (ok, opt) = vh.check_by_transactionid(transid=transid,
+                                                  passw=passw,
+                                                  options=param)
 
-
-            else:
-                param['serial'] = serial
-
-                tokens = getTokens4UserOrSerial(serial=serial)
-                if len(tokens) == 0 or len(tokens) > 1:
-                    raise Exception('tokenmismatch for token serial: %s'
-                                    % (unicode(serial)))
-
-                theToken = tokens[0]
-                tok = theToken.token
-                realms = tok.getRealmNames()
-                if realms is None or len(realms) == 0:
-                    realm = getDefaultRealm()
-                elif len(realms) > 0:
-                    realm = realms[0]
-
-                userInfo = getUserInfo(tok.LinOtpUserid, tok.LinOtpIdResolver, tok.LinOtpIdResClass)
-                user = User(login=userInfo.get('username'), realm=realm)
-
-                (ok, opt) = checkSerialPass(serial, passw, user=user,
-                                     options=param)
-
-                value['value'] = ok
-                failcount = theToken.getFailCount()
-                value['failcount'] = int(failcount)
+            value['value'] = ok
+            value['failcount'] = int(opt.get('failcount', 0))
 
             c.audit['success'] = ok
-            #c.audit['info'] += "%s=%s, " % (k, value)
             Session.commit()
 
             qr = param.get('qr', None)
@@ -521,8 +537,140 @@ class ValidateController(BaseController):
 
         finally:
             Session.close()
-            log.debug('[check_t] done')
 
+    # ------------------------------------------------------------------------ -
+
+    def accept_transaction(self):
+
+        """
+        confirms a transaction.
+
+        needs the mandatory url query parameters:
+
+            * transactionid: unique id for the transaction
+            * signature: signature for the confirmation
+        """
+
+        try:
+
+            param = self.request_params.copy()
+
+            # -------------------------------------------------------------- --
+
+            # check the parameters
+
+            if 'signature' not in param:
+                raise ParameterError("Missing parameter: 'signature'!")
+
+            if 'transactionid' not in param:
+                raise ParameterError("Missing parameter: 'transactionid'!")
+
+            # -------------------------------------------------------------- --
+
+            # start the processing
+
+            passw = {'accept': param['signature']}
+            transid = param['transactionid']
+
+            vh = ValidationHandler()
+            ok, _opt = vh.check_by_transactionid(transid=transid,
+                                                 passw=passw,
+                                                 options=param)
+
+            # -------------------------------------------------------------- --
+
+            # finish the result
+
+            if 'serial' in _opt:
+                c.audit['serial'] = _opt['serial']
+
+            if 'token_type' in _opt:
+                c.audit['token_type'] = _opt['token_type']
+
+            c.audit['info'] = 'accept transaction: %r' % ok
+
+            c.audit['success'] = ok
+            Session.commit()
+
+            return sendResult(response, ok)
+
+        except Exception as exx:
+
+            log.exception("validate/accept_transaction failed: %r" % exx)
+            c.audit['info'] = "%r" % exx
+            Session.rollback()
+
+            return sendResult(response, False, 0)
+
+        finally:
+            Session.close()
+
+    # ------------------------------------------------------------------------ -
+
+    def reject_transaction(self):
+
+        """
+        rejects a transaction.
+
+        needs the mandatory url query parameters:
+
+            * transactionid: unique id for the transaction
+            * signature: signature for the rejection
+        """
+
+        try:
+
+            param = self.request_params.copy()
+
+            # -------------------------------------------------------------- --
+
+            # check the parameters
+
+            if 'signature' not in param:
+                raise ParameterError("Missing parameter: 'signature'!")
+
+            if 'transactionid' not in param:
+                raise ParameterError("Missing parameter: 'transactionid'!")
+
+            # -------------------------------------------------------------- --
+
+            # start the processing
+
+            passw = {'reject': param['signature']}
+            transid = param['transactionid']
+
+            vh = ValidationHandler()
+            ok, _opt = vh.check_by_transactionid(transid=transid,
+                                                 passw=passw,
+                                                 options=param)
+
+            # -------------------------------------------------------------- --
+
+            # finish the result
+
+            if 'serial' in _opt:
+                c.audit['serial'] = _opt['serial']
+
+            if 'token_type' in _opt:
+                c.audit['token_type'] = _opt['token_type']
+
+            c.audit['info'] = 'reject transaction: %r' % ok
+
+            c.audit['success'] = ok
+            Session.commit()
+
+            return sendResult(response, ok)
+
+        except Exception as exx:
+
+            log.exception("validate/reject_transaction failed: %r" % exx)
+            c.audit['info'] = "%r" % exx
+            Session.rollback()
+
+            return sendResult(response, False, 0)
+
+        finally:
+            Session.close()
 
     def check_s(self):
         '''
@@ -539,8 +687,7 @@ class ValidateController(BaseController):
         returns:
             JSON response
         '''
-        param = {}
-        param.update(request.params)
+        param = self.request_params
 
         options = {}
         options.update(param)
@@ -549,16 +696,16 @@ class ValidateController(BaseController):
                 del options[k]
 
         if 'init' in param:
-            if isSelfTest() == True:
+            if isSelfTest() is True:
                 options['initTime'] = param.get('init')
 
         try:
-            passw = getParam(param, "pass", optional)
-            serial = getParam(param, 'serial', optional)
+            passw = param.get("pass")
+            serial = param.get('serial')
             if serial is None:
-                user = getParam(param, 'user', optional)
-                if user is  not None:
-                    user = getUserFromParam(param, optional)
+                user = param.get('user')
+                if user is not None:
+                    user = getUserFromParam(param)
                     toks = getTokens4UserOrSerial(user=user)
                     if len(toks) == 0:
                         raise Exception("No token found!")
@@ -573,22 +720,26 @@ class ValidateController(BaseController):
                         elif len(realms) > 0:
                             realm = realms[0]
 
-                        userInfo = getUserInfo(tok.LinOtpUserid, tok.LinOtpIdResolver, tok.LinOtpIdResClass)
-                        user = User(login=userInfo.get('username'), realm=realm)
+                        userInfo = getUserInfo(tok.LinOtpUserid,
+                                               tok.LinOtpIdResolver,
+                                               tok.LinOtpIdResClass)
+                        user = User(login=userInfo.get('username'),
+                                    realm=realm)
 
                         serial = tok.getSerial()
 
             c.audit['serial'] = serial
 
-            if isSelfTest() == True:
-                initTime = getParam(param, "init", optional)
+            if isSelfTest() is True:
+                initTime = param.get("init")
                 if initTime is not None:
                     if options is None:
                         options = {}
                     options['initTime'] = initTime
 
-            (ok, opt) = checkSerialPass(serial, passw, options=options)
-
+            options['scope'] = {"check_s": True}
+            vh = ValidationHandler()
+            (ok, opt) = vh.checkSerialPass(serial, passw, options=options)
             c.audit['success'] = ok
             Session.commit()
 
@@ -610,11 +761,10 @@ class ValidateController(BaseController):
             log.exception("[check_s] validate/check_s failed: %r" % exx)
             c.audit['info'] = unicode(exx)
             Session.rollback()
-            return sendResult(response, False, 0)
+            return sendResult(response, False, id=0, status=False)
 
         finally:
             Session.close()
-            log.debug('[check_s] done')
 
 
 
@@ -642,7 +792,7 @@ class ValidateController(BaseController):
                 in case of any error
         '''
         opt = None
-        param = request.params
+        param = self.request_params
         res = []
 
         try:
@@ -656,13 +806,13 @@ class ValidateController(BaseController):
 
             Session.commit()
 
-            if ok == True:
+            if ok is True:
                 ret = u":-)"
             else:
                 ret = u":-("
             res.append(ret)
 
-            if opt != None:
+            if opt is not None:
 
                 if 'state' in opt or 'transactionid' in opt:
                     stat = opt.get('transactionid') or opt.get('state')
@@ -682,7 +832,7 @@ class ValidateController(BaseController):
 
         finally:
             Session.close()
-            log.debug('[simplecheck] done')
+
 
     def ok(self):
         return sendResult(response, True, 0)
@@ -708,12 +858,12 @@ class ValidateController(BaseController):
             JSON response
         '''
         ret = False
-        param = request.params
+        param = self.request_params
         state = ''
         message = 'No sms message defined!'
 
         try:
-            user = getUserFromParam(param, optional)
+            user = getUserFromParam(param)
             c.audit['user'] = user.login
             c.audit['realm'] = user.realm or getDefaultRealm()
             c.audit['success'] = 0
@@ -746,14 +896,89 @@ class ValidateController(BaseController):
 
         except Exception as exx:
             log.exception("[smspin] validate/smspin failed: %r" % exx)
-            # If an internal error occurs or the SMS gateway did not send the SMS, we write this to the detail info.
+            # If an internal error occurs or the SMS gateway did not send
+            # the SMS, we write this to the detail info.
             c.audit['info'] = unicode(exx)
             Session.rollback()
             return sendResult(response, False, 0)
 
         finally:
             Session.close()
-            log.debug("[smspin] done")
 
-#eof###########################################################################
+    def pair(self):
+        """
+        validate/pair: for the enrollment of qr and push token
+        """
 
+        try:
+
+            # -------------------------------------------------------------- --
+
+            enc_response = self.request_params.get('pairing_response')
+
+            if enc_response is None:
+                raise Exception('Parameter missing')
+
+            # -------------------------------------------------------------- --
+
+            dec_response = decrypt_pairing_response(enc_response)
+            token_type = dec_response.token_type
+            pairing_data = dec_response.pairing_data
+
+            if not hasattr(pairing_data, 'serial') or \
+               pairing_data.serial is None:
+
+                raise ValidateError('Pairing responses with no serial attached'
+                                    ' are currently not implemented.')
+
+            # --------------------------------------------------------------- -
+
+            # TODO: pairing policy
+            tokens = getTokens4UserOrSerial(None, pairing_data.serial)
+
+            if not tokens:
+                raise Exception('Invalid serial in pairing response')
+
+            if len(tokens) > 1:
+                raise Exception('Multiple tokens found. Pairing not possible')
+
+            token = tokens[0]
+
+            # prepare some audit entries
+            t_owner = token.getUser()
+
+            realms = token.getRealms()
+            realm = ''
+            if realms:
+                realm = realms[0]
+
+            c.audit['user'] = t_owner or ''
+            c.audit['realm'] = realm
+
+            # --------------------------------------------------------------- --
+
+            if token.type != token_type:
+                raise Exception('Serial in pairing response doesn\'t match '
+                                'supplied token_type')
+
+            # --------------------------------------------------------------- --
+
+            token.pair(pairing_data)
+            c.audit['success'] = 1
+            c.audit['serial'] = token.getSerial()
+
+            Session.commit()
+            return sendResult(response, False)
+
+        # ------------------------------------------------------------------- --
+
+        except Exception as exx:
+            log.exception("validate/pair failed: %r" % exx)
+            c.audit['info'] = unicode(exx)
+            Session.rollback()
+            return sendResult(response, False, 0, status=False)
+
+        finally:
+            Session.close()
+
+# eof #########################################################################
