@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 #
 #    LinOTP - the open source solution for two factor authentication
-#    Copyright (C) 2010 - 2016 LSE Leading Security Experts GmbH
+#    Copyright (C) 2010 - 2019 KeyIdentity GmbH
 #
 #    This file is part of LinOTP server.
 #
@@ -19,9 +19,9 @@
 #    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #
 #
-#    E-mail: linotp@lsexperts.de
+#    E-mail: linotp@keyidentity.com
 #    Contact: www.linotp.org
-#    Support: www.lsexperts.de
+#    Support: www.keyidentity.com
 #
 """
 HMAC-OTP (RFC 4226)
@@ -50,12 +50,12 @@ class HmacOtp():
         self.hashfunc = hashfunc
 
     def hmac(self, counter=None, key=None):
-        #log.error("hmacSecret()")
         counter = counter or self.counter
 
         data_input = struct.pack(">Q", counter)
         if key is None:
-            dig = str(self.secretObj.hmac_digest(data_input, self.hashfunc))
+            dig = str(self.secretObj.hmac_digest(data_input,
+                                                 hash_algo=self.hashfunc))
         else:
             if pver > 2.6:
                 dig = hmac.new(key, data_input, self.hashfunc).digest()
@@ -80,7 +80,6 @@ class HmacOtp():
         otp = str(self.truncate(self.hmac(counter=counter, key=key)))
         """  fill in the leading zeros  """
         sotp = (self.digits - len(otp)) * "0" + otp
-        #log.debug("[generate] %s %s %s" % (str(counter), str(otp), str(sotp) ) )
         if inc_counter:
             self.counter = counter + 1
         return sotp
@@ -89,21 +88,16 @@ class HmacOtp():
         res = -1
         start = self.counter
         end = self.counter + window
-        if symetric == True:
+        if symetric is True:
             # changed window/2 to window for TOTP
             start = self.counter - (window)
             start = 0 if (start < 0) else start
             end = self.counter + (window)
 
-        log.debug("[checkOTP] OTP range counter: %r - %r" % (start, end))
         for c in range(start , end):
             otpval = self.generate(c)
-            log.debug("[checkOtp] calculating counter %r: %r %r"
-                      % (c, anOtpVal, otpval))
-            #log.error("otp[%d]: %s : %s",c,otpval,anOtpVal)
 
             if (unicode(otpval) == unicode(anOtpVal)):
-                # log.debug("Match Pin: %s : %d : %s",otpval,c,anOtpVal)
                 res = c
                 break
         #return -1 or the counter
