@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 #
 #    LinOTP - the open source solution for two factor authentication
-#    Copyright (C) 2010 - 2016 LSE Leading Security Experts GmbH
+#    Copyright (C) 2010 - 2019 KeyIdentity GmbH
 #
 #    This file is part of LinOTP server.
 #
@@ -19,9 +19,9 @@
 #    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #
 #
-#    E-mail: linotp@lsexperts.de
+#    E-mail: linotp@keyidentity.com
 #    Contact: www.linotp.org
-#    Support: www.lsexperts.de
+#    Support: www.keyidentity.com
 #
 """
 Parsing of pskc files:
@@ -39,7 +39,7 @@ sha256 = hashlib.sha256
 import logging
 log = logging.getLogger(__name__)
 
-import linotp.lib.pbkdf2 as pbkdf2
+import linotp.lib.crypto.pbkdf2 as pbkdf2
 
 from linotp.lib.ImportOTP  import getTagName, ImportException
 
@@ -68,7 +68,7 @@ def getMacMethod(elem):
 
 
 def aes_decrypt(transport_b64, key_hex, serial=""):
-    import Crypto.Cipher.AES as AES
+    import Cryptodome.Cipher.AES as AES
 
     def hack(data, serial=""):
         bsize = 16
