@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 #
 #    LinOTP - the open source solution for two factor authentication
-#    Copyright (C) 2010 - 2016 LSE Leading Security Experts GmbH
+#    Copyright (C) 2010 - 2019 KeyIdentity GmbH
 #
 #    This file is part of LinOTP server.
 #
@@ -19,9 +19,9 @@
 #    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #
 #
-#    E-mail: linotp@lsexperts.de
+#    E-mail: linotp@keyidentity.com
 #    Contact: www.linotp.org
-#    Support: www.lsexperts.de
+#    Support: www.keyidentity.com
 #
 """
 This file is used used for importing SafeNet (former Aladdin)
@@ -34,7 +34,7 @@ import os
 import binascii
 from linotp.lib.util import modhex_decode
 from linotp.lib.util import modhex_encode
-from Crypto.Cipher import AES
+from Cryptodome.Cipher import AES
 
 import logging
 log = logging.getLogger(__name__)
@@ -43,12 +43,14 @@ log = logging.getLogger(__name__)
 def getKnownTypes():
     return ["feitian", "pskc", "dpw", 'dat', "vasco"]
 
+
 def getImportText():
-    return { 'feitian' : 'Feitian XML',
-        'pskc' : 'OATH compliant PSKC',
-        'dpw' : 'Tagespasswort Token File',
-        'dat' : 'eToken DAT File',
-        'vasco' : 'Vasco DPX' }
+    return {'feitian': 'Feitian XML',
+            'pskc': 'OATH compliant PSKC',
+            'dpw': 'Tagespasswort Token File',
+            'dat': 'eToken DAT File',
+            'vasco': 'Vasco DPX'}
+
 
 def create_static_password(key_hex):
     '''
@@ -63,6 +65,7 @@ def create_static_password(key_hex):
 
     return password
 
+
 class ImportException(Exception):
     def __init__(self, description):
         #self.auth_scope = scope
@@ -73,6 +76,7 @@ class ImportException(Exception):
     def __str__(self):
         return ('%s' % self.description)
 
+
 def getTagName(elem):
     match = re.match("^({.*?})(.*)$", elem.tag)
     if match:
@@ -83,91 +87,162 @@ def getTagName(elem):
 
 def parseOATHcsv(csv):
     '''
-    (#653)
+
     This function parses CSV data for oath token.
     The file format is
 
-        serial, key, [hotp,totp], [6,8], [30|60],
+        serial, key, [hotp,totp], [6,8], [30|60], [sha1|sha256|sha512],
         serial, key, ocra, [ocra-suite]
 
-    It imports sha1 hotp or totp token.
-    I can also import ocra token.
-    The default is hotp
-    if totp is set, the default seconds are 30
-    if ocra is set, an ocra-suite is required, otherwise the default ocra-suite is used.
+    It imports standard hmac algorithm based tokens
+
+    If the seed is 32 bytes long or at the end of the row the hashlib is
+       defined the tokens hashlib changes to sha265. For seeds with 64 bytes
+       the hashlib is determined as sha512 if not other specified.
+
+    It can also import ocra token.
+
+    * The default is hotp
+    * if totp is set, the default seconds are 30
+
+    * if ocra is set, an ocra-suite is required
 
     It returns a dictionary:
         {
             serial: {   'type' : xxxx,
                         'hmac_key' : xxxx,
+                        'hashlib' : 'sha1|sha256|sha512'
                         'timeStep' : xxxx,
                         'otplen' : xxx,
                         'ocrasuite' : xxx  }
         }
     '''
     TOKENS = {}
-    log.debug("[parseOATHcsv] starting to parse an oath csv file.")
 
-    #if type(csv) is str:
+    # we cant use the csv parser here as we have variable length data. So
+    # we do the split into lines manualy
+
     csv_array = csv.split('\n')
-    #else:
-    #    csv_array = csv
 
-    log.debug("[parseOATHcsv] the file contains %i tokens." % len(csv_array))
-    for line in csv_array:
-        l = line.split(',')
-        serial = ""
-        key = ""
-        ttype = "hmac"
-        seconds = 30
-        otplen = 6
-        hashlib = "sha1"
-        ocrasuite = ""
-        if len(l) >= 1:
-            serial = l[0].strip()
-        else:
-            log.error("[parseOATHcsv] the line %s did not contain a serial number" % line)
+    log.debug("[parseOATHcsv] starting to parse an oath csv file.")
+    log.debug("[parseOATHcsv] the file contains %i lines.", len(csv_array))
+
+    for csv_line in csv_array:
+
+        token = {}
+
+        # we extend the line to contain always 8 columns
+
+        line = [x.strip() for x in csv_line.split(',')]
+        line += [''] * (8 - len(line))
+
+        # ------------------------------------------------------------------ --
+
+        # 1. column: serial
+
+        serial = line[0]
+        if not serial:
+            log.error("[parseOATHcsv] the line %s did not contain"
+                      " a serial number", csv_line)
             continue
 
-        if len(l) >= 2:
-            key = l[1].strip()
-
-            if len(key) == 32:
-                hashlib = "sha256"
-        else:
-            log.error("[parseOATHcsv] the line %s did not contain a hmac key" % line)
+        if serial == '#':
             continue
 
-        # ttype
-        if len(l) >= 3:
-            ttype = l[2].strip().lower()
-            if ttype == "hotp":
-                ttype = "hmac"
+        token['serial'] = serial
 
-        # otplen or ocrasuite
-        if len(l) >= 4:
-            if ttype in ["ocra", "ocra2"]:
-                ocrasuite = l[3].strip()
+        # ------------------------------------------------------------------ --
+
+        # 2 column: seed
+
+        key = line[1]
+        if not key:
+            log.error("[parseOATHcsv] the line %s did not contain"
+                      " a hmac key" % csv_line)
+            continue
+
+        token['hmac_key'] = key
+
+        # ------------------------------------------------------------------ --
+
+        # 3 column: token type
+
+        ttype = line[2].lower()
+        if ttype == "hotp":
+            ttype = "hmac"
+
+        if not ttype:
+            ttype = "hmac"
+
+        token['type'] = ttype
+
+        # ------------------------------------------------------------------ --
+
+        # 4 column: otplen or ocrasuite
+
+        if ttype in ["ocra", "ocra2"]:
+
+            ocrasuite = line[3]
+
+            if not ocrasuite:
+                log.error("[parseOATHcsv] the line %s did not contain"
+                          " the ocrasuite for the ocra token!" % csv_line)
+                continue
+
+            token['ocrasuite'] = ocrasuite
+
+        else:
+
+            otplen = line[3]
+            if otplen:
+                otplen = int(otplen)
             else:
-                otplen = int(l[3].strip())
+                otplen = 6
 
-        # timeStep
-        if len(l) >= 5:
-            seconds = int(l[4].strip())
+            token['otplen'] = otplen
 
-        log.debug("[parseOATHcsv] read the line |%s|%s|%s|%i %s|%i|" % (serial, key, ttype, otplen, ocrasuite, seconds))
+        # ------------------------------------------------------------------ --
 
-        TOKENS[serial] = { 'type' : ttype,
-                           'hmac_key' : key,
-                           'timeStep' : seconds,
-                           'otplen' : otplen,
-                           'hashlib' : hashlib,
-                           'ocrasuite' : ocrasuite
-                          }
-    log.debug("[parseOATHcsv] read the following values: %s" % str(TOKENS))
+        # 5 column: timeStep
+
+        if ttype in ['totp']:
+            try:
+                seconds = int(line[4])
+            except ValueError:
+                seconds = 30
+
+            token['timeStep'] = seconds
+
+        # ------------------------------------------------------------------ --
+
+        # 6 column: hash lib
+
+        hash_hint = line[5].lower()
+        if hash_hint and hash_hint in ['sha1', 'sha256', 'sha512']:
+            hashlib = hash_hint
+        else:
+            if len(key) == 2 * 64:
+                hashlib = "sha512"
+            elif len(key) == 2 * 32:
+                hashlib = "sha256"
+            else:
+                hashlib = 'sha1'
+
+        if ttype not in ["ocra", "ocra2"]:
+            token['hashlib'] = hashlib
+
+        # ------------------------------------------------------------------ --
+
+        log.debug("[parseOATHcsv] read the line >%s< into token: >%r<",
+                  csv_line, token)
+
+        TOKENS[serial] = token
+
+    log.debug("[parseOATHcsv] read the following values: %r", TOKENS)
 
     return TOKENS
 
+
 def parseYubicoCSV(csv):
     '''
     This function reads the CSV data as created by the Yubico personalization GUI.
@@ -232,17 +307,18 @@ def parseYubicoCSV(csv):
                     log.warning("No public ID in line %r" % line)
                     serial_int = int(binascii.hexlify(os.urandom(4)), 16)
                 else:
-                    serial_int = int(binascii.hexlify(modhex_decode(public_id)), 16)
+                    serial_int = int(binascii.hexlify(
+                        modhex_decode(public_id)), 16)
 
                 if typ.lower() == "yubico otp":
                     ttype = "yubikey"
                     otplen = 32 + len(public_id)
                     serial = "UBAM%08d_%s" % (serial_int, slot)
-                    TOKENS[serial] = { 'type' : ttype,
-                               'hmac_key' : key,
-                               'otplen' : otplen,
-                               'description': public_id
-                              }
+                    TOKENS[serial] = {'type': ttype,
+                                      'hmac_key': key,
+                                      'otplen': otplen,
+                                      'description': public_id
+                                      }
                 elif typ.lower() == "oath-hotp":
                     '''
                     TODO: this does not work out at the moment, since the GUI either
@@ -251,14 +327,18 @@ def parseYubicoCSV(csv):
                     '''
                     ttype = "hmac"
                     otplen = 6
+                    if l and len(l) >= 11 and l[11] and l[11].strip():
+                        otplen = int(l[11])
                     serial = "UBOM%08d_%s" % (serial_int, slot)
-                    TOKENS[serial] = { 'type' : ttype,
-                               'hmac_key' : key,
-                               'otplen' : otplen,
-                               'description': public_id
-                              }
+                    TOKENS[serial] = {
+                        'type': ttype,
+                        'hmac_key': key,
+                        'otplen': otplen,
+                        'description': public_id
+                    }
                 else:
-                    log.warning("[parseYubicoCSV] at the moment we do only support Yubico OTP and HOTP: %r" % line)
+                    log.warning(
+                        "[parseYubicoCSV] at the moment we do only support Yubico OTP and HOTP: %r" % line)
                     continue
             elif first_column.isdigit():
                 # first column is a number, (serial number), so we are in the yubico format
@@ -286,16 +366,16 @@ def parseYubicoCSV(csv):
                     serial = "UBAM%s_%s" % (serial, slot)
                     public_id = l[1].strip()
                     otplen = 32 + len(public_id)
-                TOKENS[serial] = { 'type' : typ,
-                               'hmac_key' : key,
-                               'otplen' : otplen,
-                               'description': public_id
-                              }
+                TOKENS[serial] = {'type': typ,
+                                  'hmac_key': key,
+                                  'otplen': otplen,
+                                  'description': public_id
+                                  }
         else:
-            log.warning("[parseYubicoCSV] the line %r did not contain a enough values" % line)
+            log.warning(
+                "[parseYubicoCSV] the line %r did not contain a enough values" % line)
             continue
 
-
     log.debug("[parseOATHcsv] read the following values: %s" % str(TOKENS))
 
     return TOKENS
@@ -316,7 +396,6 @@ def parseSafeNetXML(xml):
     if getTagName(elem_tokencontainer) != "Tokens":
         raise ImportException("No toplevel element Tokens")
 
-
     for elem_token in list(elem_tokencontainer):
         SERIAL = None
         COUNTER = None
@@ -329,7 +408,8 @@ def parseSafeNetXML(xml):
                 tag = getTagName(elem_tdata)
                 if "ProductName" == tag:
                     DESCRIPTION = elem_tdata.text
-                    log.debug("The Token with the serial %s has the productname %s" % (SERIAL, DESCRIPTION))
+                    log.debug("The Token with the serial %s has the productname %s" % (
+                        SERIAL, DESCRIPTION))
                 if "Applications" == tag:
                     for elem_apps in elem_tdata:
                         if getTagName(elem_apps) == "Application":
@@ -348,12 +428,13 @@ def parseSafeNetXML(xml):
                         hashlib = "sha256"
 
                     TOKENS[SERIAL] = {
-                        'hmac_key' : HMAC,
-                        'counter' : COUNTER,
-                        'type' : 'HMAC',
-                        'hashlib' : hashlib
+                        'hmac_key': HMAC,
+                        'counter': COUNTER,
+                        'type': 'HMAC',
+                        'hashlib': hashlib
                     }
                 else:
-                    log.error("Found token %s without a element 'Seed'" % SERIAL)
+                    log.error(
+                        "Found token %s without a element 'Seed'" % SERIAL)
 
     return TOKENS
