@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 #
 #    LinOTP - the open source solution for two factor authentication
-#    Copyright (C) 2010 - 2016 LSE Leading Security Experts GmbH
+#    Copyright (C) 2010 - 2019 KeyIdentity GmbH
 #
 #    This file is part of LinOTP server.
 #
@@ -19,9 +19,9 @@
 #    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #
 #
-#    E-mail: linotp@lsexperts.de
+#    E-mail: linotp@keyidentity.com
 #    Contact: www.linotp.org
-#    Support: www.lsexperts.de
+#    Support: www.keyidentity.com
 #
 
 """The application's Globals object"""
@@ -47,9 +47,7 @@ class Globals(object):
         'app_globals' variable
 
         """
-        self.rwl = RWLock()
         self.rwl2 = RWLock()
-        self.resolverLock = RWLock()
         self.rcount = 0
 
         self.config = {}
@@ -57,33 +55,9 @@ class Globals(object):
         self.configLock = RWLock()
         secLock = RWLock()
 
-        self.tokenprefixes = {}
-        self.tokenclasses = {}
         self.security_provider = SecurityProvider(secLock)
 
-        self.resolver_clazzes = {}
-        self.resolver_types = {}
-
-    def setResolverClasses(self, resolver_clazzes=None):
-        '''
-        setter to hold the reference to all resolver class objects
-        '''
-        if resolver_clazzes is not None:
-            self.resolver_clazzes = resolver_clazzes
-
-    def getResolverClasses(self):
-        return self.resolver_clazzes
-
-    def setResolverTypes(self, resolver_types=None):
-        """
-        setter to hold the reference to all resolver class names
-        """
-        if resolver_types is not None:
-            self.resolver_types = resolver_types
-
-    def getResolverTypes(self):
-        return self.resolver_types
-
+        self.cache_manager = None
 
     def getConfig(self):
         '''
@@ -96,21 +70,6 @@ class Globals(object):
             self.configLock.release()
         return config
 
-    def setTokenclasses(self, tcl):
-        self.tokenclasses = tcl
-        return
-
-    def getTokenclasses(self):
-        return self.tokenclasses
-
-    def setTokenprefixes(self, tpl):
-        self.tokenprefixes = tpl
-        return
-
-    def getTokenprefixes(self):
-        return self.tokenprefixes
-
-
     def setConfig(self, config, replace=False):
         '''
             set the app global config for linotp
@@ -125,7 +84,7 @@ class Globals(object):
 
             else:
                 conf = copy.deepcopy(config)
-                if replace == True:
+                if replace is True:
                     self.config = conf
                 else:
                     self.config.update(conf)
@@ -175,10 +134,6 @@ class Globals(object):
             self.configLock.release()
         return
 
-
-    def getLock(self):
-        return self.rwl
-
     def setConfigReadLock(self):
         self.rcount = self.rcount + 1
         self.rwl2.acquire_read()
