@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 #
 #    LinOTP - the open source solution for two factor authentication
-#    Copyright (C) 2010 - 2016 LSE Leading Security Experts GmbH
+#    Copyright (C) 2010 - 2019 KeyIdentity GmbH
 #
 #    This file is part of LinOTP server.
 #
@@ -19,9 +19,9 @@
 #    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #
 #
-#    E-mail: linotp@lsexperts.de
+#    E-mail: linotp@keyidentity.com
 #    Contact: www.linotp.org
-#    Support: www.lsexperts.de
+#    Support: www.keyidentity.com
 #
 """
 
@@ -40,7 +40,7 @@ import urllib
 import logging
 log = logging.getLogger(__name__)
 
-from linotp.lib.policy import get_tokenlabel
+from linotp.lib.policy import get_tokenlabel, get_tokenissuer
 from urllib import quote
 
 class NoOtpAuthTokenException(Exception):
@@ -74,8 +74,8 @@ def create_google_authenticator(param, user=None):
         login = user.login or ''
         realm = user.realm or ''
 
-    login = param.get('user.login', login) or ''
-    realm = param.get('user.realm', realm) or ''
+    login = login or param.get('user.login', '')
+    realm = realm or param.get('user.realm', '')
 
     url_param = {}
 
@@ -104,6 +104,10 @@ def create_google_authenticator(param, user=None):
     if 'timeStep' in param:
         url_param['period'] = param.get('timeStep')
 
+    issuer = get_tokenissuer(login, realm, serial)
+    if issuer:
+        url_param['issuer'] = quote(issuer)
+
     ga = "otpauth://%s/%s" % (typ, serial)
     qg_param = urllib.urlencode(url_param)
 
@@ -125,8 +129,10 @@ def create_google_authenticator(param, user=None):
         if len(param.get('description', '')) > 0:
             label = label + ':' + param.get('description')
 
+    if issuer:
+        label = issuer + ':' + label
     label = label[0:allowed_label_len]
-    url_label = quote(label)
+    url_label = quote(label, ':')
 
     ga = "otpauth://%s/%s?%s" % (typ, url_label, qg_param)
     log.debug("google authenticator: %r" % ga[:20])
