@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 #
 #    LinOTP - the open source solution for two factor authentication
-#    Copyright (C) 2010 - 2016 LSE Leading Security Experts GmbH
+#    Copyright (C) 2010 - 2019 KeyIdentity GmbH
 #
 #    This file is part of LinOTP server.
 #
@@ -19,9 +19,9 @@
 #    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #
 #
-#    E-mail: linotp@lsexperts.de
+#    E-mail: linotp@keyidentity.com
 #    Contact: www.linotp.org
-#    Support: www.lsexperts.de
+#    Support: www.keyidentity.com
 #
 """
 This is the Audit Class, that writes Audits to SQL DB
@@ -46,27 +46,30 @@ from linotp.lib.audit.base import AuditBase
 from pylons import config
 
 import logging.config
-
+import traceback
 
 import linotp
 
+from linotp.lib.text_utils import utf8_slice
+
 # Create the logging object from the linotp.ini config file
 ini_file = config.get("__file__")
 if ini_file is not None:
     # When importing the module with Sphinx to generate documentation
     # 'ini_file' is None. In other cases this should not be the case.
     logging.config.fileConfig(ini_file, disable_existing_loggers=False)
+
 log = logging.getLogger(__name__)
 
 metadata = schema.MetaData()
 
-
 def now():
     u_now = u"%s" % datetime.datetime.now().strftime("%Y-%m-%d %H:%M:%S.%f")
     return u_now
 
 ######################## MODEL ################################################
 table_prefix = config.get("linotpAudit.sql.table_prefix", "")
+
 audit_table_name = '%saudit' % table_prefix
 
 audit_table = schema.Table(audit_table_name, metadata,
@@ -95,17 +98,17 @@ AUDIT_ENCODE = ["action", "serial", "success", "user", "realm", "tokentype",
                 "administrator", "action_detail", "info", "linotp_server",
                 "client", "log_level"]
 
-
 class AuditTable(object):
 
     def __init__(self, serial=u"", action=u"", success=u"False",
-                tokentype=u"", user=u"",
-                realm=u"", administrator=u"",
-                action_detail=u"", info=u"",
-                linotp_server=u"",
-                client=u"",
-                log_level=u"INFO",
-                clearance_level=0):
+                 tokentype=u"", user=u"",
+                 realm=u"", administrator=u"",
+                 action_detail=u"", info=u"",
+                 linotp_server=u"",
+                 client=u"",
+                 log_level=u"INFO",
+                 clearance_level=0,
+                 config_param=None):
         """
         build an audit db entry
 
@@ -144,6 +147,12 @@ class AuditTable(object):
         log.debug("[__init__] creating AuditTable object, action = %s"
                   % action)
 
+        if config_param:
+            self.config = config_param
+        else:
+            self.config = config
+        self.trunc_as_err = self.config.get('linotpAudit.error_on_truncation',
+                                            'False') == 'True'
         self.serial = unicode(serial or '')
         self.action = unicode(action or '')
         self.success = unicode(success or '0')
@@ -151,8 +160,14 @@ class AuditTable(object):
         self.user = unicode(user or '')
         self.realm = unicode(realm or '')
         self.administrator = unicode(administrator or '')
-        self.action_detail = unicode(action_detail or '')
-        self.info = unicode(info or '')
+
+        #
+        # we have to truncate the 'action_detail' and the 'info' data
+        # in utf-8 compliant way
+        #
+        self.action_detail = utf8_slice(unicode(action_detail or ''), 512).next()
+        self.info = utf8_slice(unicode(info or ''), 512).next()
+
         self.linotp_server = unicode(linotp_server or '')
         self.client = unicode(client or '')
         self.log_level = unicode(log_level or '')
@@ -183,15 +198,13 @@ class AuditTable(object):
         """
         if type(value) in [str, unicode]:
             field_len = self._get_field_len(name)
-            encoded_value = linotp.lib.crypt.uencode(value)
+            encoded_value = linotp.lib.crypto.uencode(value)
             if field_len != -1 and len(encoded_value) > field_len:
-                log.warning("truncating audit data: [audit.%s] %s"
-                            % (name, value))
-                trunc_as_err = config.get("linotpAudit.error_on_truncation",
-                                          False) or False
-                if trunc_as_err != False:
-                    raise Exception("truncating audit data: [audit.%s] %s"
-                                    % (name, value))
+                log.warning("truncating audit data: [audit.%s] %s",
+                            name, value)
+                if self.trunc_as_err is not False:
+                    raise Exception("truncating audit data: [audit.%s] %s" %
+                                    (name, value))
 
                 ## during the encoding the value might expand -
                 ## so we take this additional length into account
@@ -201,7 +214,7 @@ class AuditTable(object):
         if name in AUDIT_ENCODE:
             ## encode data
             if value:
-                value = linotp.lib.crypt.uencode(value)
+                value = linotp.lib.crypto.uencode(value)
         super(AuditTable, self).__setattr__(name, value)
 
     def __getattribute__(self, name):
@@ -217,7 +230,7 @@ class AuditTable(object):
         value = object.__getattribute__(self, name)
         if name in AUDIT_ENCODE:
             if value:
-                value = linotp.lib.crypt.udecode(value)
+                value = linotp.lib.crypto.udecode(value)
             else:
                 value = ""
 
@@ -255,7 +268,6 @@ def add_column(engine, table, column):
 
     except Exception as exx:
         # Obviously we already migrated the database.
-        log.info("[__init__] Error during database migration: %r" % exx)
         result = False
 
     return result
@@ -266,12 +278,13 @@ class Audit(AuditBase):
     """
     Audit Implementation to the generic audit interface
     """
-    def __init__(self):
+    def __init__(self, config):
         self.name = "SQlAudit"
-
+        self.config = config
         connect_string = config.get("linotpAudit.sql.url")
         pool_recycle = config.get("linotpAudit.sql.pool_recyle", 3600)
         implicit_returning = config.get("linotpSQL.implicit_returning", True)
+
         self.engine = None
         ########################## SESSION ##################################
 
@@ -300,15 +313,12 @@ class Audit(AuditBase):
         # initialize signing keys
         self.readKeys()
 
-        self.PublicKey = RSA.load_pub_key(config.get("linotpAudit.key.public"))
+        self.PublicKey = RSA.load_pub_key(
+                                          self.config.get("linotpAudit.key.public"))
         self.VerifyEVP = EVP.PKey()
         self.VerifyEVP.reset_context(md='sha256')
         self.VerifyEVP.assign_rsa(self.PublicKey)
 
-        # create the column "client"
-        column = schema.Column("client", types.Unicode(80))
-        add_column(self.engine, audit_table, column)
-
         return
 
     def _attr_to_dict(self, audit_line):
@@ -343,15 +353,14 @@ class Audit(AuditBase):
         '''
         line = self._attr_to_dict(audit_line)
         s_audit = getAsString(line)
-        log.debug("[_sign] signing %s" % s_audit)
 
         key = EVP.load_key_string(self.private)
         key.reset_context(md='sha256')
         key.sign_init()
         key.sign_update(s_audit)
         signature = key.sign_final()
-        log.debug("[_sign] signature : %s" % hexlify(signature))
-        return hexlify(signature)
+        return u'' + hexlify(signature)
+
 
     def _verify(self, auditline, signature):
         '''
@@ -363,7 +372,6 @@ class Audit(AuditBase):
             return res
 
         s_audit = getAsString(auditline)
-        log.debug("[_verify] verifying %s" % s_audit)
 
         self.VerifyEVP.verify_init()
         self.VerifyEVP.verify_update(s_audit)
@@ -376,15 +384,14 @@ class Audit(AuditBase):
         This method is used to log the data. It splits information of
         multiple tokens (e.g from import) in multiple audit log entries
         '''
-        log.debug("[log] writing audit log message")
 
         try:
             serial = param.get('serial', '') or ''
             if not serial:
-                ## if no serial, do as before
+                # if no serial, do as before
                 self.log_entry(param)
             else:
-                ## look if we have multiple serials inside
+                # look if we have multiple serials inside
                 serials = serial.split(',')
                 for serial in serials:
                     p = {}
@@ -392,17 +399,11 @@ class Audit(AuditBase):
                     p['serial'] = serial
                     self.log_entry(p)
 
-            #self.session.commit()
-            log.debug("[log] writing log done!")
-
         except Exception as  exx:
             log.exception("[log] error writing log message: %r" % exx)
             self.session.rollback()
             raise exx
 
-        finally:
-            log.debug("[log] writing log done!")
-
         return
 
     def log_entry(self, param):
@@ -424,7 +425,8 @@ class Audit(AuditBase):
                     linotp_server=param.get('linotp_server'),
                     client=param.get('client'),
                     log_level=param.get('log_level'),
-                    clearance_level=param.get('clearance_level')
+                    clearance_level=param.get('clearance_level'),
+                    config_param=self.config,
             )
 
         self.session.add(at)
@@ -434,6 +436,7 @@ class Audit(AuditBase):
         self.session.merge(at)
         self.session.flush()
 
+
     def initialize_log(self, param):
         '''
         This method initialized the log state.
@@ -449,6 +452,7 @@ class Audit(AuditBase):
         '''
         pass
 
+
     def _buildCondition(self, param, AND):
         '''
         create the sqlalchemy condition from the params
@@ -458,9 +462,6 @@ class Audit(AuditBase):
         if not AND:
             boolCheck = or_
 
-        log.debug("[_buildCondition] building condition for params %s with %s"
-                  % (param, AND))
-
         for k, v in param.items():
             if "" != v:
                 if "serial" == k:
@@ -494,7 +495,6 @@ class Audit(AuditBase):
         if conditions:
             all_conditions = boolCheck(*conditions)
 
-        log.debug("[_buildCondition] return %s" % all_conditions)
         return all_conditions
 
     def row2dict(self, audit_line):
@@ -510,14 +510,13 @@ class Audit(AuditBase):
         ## if we have an \uencoded data, we extract the unicode back
         for key, value in line.items():
             if value and type(value) in [str, unicode]:
-                value = linotp.lib.crypt.udecode(value)
+                value = linotp.lib.crypto.udecode(value)
                 line[key] = value
             elif value is None:
                 line[key] = ''
 
         # Signature check
         # TODO: use instead the verify_init
-        log.debug("[search] old sig = %s" % audit_line.signature)
 
         res = self._verify(line, audit_line.signature)
         if res == 1:
@@ -525,6 +524,7 @@ class Audit(AuditBase):
         else:
             line['sig_check'] = "FAIL"
 
+
         return line
 
     def searchQuery(self, param, AND=True, display_error=True, rp_dict=None):
@@ -546,13 +546,8 @@ class Audit(AuditBase):
             if "true" == param['or'].lower():
                 AND = False
 
-        log.debug("[search] got the params %s" % param)
-        log.debug("[search] got the rp_dict %s" % rp_dict)
-
         # build the condition / WHERE clause
         condition = self._buildCondition(param, AND)
-        log.debug("[search] the following condition was built from the "
-                  "parameters %s (%s): %s" % (param, AND, condition))
 
         order = AuditTable.id
         if rp_dict.get("sortname"):
@@ -630,22 +625,21 @@ class Audit(AuditBase):
         result = self.session.execute(audit_q.statement)
         return result
 
+
+
     def getTotal(self, param, AND=True, display_error=True):
         '''
         This method returns the total number of audit entries in
         the audit store
         '''
         condition = self._buildCondition(param, AND)
-        log.debug("[getTotal] condition: %s" % condition)
         if type(condition).__name__ == 'NoneType':
             c = self.session.query(AuditTable).count()
         else:
             c = self.session.query(AuditTable).filter(condition).count()
 
-        log.debug("[getTotal] count=%s " % str(c))
         return c
 
-
 def getAsString(data):
     '''
     We need to distinguish, if this is an entry after the adding the
