@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 #
 #    LinOTP - the open source solution for two factor authentication
-#    Copyright (C) 2010 - 2016 LSE Leading Security Experts GmbH
+#    Copyright (C) 2010 - 2019 KeyIdentity GmbH
 #
 #    This file is part of LinOTP server.
 #
@@ -19,9 +19,9 @@
 #    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #
 #
-#    E-mail: linotp@lsexperts.de
+#    E-mail: linotp@keyidentity.com
 #    Contact: www.linotp.org
-#    Support: www.lsexperts.de
+#    Support: www.keyidentity.com
 #
 ''' definition of some specific error classes'''
 
@@ -53,7 +53,7 @@ class LinotpError(Exception):
         if type(self.message) in [str, unicode]:
             pstr = "ERR%d: %s"
 
-        ### if we have here unicode, we might fail with conversion error
+        # if we have here unicode, we might fail with conversion error
         try:
             res = pstr % (self.id, self.message)
         except Exception as exx:
@@ -110,6 +110,38 @@ class TokenTypeNotSupportedError(LinotpError):
     def __init__(self, description="this token type is not supported on this setup!", id=906):
         LinotpError.__init__(self, description=description, id=id)
 
+class ProgrammingError (Exception):
+    pass
+
+class InvalidFunctionParameter (Exception):
+
+    """
+    used to signify an invalid function parameter
+
+    Example:
+    >>> def foo(bar):
+    >>>     raise ArgumentError('bar', 'invalid bar value')
+    """
+
+    def __init__(self, parameter_name, message):
+        self.parameter_name = parameter_name
+        self.message = message
+        Exception.__init__(self, 'Parameter %s: %s' % (parameter_name, message))
+
+
+class TokenStateError (UserError):
+
+    """
+    raised by StatefulTokenMixin, if a stateful token got
+    a request that doesn't fit its internal rollout state
+
+    The exception will be mapped into the generic message
+    'Unfitting request for this token' when it gets
+    transported to the API.
+    """
+
+    def __init__(self, message):
+        log.debug('TokenStateError occured. Message: %s' % message)
+        UserError.__init__(self, 'Unfitting request for this token')
 
 #eof###########################################################################
-
