@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 #
 #    LinOTP - the open source solution for two factor authentication
-#    Copyright (C) 2010 - 2016 LSE Leading Security Experts GmbH
+#    Copyright (C) 2010 - 2019 KeyIdentity GmbH
 #
 #    This file is part of LinOTP server.
 #
@@ -19,32 +19,24 @@
 #    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #
 #
-#    E-mail: linotp@lsexperts.de
+#    E-mail: linotp@keyidentity.com
 #    Contact: www.linotp.org
-#    Support: www.lsexperts.de
+#    Support: www.keyidentity.com
 #
 """Helper functions
 
 Consists of functions to typically be used within templates, but also
 available to Controllers. This module is available to templates as 'h'.
-"""
-# Import helpers as desired, or define your own, ie:
-#from webhelpers.html.tags import checkbox, password
 
+
+Import helpers as desired, or define your own, ie:
+   from webhelpers.html.tags import checkbox, password
+
+depending on the version there are two different possible locations...
+see:
+http://stackoverflow.com/questions/2219316/pylons-webhelpers-missing-secure-form-module
+
+"""
 import logging
 
-
-#CKO: depending on the version there are two different possible locations...
-# see http://stackoverflow.com/questions/2219316/pylons-webhelpers-missing-secure-form-module
-from linotp.lib.error import ParameterError
-
-
 log = logging.getLogger(__name__)
-
-def getParam(param, which, optional):
-    if param.has_key(which):
-        return  param[which]
-    else:
-        if (optional == False):
-            raise ParameterError("Missing parameter: %r" % which, id=905)
-    return
