@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 #
 #    LinOTP - the open source solution for two factor authentication
-#    Copyright (C) 2010 - 2016 LSE Leading Security Experts GmbH
+#    Copyright (C) 2010 - 2019 KeyIdentity GmbH
 #
 #    This file is part of LinOTP server.
 #
@@ -19,9 +19,9 @@
 #    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #
 #
-#    E-mail: linotp@lsexperts.de
+#    E-mail: linotp@keyidentity.com
 #    Contact: www.linotp.org
-#    Support: www.lsexperts.de
+#    Support: www.keyidentity.com
 #
 """create responses"""
 
@@ -37,9 +37,12 @@ except ImportError:
 from pylons import request
 from pylons import tmpl_context as c
 
-from linotp.lib.error       import LinotpError
-from linotp.lib.util        import get_version
+from linotp.lib.error import LinotpError
+from linotp.lib.util import get_version
+from linotp.lib.util import get_api_version
 
+from linotp.lib.context import request_context_safety
+from linotp.lib.context import request_context
 
 optional = True
 required = False
@@ -191,7 +194,7 @@ def sendError(response, exception, id=1, context=None):
 
     ## handle the different types of exception:
     ## Exception, LinOtpError, str/unicode
-    if hasattr(exception, '__class__') == True \
+    if hasattr(exception, '__class__') is True \
     and isinstance(exception, Exception):
         errDesc = unicode(exception)
         if isinstance(exception, LinotpError):
@@ -243,7 +246,7 @@ def sendError(response, exception, id=1, context=None):
     else:
         # Send JSON response with HTTP status 200 OK
         response.content_type = 'application/json'
-        res = { "jsonrpc": "2.0",
+        res = { "jsonrpc": get_api_version(),
                 "result" :
                     {"status": False,
                         "error": {
@@ -265,7 +268,7 @@ def sendError(response, exception, id=1, context=None):
     return ret
 
 
-def sendResult(response, obj, id=1, opt=None):
+def sendResult(response, obj, id=1, opt=None, status=True):
     '''
         sendResult - return an json result document
 
@@ -285,8 +288,8 @@ def sendResult(response, obj, id=1, opt=None):
 
     response.content_type = 'application/json'
 
-    res = { "jsonrpc": "2.0",
-            "result": { "status": True,
+    res = { "jsonrpc": get_api_version(),
+            "result": { "status": status,
                         "value": obj,
                       },
            "version": get_version(),
@@ -298,9 +301,11 @@ def sendResult(response, obj, id=1, opt=None):
     return json.dumps(res, indent=3)
 
 
-def sendResultIterator(obj, id=1, opt=None, rp=None, page=None):
+def sendResultIterator(obj, id=1, opt=None, rp=None, page=None,
+                       request_context_copy=None):
     '''
         sendResultIterator - return an json result document in a streamed mode
+                             which requires a request context to be avaliable
 
         :param obj: iterator of generator object like dict, string or list
         :param  id: id value, for future versions
@@ -310,90 +315,105 @@ def sendResultIterator(obj, id=1, opt=None, rp=None, page=None):
 
         :return: generator of response data (yield)
     '''
-    res = {"jsonrpc": "2.0",
-            "result": {"status": True,
-                       "value": "[DATA]",
-                      },
-           "version": get_version(),
-           "id": id}
 
-    err = {"jsonrpc": "2.0",
-            "result":
-                {"status": False,
-                 "error": {},
-                },
-            "version": get_version(),
-            "id": id
-        }
+    if request_context_copy is None:
+        request_context_copy = {}
 
-    start_at = 0
-    stop_at = 0
-    if page:
-        if not rp:
-            rp = 16
-        try:
-            start_at = int(page) * int(rp)
-            stop_at = start_at + int(rp)
-        except ValueError as exx:
-            err['result']['error'] = {
-                            "code": 9876,
-                            "message": "%r" % exx,
-                            }
-            log.exception("failed to convert paging request parameters: %r"
-                          % exx)
-            yield json.dumps(err)
-            # finally we signal end of error result
-            raise StopIteration()
+    # establish the request context within the pylons middleware
 
-    typ = "%s" % type(obj)
-    if 'generator' not in typ and 'iterator' not in typ:
-        raise Exception('no iterator method for object %r' % obj)
+    with request_context_safety():
 
-    res = {"jsonrpc": "2.0",
-            "result": {"status": True,
-                       "value": "[DATA]",
-                      },
-           "version": get_version(),
-           "id": id}
-    if page:
-        res['result']['page'] = int(page)
+        for key, value in request_context_copy.items():
+            request_context[key] = value
 
-    if opt is not None and len(opt) > 0:
-        res["detail"] = opt
+        api_version = get_api_version()
+        linotp_version = get_version()
 
-    surrounding = json.dumps(res)
-    prefix, postfix = surrounding.split('"[DATA]"')
+        res = {"jsonrpc": api_version,
+                "result": {"status": True,
+                           "value": "[DATA]",
+                          },
+               "version": linotp_version,
+               "id": id}
 
-    # first return the opening
-    yield prefix + " ["
+        err = {"jsonrpc": api_version,
+                "result":
+                    {"status": False,
+                     "error": {},
+                    },
+                "version": linotp_version,
+                "id": id
+            }
 
-    sep = ""
-    counter = 0
-    for next_one in obj:
-        counter = counter + 1
-        # are we running in paging mode?
+
+        start_at = 0
+        stop_at = 0
         if page:
-            if counter >= start_at and counter < stop_at:
+            if not rp:
+                rp = 16
+            try:
+                start_at = int(page) * int(rp)
+                stop_at = start_at + int(rp)
+            except ValueError as exx:
+                err['result']['error'] = {
+                                "code": 9876,
+                                "message": "%r" % exx,
+                                }
+                log.exception("failed to convert paging request parameters: %r"
+                              % exx)
+                yield json.dumps(err)
+                # finally we signal end of error result
+                raise StopIteration()
+
+        typ = "%s" % type(obj)
+        if 'generator' not in typ and 'iterator' not in typ:
+            raise Exception('no iterator method for object %r' % obj)
+
+        res = {"jsonrpc": api_version,
+                "result": {"status": True,
+                           "value": "[DATA]",
+                          },
+               "version": linotp_version,
+               "id": id}
+        if page:
+            res['result']['page'] = int(page)
+
+        if opt is not None and len(opt) > 0:
+            res["detail"] = opt
+
+
+        surrounding = json.dumps(res)
+        prefix, postfix = surrounding.split('"[DATA]"')
+
+        # first return the opening
+        yield prefix + " ["
+
+
+        sep = ""
+        counter = 0
+        for next_one in obj:
+            counter = counter + 1
+            # are we running in paging mode?
+            if page:
+                if counter >= start_at and counter < stop_at:
+                    res = "%s%s\n" % (sep, next_one)
+                    sep = ','
+                    yield res
+                if counter >= stop_at:
+                    # stop iterating if we reached the last one of the page
+                    break
+            else:
+                # no paging - no limit
                 res = "%s%s\n" % (sep, next_one)
                 sep = ','
                 yield res
-            if counter >= stop_at:
-                # stop iterating if we reached the last one of the page
-                break
-        else:
-            # no paging - no limit
-            res = "%s%s\n" % (sep, next_one)
-            sep = ','
-            yield res
 
-    log.debug('Result iteration finished!')
+        # we add the amount of queried objects
+        total = '"queried" : %d' % counter
+        postfix = ', %s %s' % (total, postfix)
 
-    # we add the amount of queried objects
-    total = '"queried" : %d' % counter
-    postfix = ', %s %s' % (total, postfix)
-
-    # last return the closing
-    yield "] " + postfix
+        # last return the closing
+        yield "] " + postfix
 
 
 def sendCSVResult(response, obj, flat_lines=False,
@@ -404,13 +424,16 @@ def sendCSVResult(response, obj, flat_lines=False,
     :param response: The pylons response object
     :param obj: The data, that gets serialized as CSV
     :type obj: JSON object
-    :param flat_lines: If True the object only contains a list of the dict { 'cell': ..., 'id': ... }
-                        as in all the flexigrid functions.
+    :param flat_lines: If True the object only contains a list of the
+                         dict { 'cell': ..., 'id': ... }
+                       as in all the flexigrid functions.
     'type flat_lines: boolean
     '''
     delim = "'"
+    seperator = ';'
     response.content_type = "application/force-download"
-    response.headers['Content-disposition'] = 'attachment; filename=%s' % filename
+    response.headers['Content-disposition'] = ('attachment; filename=%s'
+                                               % filename)
     output = u""
 
     if not flat_lines:
@@ -422,7 +445,7 @@ def sendCSVResult(response, obj, flat_lines=False,
             # Do the header
             if not headers_printed:
                 for k in data[0].keys():
-                    output += "%s%s%s, " % (delim, k, delim)
+                    output += "%s%s%s%s " % (delim, k, delim, seperator)
                 output += "\n"
                 headers_printed = True
 
@@ -431,28 +454,71 @@ def sendCSVResult(response, obj, flat_lines=False,
                     value = val.replace("\n", " ")
                 else:
                     value = val
-                output += "%s%s%s, " % (delim, value, delim)
+                output += "%s%s%s%s " % (delim, value, delim, seperator)
             output += "\n"
     else:
         for l in obj:
             for elem in l.get("cell", []):
-                output += "'%s', " % elem
+                output += "'%s'%s " % (elem, seperator)
 
             output += "\n"
 
     return output
 
-def sendXMLResult(response, obj, id=1):
+
+def json2xml(json_obj, line_padding=""):
+    result_list = list()
+
+    json_obj_type = type(json_obj)
+
+    if json_obj_type is list:
+        for sub_elem in json_obj:
+            result_list.append("%s<value>" % line_padding)
+            result_list.append(json2xml(sub_elem, line_padding))
+            result_list.append("%s</value>" % line_padding)
+
+        return "".join(result_list)
+
+    if json_obj_type is dict:
+        for tag_name in json_obj:
+            sub_obj = json_obj[tag_name]
+            result_list.append("%s<%s>" % (line_padding, tag_name))
+            result_list.append(json2xml(sub_obj, "" + line_padding))
+            result_list.append("%s</%s>" % (line_padding, tag_name))
+
+        return "".join(result_list)
+
+    return "%s%s" % (line_padding, json_obj)
+
+
+def sendXMLResult(response, obj, id=1, opt=None):
+    """
+    send the result as an xml format
+    """
     response.content_type = 'text/xml'
     res = '<?xml version="1.0" encoding="UTF-8"?>\
-            <jsonrpc version="2.0">\
+            <jsonrpc version="%s">\
             <result>\
                 <status>True</status>\
                 <value>%s</value>\
             </result>\
             <version>%s</version>\
             <id>%s</id>\
-            </jsonrpc>' % (obj, get_version(), id)
+            </jsonrpc>' % (get_api_version(), obj, get_version(), id)
+    xml_options = ""
+    if opt:
+        xml_options = "\n<options>" + json2xml(opt) + "</options>"
+    xml_object = json2xml(obj)
+
+    res = """<?xml version="1.0" encoding="UTF-8"?>
+<jsonrpc version="2.0">
+    <result>
+        <status>True</status>
+        <value>%s</value>
+    </result>
+    <version>%s</version>
+    <id>%s</id>%s
+</jsonrpc>""" % (xml_object, get_version(), id, xml_options)
     return res
 
 
@@ -465,7 +531,7 @@ def sendXMLError(response, exception, id=1):
         errId = exception.getId()
         errDesc = exception.getDescription()
     res = '<?xml version="1.0" encoding="UTF-8"?>\
-            <jsonrpc version="2.0">\
+            <jsonrpc version="%s">\
             <result>\
                 <status>False</status>\
                 <error>\
@@ -475,7 +541,7 @@ def sendXMLError(response, exception, id=1):
             </result>\
             <version>%s</version>\
             <id>%s</id>\
-            </jsonrpc>' % (errId, errDesc, get_version(), id)
+            </jsonrpc>' % (get_api_version(), errId, errDesc, get_version(), id)
     return res
 
 
@@ -491,7 +557,6 @@ def sendQRImageResult(response, data, param=None, id=1, typ='html'):
         html     - print qrcode wrapped by html or not
 
     '''
-    log.debug('[sendQRImageResult]')
 
     width = 0
     alt = None
@@ -529,7 +594,6 @@ def sendQRImageResult(response, data, param=None, id=1, typ='html'):
         response.content_type = 'text/html'
         ret = create_html(img_data, width, param)
 
-    log.debug('[sendQRImageResult]:')
     return ret
 
 
@@ -548,7 +612,27 @@ def create_png(data, alt=None):
     return o_data
 
 
-def create_img(data, width=0, alt=None):
+def create_img_src(data):
+    '''
+        _create_img - create the qr image data
+
+        :param data: input data that will be munched into the qrcode
+        :type  data: string
+        :param width: image width in pixel
+        :type  width: int
+
+        :return: <img/> taged data
+        :rtype:  string
+    '''
+
+    o_data = create_png(data)
+    data_uri = o_data.encode("base64").replace("\n", "")
+    ret_img_src = 'data:image/png;base64,%s' % data_uri
+
+    return ret_img_src
+
+
+def create_img(data, width=0, alt=None, img_id="challenge_qrcode"):
     '''
         _create_img - create the qr image data
 
@@ -563,22 +647,22 @@ def create_img(data, width=0, alt=None):
     width_str = ''
     alt_str = ''
 
-    o_data = create_png(data, alt=alt)
-    data_uri = o_data.encode("base64").replace("\n", "")
+    img_src = create_img_src(data)
 
     if width != 0:
         width_str = " width=%d " % (int(width))
 
     if alt is not None:
-        val = urllib.urlencode({'alt':alt})
+        val = urllib.urlencode({'alt': alt})
         alt_str = " alt=%r " % (val[len('alt='):])
 
-    ret_img = '<img %s  %s  src="data:image/png;base64,%s"/>' % (alt_str, width_str, data_uri)
+    ret_img = ('<img id="%s" %s  %s  src="%s"/>' %
+               (img_id, alt_str, width_str, img_src))
 
     return ret_img
 
 
-def create_html(data, width=0, alt=None):
+def create_html(data, width=0, alt=None, list_id="challenge_data"):
     '''
         _create_html - create the qr image data embeded in html tag
 
@@ -591,7 +675,7 @@ def create_html(data, width=0, alt=None):
         :rtype:  string
     '''
     alt_str = ''
-    img = create_img(data, width=width, alt=alt)
+    img = create_img(data, width=width, alt=data)
 
     if alt is not None:
         if type(alt) in (str, u''):
@@ -599,18 +683,55 @@ def create_html(data, width=0, alt=None):
         elif type(alt) == dict:
             alta = []
             for k in alt.keys():
-                alta.append('<li> %s:%s </li>' % (k, alt.get(k)))
-            alt_str = '<ul>%s</ul>' % " ".join(alta)
+                alta.append('<li> %s: <span class="%s">%s</span> </li>' % (k, k, alt.get(k)))
+            alt_str = '<ul id="%s">%s</ul>' % ( list_id, " ".join(alta))
         elif type(alt) == list:
             alta = []
             for k in alt:
                 alta.append('<li> %s </li>' % (k))
 
 
-    ret_html = '<html><body>%s%s</body></html>' % (img , alt_str)
+    ret_html = '<html><body><div>%s%s</div></body></html>' % (img , alt_str)
 
     return ret_html
 
 
+def sendCSVIterator(obj, headers=True):
+    delim = '"'
+    output = ""
+
+    typ = "%s" % type(obj)
+    if 'generator' not in typ and 'iterator' not in typ:
+        raise Exception('no iterator method for object %r' % obj)
+
+    try:
+        for row in obj:
+            row = json.loads(row)
+            # do the header
+            if headers:
+                for key in row:
+                    output += "%s%s%s," % (delim, key, delim)
+                output += "\n"
+                yield str(output)
+                headers = False
+
+            output = ""
+            for val in row.values():
+                if type(val) in [str, unicode]:
+                    value = val.replace("\n", " ")
+                    output += "%s%s%s, " % (delim, value, delim)
+                elif type(val) in [int, long]:
+                    value = '%d' % val
+                    output += "%s, " % (value)
+                else:
+                    output += "%s%s%s, " % (delim, value, delim)
+                # output += "%s%s%s, " % (delim, value, delim)
+            output += "\n"
+            yield str(output)
+
+    except Exception as exx:
+        log.debug('error when iterating result for csv output')
+        raise exx
+
 #eof#######################################################
 
