@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 #
 #    LinOTP - the open source solution for two factor authentication
-#    Copyright (C) 2010 - 2016 LSE Leading Security Experts GmbH
+#    Copyright (C) 2010 - 2019 KeyIdentity GmbH
 #
 #    This file is part of LinOTP server.
 #
@@ -19,55 +19,51 @@
 #    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #
 #
-#    E-mail: linotp@lsexperts.de
+#    E-mail: linotp@keyidentity.com
 #    Contact: www.linotp.org
-#    Support: www.lsexperts.de
+#    Support: www.keyidentity.com
 #
 """module for SecurityModules / devices like hsms"""
 import logging
 log = logging.getLogger(__name__)
 
 
+class FatalHSMException(Exception):
+    pass
+
 
 class SecurityModule(object):
 
-    def __init__(self, config=None):
-        log.error("[__init__] This is the base class. You should implement this!")
+    @classmethod
+    def getAdditionalClassConfig(cls):
+        return []
+
+    def __init__(self, config=None, add_conf=None):
         self.name = "SecurityModule"
 
     def isReady(self):
         fname = 'isReady'
-        log.error("[%s] This is the base class. You should implement "
-                  "the method : %s " % (self.name, fname))
         raise NotImplementedError("Should have been implemented %s"
                                    % fname)
 
     def setup_module(self, params):
         fname = 'setup_module'
-        log.error("[%r] This is the base class. You should implement "
-                  "the method : %s " % (self.name, fname))
         raise NotImplementedError("Should have been implemented %s"
                                    % fname)
 
     ''' base methods '''
     def random(self, len):
         fname = 'random'
-        log.error("[%s] This is the base class. You should implement "
-                  "the method : %s " % (self.name, fname))
         raise NotImplementedError("Should have been implemented %s"
                                    % fname)
 
-    def encrypt(self, value, iv=None):
+    def encrypt(self, value, iv=None, id=0):
         fname = 'encrypt'
-        log.error("[%s] This is the base class. You should implement "
-                  "the method : %s " % (self.name, fname))
         raise NotImplementedError("Should have been implemented %s"
                                    % fname)
 
-    def decrypt(self, value, iv=None):
+    def decrypt(self, value, iv=None, id=0):
         fname = 'decrypt'
-        log.error("[%s] This is the base class. You should implement "
-                  "the method : %s " % (self.name, fname))
         raise NotImplementedError("Should have been implemented %s"
                                    % fname)
 
@@ -75,33 +71,34 @@ class SecurityModule(object):
     ''' higer level methods '''
     def encryptPassword(self, cryptPass):
         fname = 'decrypt'
-        log.error("[%s] This is the base class. You should implement "
-                  "the method : %s " % (self.name, fname))
         raise NotImplementedError("Should have been implemented %s"
                                    % fname)
 
-    def encryptPin(self, cryptPin):
+    def encryptPin(self, cryptPin, iv=None):
         fname = 'decrypt'
-        log.error("[%s] This is the base class. You should implement "
-                  "the method : %s " % (self.name, fname))
         raise NotImplementedError("Should have been implemented %s"
                                    % fname)
 
-
     def decryptPassword(self, cryptPass):
         fname = 'decrypt'
-        log.error("[%s] This is the base class. You should implement "
-                  "the method : %s " % (self.name, fname))
         raise NotImplementedError("Should have been implemented %s"
-                                   % fname)
+                                  % fname)
 
     def decryptPin(self, cryptPin):
         fname = 'decrypt'
-        log.error("[%s] This is the base class. You should implement "
-                  "the method : %s " % (self.name, fname))
         raise NotImplementedError("Should have been implemented %s"
-                                   % fname)
+                                  % fname)
 
+    def signMessage(self, message, method=None, slot_id=3):
+        fname = 'signMessage'
+        raise NotImplementedError("Should have been implemented %s"
+                                  % fname)
 
-#eof###########################################################################
+    def verfiyMessageSignature(self, message, hex_mac, method=None,
+                               slot_id=3):
 
+        fname = 'verfiyMessageSignature'
+        raise NotImplementedError("Should have been implemented %s"
+                                  % fname)
+
+# eof ########################################################################
