@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 #
 #    LinOTP - the open source solution for two factor authentication
-#    Copyright (C) 2010 - 2016 LSE Leading Security Experts GmbH
+#    Copyright (C) 2010 - 2019 KeyIdentity GmbH
 #
 #    This file is part of LinOTP server.
 #
@@ -19,9 +19,9 @@
 #    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #
 #
-#    E-mail: linotp@lsexperts.de
+#    E-mail: linotp@keyidentity.com
 #    Contact: www.linotp.org
-#    Support: www.lsexperts.de
+#    Support: www.keyidentity.com
 #
 """
 The security provider is a dynamic handler for security relevant tasks like
@@ -32,8 +32,10 @@ import thread
 import time
 import logging
 
-from linotp.lib.crypt import zerome
+from linotp.lib.crypto import zerome
 from linotp.lib.error import HSMException
+from linotp.lib.security import FatalHSMException
+
 
 DEFAULT_KEY = 0
 CONFIG_KEY = 1
@@ -43,9 +45,10 @@ VALUE_KEY = 3
 
 log = logging.getLogger(__name__)
 
+
 class SecurityProvider(object):
     '''
-    the Security provider is the singleton in the server who provides
+    the security provider is the singleton in the server who provides
     the security modules to run security relevant methods
 
     - read the hsm configurations
@@ -53,11 +56,11 @@ class SecurityProvider(object):
     - bind a hsm to one session
     - free the hsm from session after usage
 
-    as session identifier the thread id is used
+    the thread id is used as session identifier
     '''
     def __init__(self, secLock):
         '''
-        setup the SecurityProvider, which is called on server startup
+        setup the security provider, which is called on server startup
         from the app_globals init
 
         :param secLock: RWLock() to support server wide locking
@@ -80,41 +83,58 @@ class SecurityProvider(object):
         :param config:
 
         '''
-        keyFile = '/home/work/project/pylons/linotp2/v2.4old/dev/linotpd/src/secKey'
-        if config.has_key('linotpSecretFile'):
-            keyFile = config.get('linotpSecretFile')
-        self.config['default'] = {'pinHandle'   : TOKEN_KEY,
-                                  'passHandle'  : CONFIG_KEY,
-                                  'valueHandle' : VALUE_KEY,
-                                  'defaultHandle' : DEFAULT_KEY,
-                                  'crypted'     : 'FALSE',
-                                  'file'        : keyFile,
-                                  'module'      : 'linotp.lib.security.default.DefaultSecurityModule',
-                                  'poolsize'    : 20,
-                                  }
+        provider_config = {}
 
-        self.config['err'] = {'pinHandle'   : TOKEN_KEY,
-                                  'passHandle'  : CONFIG_KEY,
-                                  'valueHandle' : VALUE_KEY,
-                                  'defaultHandle' : DEFAULT_KEY,
-                                  'crypted'     : 'FALSE',
-                                  'file'        : keyFile,
-                                  'module'      : 'linotp.lib.security.default.ErrSecurityModule',
-                                  'poolsize'    : 20,
-                                  }
+        keyFile = config['linotpSecretFile']
+        provider_config['default'] = {
+            'pinHandle': TOKEN_KEY,
+            'passHandle': CONFIG_KEY,
+            'valueHandle': VALUE_KEY,
+            'defaultHandle': DEFAULT_KEY,
+            'crypted': 'FALSE',
+            'file': keyFile,
+            'module': 'linotp.lib.security.default.DefaultSecurityModule',
+            'poolsize': 20, }
+
+        provider_config['err'] = {
+            'pinHandle': TOKEN_KEY,
+            'passHandle': CONFIG_KEY,
+            'valueHandle': VALUE_KEY,
+            'defaultHandle': DEFAULT_KEY,
+            'crypted': 'FALSE',
+            'file': keyFile,
+            'module': 'linotp.lib.security.default.ErrSecurityModule',
+            'poolsize': 20, }
+
+        for key, value in config.items():
+            for provider in provider_config.keys():
+                if key.startswith('linotpSecurity.%s' % provider):
+                    entry = key.split('.')[-1]
+                    provider_config[provider][entry] = value
+
+        return provider_config
 
     def load_config(self, config):
         '''
         load the security modules configuration
         '''
+        p_config = {}
+        p_config.update(config)
 
         try:
-            ## load backward compatible defaults
-            self. __createDefault__(config)
+            # load backward compatible defaults
+            default_config = self.__createDefault__(config)
+            self.config.update(default_config)
+
+            if 'linotpActiveSecurityModule' in config:
+                # look the active security module up
+                self.activeOne = config['linotpActiveSecurityModule']
+                log.debug("[SecurityProvider:load_config] setting active"
+                          " security module: %s", self.activeOne)
 
             for key in config:
 
-                ## lookup, which is the active security module
+                ## look the active security module up
                 if key == 'linotpActiveSecurityModule':
                     self.activeOne = config.get(key)
                     log.debug("[SecurityProvider:load_config] setting active security module: %s" % self.activeOne)
@@ -129,7 +149,7 @@ class SecurityProvider(object):
                         log.exception(error)
                         raise HSMException(error, id=707)
 
-                    if self.config.has_key(id):
+                    if id in self.config:
                         id_config = self.config.get(id)
                         id_config[val] = config.get(key)
                     else:
@@ -140,7 +160,7 @@ class SecurityProvider(object):
             error = "failed to identify module: %s " % unicode(e)
             raise HSMException(error, id=707)
 
-        ## now create for each module a pool of hsm objects
+        ## now create a pool of hsm objects for each module
         self.rwLock.acquire_write()
         try:
             for id in self.config:
@@ -169,11 +189,11 @@ class SecurityProvider(object):
 
         log.debug("[loadSecurityModule] Loading module %s" % id)
 
-        if self.config.has_key(id) == False:
+        if id not in self.config:
             return ret
 
         config = self.config.get(id)
-        if config.has_key('module') == False:
+        if 'module' not in config:
             return ret
 
         module = config.get('module')
@@ -186,25 +206,42 @@ class SecurityProvider(object):
 
         mod = __import__(packageName, globals(), locals(), [className])
         klass = getattr(mod, className)
+        config_name = klass.getAdditionalClassConfig()
+        additional_config = self.get_config_entries(config_name)
 
         for method in methods:
-            if hasattr(klass, method) == False:
+            if hasattr(klass, method) is False:
                 error = ("[loadSecurityModule] Security Module %s misses the "
                          "following interface: %s" % (unicode(module), unicode(method)))
                 log.error(error)
                 raise NameError(error)
 
-        ret = klass(config)
+        ret = klass(config, add_conf=additional_config)
         self.security_modules[id] = ret
 
         log.debug("[loadSecurityModule] returning %r" % ret)
 
         return ret
 
+    def get_config_entries(self, config_name):
+        """
+        :param names: list of config entries by modulename
+        :return: dict
+        """
+        merged_config = {}
+
+        for provider, provider_config in self.config.items():
+
+            module = provider_config.get('module')
+            provider_class = module.split('.')[-1]
+            if provider_class in config_name:
+                merged_config = self.config[provider]
+
+        return merged_config
 
     def _getHsmPool_(self, hsm_id):
         ret = None
-        if self.hsmpool.has_key(hsm_id):
+        if hsm_id in self.hsmpool:
             ret = self.hsmpool.get(hsm_id)
         return ret
 
@@ -212,7 +249,7 @@ class SecurityProvider(object):
     def setupModule(self, hsm_id, config=None):
         '''
         setupModule is called during runtime to define
-        the config parameters like passw or connection strings
+        the config parameters like password or connection strings
         '''
         self.rwLock.acquire_write()
         try:
@@ -239,41 +276,51 @@ class SecurityProvider(object):
 
     def createHSMPool(self, hsm_id=None, *args, **kw):
         '''
-        setup a pool of secutity provider
+        setup a pool of security providers
         '''
         pool = None
         ## amount has to be taken from the hsm-id config
-        if hsm_id == None:
-            ids = self.config
+        if hsm_id is None:
+            provider_ids = self.config
         else:
-            if self.config.has_key(hsm_id):
-                ids = []
-                ids.append(hsm_id)
+            if hsm_id in self.config:
+                provider_ids = []
+                provider_ids.append(hsm_id)
             else:
-                error = "[createHSMPool] failed to find hsm_id: %s" % (unicode(hsm_id))
+                error = "[createHSMPool] failed to find hsm_id: %r" % hsm_id
                 log.error(error)
                 raise HSMException(error, id=707)
 
-        for id in ids:
-            pool = self._getHsmPool_(id)
+        for provider_id in provider_ids:
+            pool = self._getHsmPool_(provider_id)
             log.debug("[createHSMPool] already got this pool: %r" % pool)
             if pool is None:
                 ## get the number of entries from the hsd (id) config
-                conf = self.config.get(id)
-                amount = int(conf.get('poolsize', 10))
-                log.debug("[createHSMPool] creating pool for %r with size %r" % (id, amount))
+                conf = self.config.get(provider_id)
+                poolsize = int(conf.get('poolsize', 10))
+                log.debug("[createHSMPool] creating pool for %r with size %r",
+                          provider_id, poolsize)
+
                 pool = []
-                for i in range(0, amount):
+                for _i in range(0, poolsize):
                     error = ''
                     hsm = None
                     try:
-                        hsm = self.loadSecurityModule(id)
-                    except Exception as e:
-                        error = u"%r" % e
-                        log.exception("[createHSMPool] %r " % (e))
-                    pool.append({'obj': hsm , 'session': 0, 'error':error})
+                        hsm = self.loadSecurityModule(provider_id)
+                    except FatalHSMException as exx:
+                        log.exception("[createHSMPool] %r %r ",
+                                      provider_id, exx)
+                        if provider_id == self.activeOne:
+                            raise exx
+                        error = u"%r: %r" % (provider_id, exx)
 
-                self.hsmpool[id] = pool
+                    except Exception as exx:
+                        log.exception("[createHSMPool] %r ", exx)
+                        error = u"%r: %r" % (provider_id, exx)
+
+                    pool.append({'obj': hsm, 'session': 0, 'error': error})
+
+                self.hsmpool[provider_id] = pool
         return pool
 
     def _findHSM4Session(self, pool, sessionId):
@@ -311,7 +358,7 @@ class SecurityProvider(object):
         if sessionId is None:
             sessionId = unicode(thread.get_ident())
 
-        if self.config.has_key(hsm_id) == False:
+        if hsm_id not in self.config:
             error = ('[SecurityProvider:dropSecurityModule] no config found '
                      'for hsm with id %s ' % (unicode(hsm_id)))
             log.error(error)
@@ -339,7 +386,7 @@ class SecurityProvider(object):
         if sessionId is None:
             sessionId = unicode(thread.get_ident())
 
-        if self.config.has_key(hsm_id) == False:
+        if hsm_id not in self.config:
             error = ('[SecurityProvider:getSecurityModule] no config found for '
                      'hsm with id %s ' % (unicode(hsm_id)))
             log.error(error)
@@ -349,7 +396,7 @@ class SecurityProvider(object):
         tries = 0
         locked = False
 
-        while retry == True:
+        while retry is True:
             try:
                 pool = self._getHsmPool_(hsm_id)
                 self.rwLock.acquire_write()
@@ -380,7 +427,7 @@ class SecurityProvider(object):
                             error = ('[SecurityProvider:getSecurityModule] '
                                      'max_retry %d: could not bind hsm to '
                                      'session  - going to sleep for  %r'
-                                     % tries, 10 * tries)
+                                     % (tries, 10 * tries))
                             log.error(error)
                             raise Exception(error)
                         retry = True
@@ -388,20 +435,29 @@ class SecurityProvider(object):
                         retry = False
 
             finally:
-                if locked == True:
+                if locked is True:
                     self.rwLock.release()
 
         return found
-        #return self.loadSecurityModule(id)
+
 
 def main():
-    ## hook for local provider test
-    sep = SecurityProvider()
+
+    class DummySecLock():
+
+        def release(self):
+            return
+
+        def acquire_write(self):
+            return
+
+    # hook for local provider test
+    sep = SecurityProvider(secLock=DummySecLock())
     sep.load_config({})
     sep.createHSMPool('default')
-    sep.setupModule('default', {'passwd' : 'test123'})
+    sep.setupModule('default', {'passwd': 'test123'})
 
-    ## runtime catch an hsm for session
+    # runtime catch an hsm for session
     hsm = sep.getSecurityModule()
 
     passwo = 'password'
@@ -418,7 +474,7 @@ def main():
 
     zerome(passw)
 
-    ## session shutdown
+    # session shutdown
     sep.dropSecurityModule(sessionId='session2')
     sep.dropSecurityModule()
 
@@ -430,4 +486,3 @@ if __name__ == '__main__':
     main()
 
 #eof###########################################################################
-
