@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 #
 #    LinOTP - the open source solution for two factor authentication
-#    Copyright (C) 2010 - 2016 LSE Leading Security Experts GmbH
+#    Copyright (C) 2010 - 2019 KeyIdentity GmbH
 #
 #    This file is part of LinOTP server.
 #
@@ -19,33 +19,36 @@
 #    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #
 #
-#    E-mail: linotp@lsexperts.de
+#    E-mail: linotp@keyidentity.com
 #    Contact: www.linotp.org
-#    Support: www.lsexperts.de
+#    Support: www.keyidentity.com
 #
-""" idetify if module runs in selftest """
+
+"""
+check if we are running in selftest mode
+"""
+
+import logging
 
 from linotp.lib.config import getFromConfig
 
-import logging
 log = logging.getLogger(__name__)
 
 
-def isSelfTest():
+def isSelfTest(config=None):
     '''
-        check if we are running in the selftest mode, which is
-        used especially for debugging / development or unit tests
+    check if we are running in the selftest mode, which is
+    used especially for debugging / development or unit tests
 
-        @return : True or False
-        @rtype  : boolean
+    :param config: config dictionary
+    :return: boolean
     '''
-    ret = False
-
-    if False != getFromConfig("selfTest", False):
-        ret = True
-
-    return ret
-
 
+    if not config:
+        selftest = getFromConfig("selfTest", False) is not False
+    else:
+        selftest = config.get("selfTest", False) is not False
 
+    return selftest
 
+# eof ########################################################################
