@@ -1,6 +1,6 @@
 /*!
  *   LinOTP - the open source solution for two factor authentication
- *   Copyright (C) 2010 - 2016 LSE Leading Security Experts GmbH
+ *   Copyright (C) 2010 - 2019 KeyIdentity GmbH
  *
  *   This file is part of LinOTP server.
  *
@@ -18,9 +18,9 @@
  *   along with this program.  If not, see <http://www.gnu.org/licenses/>.
  *
  *
- *    E-mail: linotp@lsexperts.de
+ *    E-mail: linotp@keyidentity.com
  *    Contact: www.linotp.org
- *    Support: www.lsexperts.de
+ *    Support: www.keyidentity.com
  *
  */
 function create_aladdin_dialog() {
@@ -46,9 +46,10 @@ function create_aladdin_dialog() {
 				}
         },
         open: function() {
-            translate_import_safenet();
             _fill_realms($('#safenet_realm'),1);
-            do_dialog_icons();
+
+            $(this).dialog_icons();
+            translate_import_safenet();
         }
     });
 	return $dialog_load_tokens_aladdin ;
