@@ -82,7 +82,7 @@
 			resizable: true, //allow table resizing
 			url: false, //URL if using data from AJAX
 			method: 'POST', //data sending method
-			dataType: 'xml', //type of data for AJAX, either xml or json
+			dataType: 'json', //type of data for AJAX, either xml or json
 			errormsg: 'Connection Error',
 			usepager: false,
 			nowrap: true,
@@ -109,7 +109,7 @@
 			hideOnSubmit: true,
 			autoload: true,
 			blockOpacity: 0.5,
-			preProcess: false,
+			preProcess: true,
 			addTitleToCell: false, // add a title attr to cells with truncated contents
 			dblClickResize: false, //auto resize column by double clicking
 			onDragCol: false,
@@ -153,7 +153,6 @@
 					top: g.hDiv.offsetTop + 1
 				});
 				var cdpad = this.cdpad;
-				var cdcounter=0;
 				$('div', g.cDrag).hide();
 				$('thead tr:first th:visible', this.hDiv).each(function () {
 					var n = $('thead tr:first th:visible', g.hDiv).index(this);
@@ -164,10 +163,9 @@
 						cdpos = 0;
 					}
 					$('div:eq(' + n + ')', g.cDrag).css({
-						'left': (!(browser.mozilla) ? cdpos - cdcounter : cdpos) + 'px'
+						'left':  cdpos + 'px'
 					}).show();
 					cdleft = cdpos;
-					cdcounter++;
 				});
 			},
 			fixHeight: function (newH) {
