@@ -1,6 +1,6 @@
 /*!
  *   LinOTP - the open source solution for two factor authentication
- *   Copyright (C) 2010 - 2016 LSE Leading Security Experts GmbH
+ *   Copyright (C) 2010 - 2019 KeyIdentity GmbH
  *
  *   This file is part of LinOTP server.
  *
@@ -18,20 +18,16 @@
  *   along with this program.  If not, see <http://www.gnu.org/licenses/>.
  *
  *
- *    E-mail: linotp@lsexperts.de
+ *    E-mail: linotp@keyidentity.com
  *    Contact: www.linotp.org
- *    Support: www.lsexperts.de
+ *    Support: www.keyidentity.com
  *
  */
 
- /* Use Jed for i18n. The correct JSON file is dynamically loaded later. */
-var i18n = new Jed({});
-var sprintf = Jed.sprintf;
-
 function create_oathcsv_dialog() {
  var $dialog_load_tokens_oathcsv = $('#dialog_import_oath').dialog({
         autoOpen: false,
-        title: 'OATH CSV Token File',
+        title: i18n.gettext('OATH CSV Token File'),
         width: 600,
         modal: true,
         buttons: {
@@ -41,18 +37,23 @@ function create_oathcsv_dialog() {
                 $(this).dialog('close');
                 },
                 id: "button_oathcsv_load",
-                text: i18n.gettext("Load Token File")
+                text: i18n.gettext("Load Token File"),
+                icons: {
+                    primary: 'ui-icon-folder-open'
+                }
                 },
             Cancel: {click: function(){
                 $(this).dialog('close');
                 },
                 id: "button_oathcsv_cancel",
-                text: i18n.gettext("Cancel")
-                }
+                text: i18n.gettext("Cancel"),
+                icons: {
+                    primary: 'ui-icon-cancel'
+                },
+            }
         },
         open: function() {
             _fill_realms($('#oath_realm'),1);
-            do_dialog_icons();
         }
     });
     return $dialog_load_tokens_oathcsv ;
