@@ -1,6 +1,6 @@
 /*!
  *   LinOTP - the open source solution for two factor authentication
- *   Copyright (C) 2010 - 2016 LSE Leading Security Experts GmbH
+ *   Copyright (C) 2010 - 2019 KeyIdentity GmbH
  *
  *   This file is part of LinOTP server.
  *
@@ -18,13 +18,12 @@
  *   along with this program.  If not, see <http://www.gnu.org/licenses/>.
  *
  *
- *    E-mail: linotp@lsexperts.de
+ *    E-mail: linotp@keyidentity.com
  *    Contact: www.linotp.org
- *    Support: www.lsexperts.de
+ *    Support: www.keyidentity.com
  *
  */
 
-
 /* For compatibility with <=IE8 */
 if (!Object.keys) Object.keys = function(o) {
 	if (o !== Object(o))
@@ -36,41 +35,20 @@ if (!Object.keys) Object.keys = function(o) {
 
 window.onerror = error_handling;
 
-/* Use Jed for i18n. The correct JSON file is dynamically loaded later. */
-var i18n = new Jed({});
-var sprintf = Jed.sprintf;
-
-if (!String.sprintf) {
-    String.sprintf = Jed.sprintf;
-}
-
 /* The HTTP status code, that determines that
  * the Login to the selfservice portal is required.
  * Is also defined in controllers/account.py
  */
-LOGIN_CODE = 576
+LOGIN_CODE = 403
 
-function SelfLogout(logout_url) {
 /* clear the admin cookie and
    * for IE try to clean the ClearAuthenticationCache and reload same page
-   * for Firefox redirect to a location, with
+   * for evergreen browsers: clear the cookie and redirect to the logout page
 */
-
-    var done = false;
-    done = document.execCommand("ClearAuthenticationCache", false);
-    $.cookie("linotp_selfservice", "invalid", {expires: 0,  path: '/'});
-
-    if (done == true) {
-        window.location.href = document.URL;
-    } else {
-        window.location.href = logout_url;
-    }
-
+function SelfLogout(logout_url) {
+    window.location.href = logout_url;
 }
 
-
-
-
 function self_alert_box(params /* dict or parameters */){
 	/*
 	 * alert_box - pop up an dialog to show some info, which could contain html
@@ -138,14 +116,25 @@ function error_handling(message, file, line) {
 function get_selfservice_session() {
     var session = "";
     if (document.cookie) {
-        session = getcookie("linotp_selfservice");
+        session = getcookie("user_selfservice");
         if (session == "") {
-            alert(i18n.gettext("there is no linotp_selfservice cookie"));
+            alert(i18n.gettext("there is no user_selfservice cookie"));
         }
     }
     return session;
 }
 
+function clientUrlFetchSync(myUrl, params) {
+    var resp = $.ajax({
+        url : myUrl,
+        data : params,
+        async : false,
+        type : 'POST',
+    });
+
+    return resp;
+}
+
 function run_sync_request(url,params){
     /*
      * run_sync_request - to submit a syncronous  http request
@@ -256,7 +245,6 @@ function assign() {
             }
         } // end of else
     }
-    showTokenlist();
     return false;
 }
 
@@ -301,7 +289,6 @@ function token_call(formid, params) {
     } else {
         alert(i18n.gettext("Form data not valid."));
     }
-    showTokenlist();
     return false;
 
 }
@@ -355,6 +342,14 @@ function enroll_token(params) {
                     details = details + '<li> otpkey: ' + escape(detail.otpkey) + '</li>';
                 }
             }
+            if (detail.hasOwnProperty('lse_qr_url')) {
+                details = details + '<li>' + detail.lse_qr_url.description + '</li>';
+                if (detail.lse_qr_url.hasOwnProperty('img')) {
+                   details = details + '<p>' + detail.lse_qr_url.img + '</p>';
+                }
+                details = details + '<p class="enrollment_value"><a href="' +
+                    detail.lse_qr_url.value + '">' + detail.lse_qr_url.value + '</a></li>';
+            }
             if (detail.hasOwnProperty('ocraurl')) {
                 details = details + '<li>OCRA QR Code</li>';
                 if (detail.ocraurl.hasOwnProperty('img')) {
@@ -380,7 +375,6 @@ function enroll_token(params) {
         alert(i18n.gettext("Failed to enroll token: ") + escape(data.result.error.message));
     };
 
-
     showTokenlist();
     return false;
 
@@ -394,7 +388,6 @@ function reset_failcounter() {
     var data = run_sync_request("/userservice/reset", params);
     if (data.result.status == true) {
         alert(i18n.gettext("Failcounter resetted successfully"));
-        showTokenlist();
         $('.selectedToken').val("");
     } else {
         alert(i18n.gettext("Failed to reset failcounter!\n") + escape(data.result.error.message));
@@ -470,7 +463,6 @@ function getotp() {
         } else {
             alert(i18n.gettext("Error getting otp values") + ":\n" + escape(data.result.value.error));
         }
-        showTokenlist();
         $('.selectedToken').val("");
     } else {
         alert(i18n.gettext("Error getting otp values") + ":\n" + escape(data.result.error.message));
@@ -517,7 +509,6 @@ function provisionOath() {
         if (data.result.value.init == true) {
             // The token was successfully initialized and we will display the url
             showTokenlist();
-            //$('#oath_info').hide();
             var url = data.result.value.oathtoken.url;
             var img = data.result.value.oathtoken.img;
             $('#oath_link').attr("href", url);
@@ -547,8 +538,6 @@ function provisionOcra() {
     if (data.result.status == true) {
         if (data.result.value.activate == true) {
             // The token was successfully initialized and we will display the url
-            showTokenlist();
-            // console_log(data.result.value)
             var img = data.result.value.ocratoken.img;
             var url = data.result.value.ocratoken.url;
             var trans = data.result.value.ocratoken.transaction;
@@ -584,16 +573,16 @@ function finishOcra() {
     if (data.result.status == true) {
         // The token was successfully initialized and we will display the url
         // if not (false) display an ocra_finish_fail message for retry
-        showTokenlist();
         if (data.result.value.result == false) {
             alert(ocra_finish_fail);
         } else {
-            alert(String.sprintf(ocra_finish_ok, serial));
+            showTokenlist();
+            alert(sprintf(ocra_finish_ok, serial));
             $('#qr_completed').show();
             $('#qr_finish').hide();
             //$('#ocra_check').attr("disabled","disabled");
             $('#ocra_qr_code').html('<div/>');
-            $('#qr_completed').html(escape(String.sprintf(ocra_finish_ok, serial)));
+            $('#qr_completed').html(escape(sprintf(ocra_finish_ok, serial)));
         }
     } else {
         alert(i18n.gettext("Failed to enroll token!\n") + escape(data.result.error.message));
@@ -611,9 +600,8 @@ function provisionGoogle() {
     var data = run_sync_request("/userservice/webprovision", params);
     if (data.result.status == true) {
         if (data.result.value.init == true) {
-            showTokenlist();
             // The token was successfully initialized and we will display the url
-            //var qr_code = generate_qrcode(10, data.result.value.oathtoken.url);
+            showTokenlist();
             var url = data.result.value.oathtoken.url;
             var img = data.result.value.oathtoken.img;
             $('#google_link').attr("href", url);
@@ -725,7 +713,6 @@ $(document).ready(function() {
     );
 
     $("#tabs").tabs({
-        collapsible : true,
         spinner : 'Retrieving data...',
         beforeLoad: function( event, ui ) {
             // The purpose of the following is to prevent automatic reloads
@@ -738,12 +725,11 @@ $(document).ready(function() {
                 event.preventDefault();
             }
             else {
-                ui.jqXHR.success(function() {
-                    ui.tab.data ( "loaded", true );
+                ui.jqXHR.then(function () {
                 });
                 // Following replaces ajaxOptions error function. ajaxOptions was
                 // removed in jQuery UI 1.10
-                ui.jqXHR.error(function( jqXHR ){
+                ui.jqXHR.fail(function (jqXHR) {
                     if (jqXHR.status == LOGIN_CODE) {
                         alert(i18n.gettext("Your session has expired!"));
                         location.reload();
@@ -763,10 +749,9 @@ $(document).ready(function() {
 
     // delegated event
     // The list of tokens is dynamic and can change after page load
-    // If it were static one could use $("#tokenDiv ul li").click(function ...
-    $("#tokenDiv").on("click", "ul li", function(event) {
-        event.preventDefault();
-        selectToken($.trim($(this).text()));
+    $("#tokenDiv").on("click", "button", function(e) {
+        var serial = $(this).attr("data-serial");
+        selectToken(serial);
     });
 });
 
@@ -848,24 +833,4 @@ function view_audit_selfservice() {
     });
 }
 
-/*
- * window.CURRENT_LANGUAGE is set in the template from the mako lib.
- * Here, we dynamically load the desired language JSON file for Jed.
- */
-var browser_lang = window.CURRENT_LANGUAGE || 'en';
-if (browser_lang && browser_lang !== 'en') {
-    try {
-        var url = sprintf("/i18n/%s.json", browser_lang);
-        $.get(
-            url,
-            {},
-            function(data, textStatus) {
-                i18n.options.locale_data.messages = data;
-            },
-            "json"
-        );
-    } catch(e) {
-        alert('Unsupported localisation for ' + escape(browser_lang));
-    }
-}
-
+loadTranslations();
