@@ -1,6 +1,6 @@
 /*!
  *   LinOTP - the open source solution for two factor authentication
- *   Copyright (C) 2010 - 2016 LSE Leading Security Experts GmbH
+ *   Copyright (C) 2010 - 2019 KeyIdentity GmbH
  *
  *   This file is part of LinOTP server.
  *
@@ -18,9 +18,9 @@
  *   along with this program.  If not, see <http://www.gnu.org/licenses/>.
  *
  *
- *    E-mail: linotp@lsexperts.de
+ *    E-mail: linotp@keyidentity.com
  *    Contact: www.linotp.org
- *    Support: www.lsexperts.de
+ *    Support: www.keyidentity.com
  *
  */
 
@@ -37,19 +37,25 @@ function create_vasco_dialog() {
                 $(this).dialog('close');
                 },
                 id: "button_vasco_load",
-                text: "Load DPX File"
+                text: "Load DPX File",
+                icons: {
+                    primary: 'ui-icon-folder-open'
+                }
                 },
             Cancel: {click: function(){
                 $(this).dialog('close');
                 },
                 id: "button_vasco_cancel",
-                text: "Cancel"
+                text: "Cancel",
+                icons: {
+                    primary: 'ui-icon-cancel'
+                }
                 }
         },
         open: function(){
-            translate_import_vasco();
             _fill_realms($('#vasco_realm'),1);
-            do_dialog_icons();
+
+            translate_import_vasco();
         }
        });
        return $dialog;
