@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 #
 #    LinOTP - the open source solution for two factor authentication
-#    Copyright (C) 2010 - 2016 LSE Leading Security Experts GmbH
+#    Copyright (C) 2010 - 2019 KeyIdentity GmbH
 #
 #    This file is part of LinOTP server.
 #
@@ -19,9 +19,9 @@
 #    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #
 #
-#    E-mail: linotp@lsexperts.de
+#    E-mail: linotp@keyidentity.com
 #    Contact: www.linotp.org
-#    Support: www.lsexperts.de
+#    Support: www.keyidentity.com
 #
 
 
@@ -44,6 +44,7 @@ class TestAdminController(TestController):
         self.create_common_realms()
 
     def tearDown(self):
+        self.delete_all_token()
         self.delete_all_realms()
         self.delete_all_resolvers()
         TestController.tearDown(self)
@@ -80,6 +81,16 @@ class TestAdminController(TestController):
         self.assertTrue('"value": true' in response, response)
         return serial
 
+    def createSPASS(self, serial="LSSP0001", pin="1test@pin!42"):
+        parameters = {
+                      "serial" : serial,
+                      "type"   : "spass",
+                      "pin"    : pin
+                      }
+        response = self.app.get(url(controller='admin', action='init'), params=parameters)
+        self.assertTrue('"value": true' in response, response)
+        return serial
+
     def createToken(self):
         parameters = {
                       "serial"  : "F722362",
@@ -561,6 +572,90 @@ class TestAdminController(TestController):
         self.delete_token(lost_token_name)
         return
 
+    def test_losttoken_spass(self):
+        """
+        test for losttoken callback - to register replacement for lost spass
+
+        test with user hans, who has a spass
+        - is the old one deactivated
+        - is the new one active
+        - is the new one of type 'pw'
+        - does the new password work
+
+        remark:
+            other losttoken tests depend on policy definition and are
+            part of the test_policy.py
+
+        """
+        token_name = "verloren"
+        spass_pin  = "initial_pin"
+
+        new_serial = self.createSPASS(serial=token_name, pin=spass_pin)
+        self.assertTrue(token_name == new_serial)
+
+        parameters = {"serial": token_name, "user": "hans"}
+        response = self.app.get(url(controller="admin", action="assign"),
+                                params=parameters)
+        self.assertTrue('"value": true' in response, response)
+
+        # check if this spass validates
+        response = self.app.get(url(controller='validate', action='check_s'),
+                                params={'serial': token_name,
+                                        'pass': spass_pin})
+        self.assertTrue('"value": true' in response, response)
+
+        parameters = {"serial": token_name, 'type': "spass"}
+        response = self.app.get(url(controller="admin", action="losttoken"),
+                                params=parameters)
+        self.assertTrue('"status": true' in response, response)
+
+        resp = json.loads(response.body)
+        temp_token_name = resp.get('result', {}).get('value', {}).get('serial')
+        temp_token_pass = resp.get('result', {}).get('value', {}).get('password')
+
+        # first check if old token is not active
+        parameters = {"serial": token_name}
+        response = self.app.get(url(controller="admin", action="show"),
+                                params=parameters)
+        self.assertTrue('"status": true' in response, response)
+        resp = json.loads(response.body)
+        data = resp.get("result", {}).get('value', {}).get('data', [{}])[0]
+        active = data.get("LinOtp.Isactive", True)
+        self.assertFalse(active, response)
+        user = data.get("User.username", '')
+        self.assertEqual(user, 'hans', response)
+
+        # second check if new token is active and properly assigned
+        parameters = {"serial": temp_token_name}
+        response = self.app.get(url(controller="admin", action="show"),
+                                params=parameters)
+        self.assertTrue('"status": true' in response, response)
+        resp = json.loads(response.body)
+        data = resp.get("result", {}).get('value', {}).get('data', [{}])[0]
+        active = data.get("LinOtp.Isactive", False)
+        self.assertTrue(active, response)
+
+        user = data.get("User.username", '')
+        self.assertEqual(user, 'hans', response)
+
+        ttype = data.get("LinOtp.TokenType", '')
+        self.assertEqual(ttype, 'pw', response)
+
+        # finally, check if old spass is blocked and new one works without previous pin
+        response = self.app.get(url(controller='validate', action='check_s'),
+                                params={'serial': token_name,
+                                        'pass': spass_pin})
+        self.assertTrue('"value": false' in response, response)
+        response = self.app.get(url(controller='validate', action='check_s'),
+                                params={'serial': temp_token_name,
+                                        'pass': temp_token_pass})
+        self.assertTrue('"value": true' in response, response)
+
+        # all fine, clean up
+        self.delete_token(token_name)
+        self.delete_token(temp_token_name)
+        return
+
     def test_enroll_umlaut(self):
 
         parameters = {
@@ -712,6 +807,69 @@ class TestAdminController(TestController):
         self.assertTrue('"set validityPeriodStart": 1' in response, response)
         self.assertTrue('"set validityPeriodEnd": 1' in response, response)
 
+    def test_set_validity_interface(self):
+        '''
+        Setting validity period via admin/setValidity interface
+        '''
+
+        token_serial_1 = 'token_set_validity1'
+        token_serial_2 = 'token_set_validity2'
+
+        response = self.app.get(url(controller='admin', action='init'),
+                                params={'serial': token_serial_1,
+                                        'type': 'spass'})
+
+        self.assertTrue('"value": true' in response, response)
+
+        response = self.app.get(url(controller='admin', action='init'),
+                                params={'serial': token_serial_2,
+                                        'type': 'spass'})
+
+        self.assertTrue('"value": true' in response, response)
+
+        response = self.make_admin_request('setValidity',
+                                params={
+                                    'tokens': [
+                                        token_serial_1,
+                                        token_serial_2
+                                        ],
+                                     'validityPeriodStart': '2012-10-12',
+                                     'validityPeriodEnd': '2013-12-30',
+                                     },
+                                content_type='application/json')
+
+        self.assertTrue('"status": false' in response, response)
+        msg = "invalid literal for int() with base 10: '2012-10-12'"
+        self.assertTrue(msg in response, response)
+
+        response = self.make_admin_request(
+                                action='setValidity',
+                                params={
+                                    'tokens': [
+                                        token_serial_1,
+                                        token_serial_2,
+                                        ],
+                                        'validityPeriodStart': '1355302800',
+                                        'validityPeriodEnd': '1355310000',
+                                        },
+                                content_type="application/json"
+                                )
+
+        self.assertTrue(
+            response.json['result']['status'] is True,
+            'Expected response.result.status to be True in response text: "{}"'
+                .format(response.text))
+
+        self.assertTrue(
+            token_serial_1 in response.json['result']['value'],
+            'Expected response.result.value to contain token id "{}" in response text: "{}"'
+                .format(token_serial_1, response.text))
+
+        self.assertTrue(
+            token_serial_2 in response.json['result']['value'],
+            'Expected response.result.value to contain token id "{}" in response text: "{}"'
+                .format(token_serial_2, response.text))
+
     def test_set_empty(self):
         '''
         Running set without parameter
