@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 #
 #    LinOTP - the open source solution for two factor authentication
-#    Copyright (C) 2010 - 2016 LSE Leading Security Experts GmbH
+#    Copyright (C) 2010 - 2019 KeyIdentity GmbH
 #
 #    This file is part of LinOTP server.
 #
@@ -19,14 +19,14 @@
 #    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #
 #
-#    E-mail: linotp@lsexperts.de
+#    E-mail: linotp@keyidentity.com
 #    Contact: www.linotp.org
-#    Support: www.lsexperts.de
+#    Support: www.keyidentity.com
 #
 
 
 """
-  Test linotp.lib.tokens.emailtoken
+  Test linotp.tokens.emailtoken
 """
 
 from mock import patch
@@ -51,7 +51,7 @@ class TestEmailtokenController(TestController):
         self.create_common_resolvers()
         self.create_common_realms()
         params = {
-            'EmailProvider': 'linotp.lib.emailprovider.SMTPEmailProvider',
+            'EmailProvider': 'linotp.provider.emailprovider.SMTPEmailProvider',
             'EmailProviderConfig': '{ "SMTP_SERVER": "mail.example.com",\
                                "SMTP_USER": "secret_user",\
                                "SMTP_PASSWORD": "secret_pasword" }',
@@ -89,22 +89,84 @@ class TestEmailtokenController(TestController):
         self.patch_smtp.stop()
         self.delete_all_realms()
         self.delete_all_resolvers()
+        self.delete_all_token()
         TestController.tearDown(self)
 
-    def test_default(self):
+    def test_0000_default(self):
         """
         Test the default case: enroll, assign, send challenge, get successful response
         """
+        #
+        # check: correct otp and pin should be ok
+        #
+
         response, otp = self._trigger_challenge()
         self._assert_email_sent(response)
+
         response = self.app.get(url(controller='validate', action='check'),
-                                params={'user': 'root', 'pass': self.pin + otp})
+                                params={'user': 'root',
+                                        'pass': self.pin + otp})
         response_json = response.json
         self.assertTrue(response_json['result']['status'])
         self.assertTrue(response_json['result']['value'])
 
+        #
+        # check: wrong pin should fail
+        #
 
-    def test_multiple_challenges(self):
+        response, otp = self._trigger_challenge()
+        self._assert_email_sent(response)
+
+        response = self.app.get(url(controller='validate', action='check'),
+                                params={'user': 'root',
+                                        'pass': "4321" + otp})
+        response_json = response.json
+        self.assertTrue(response_json['result']['status'])
+        self.assertFalse(response_json['result']['value'])
+
+        #
+        # replay with correct pin should fail
+        #
+
+        time.sleep(5)
+        response = self.app.get(url(controller='validate', action='check'),
+                                params={'user': 'root',
+                                        'pass': self.pin + otp})
+        response_json = response.json
+        self.assertTrue(response_json['result']['status'])
+        self.assertFalse(response_json['result']['value'])
+
+        #
+        # check with wrong otp should fail as well
+        #
+
+        response, otp = self._trigger_challenge()
+        self._assert_email_sent(response)
+
+        response = self.app.get(url(controller='validate', action='check'),
+                                params={'user': 'root',
+                                        'pass': self.pin + "123456"})
+        response_json = response.json
+        self.assertTrue(response_json['result']['status'])
+        self.assertFalse(response_json['result']['value'])
+
+        #
+        # check with correct otp and pin should be ok
+        #
+
+        response, otp = self._trigger_challenge()
+        self._assert_email_sent(response)
+
+        response = self.app.get(url(controller='validate', action='check'),
+                                params={'user': 'root',
+                                        'pass': self.pin + otp})
+        response_json = response.json
+        self.assertTrue(response_json['result']['status'])
+        self.assertTrue(response_json['result']['value'])
+
+        return
+
+    def test_00000_multiple_challenges(self):
         """
         Test with multiple challenges
 
@@ -129,6 +191,8 @@ class TestEmailtokenController(TestController):
         time.sleep(5)
         # trigger 3rd challenge and store resulting information
         stored_response, stored_otp = self._trigger_challenge()
+        transaction_id = stored_response['detail']['transactionid']
+
         self._assert_email_sent(response)
         time.sleep(5)
         # trigger 4th challenge
@@ -136,10 +200,12 @@ class TestEmailtokenController(TestController):
         self._assert_email_sent(response)
 
         # Send the response with the stored values from the 3rd challenge
-        transaction_id = stored_response['detail']['transactionid']
         # since we are sending the transactionid we only need the otp (without pin)
+        params = {'user': 'root',
+                  'pass': stored_otp,
+                  'transactionid': transaction_id}
         response = self.app.get(url(controller='validate', action='check'),
-                                params={'user': 'root', 'pass': stored_otp, 'transactionid': transaction_id})
+                                    params=params)
         response = response.json
         self.assertTrue(response['result']['status'])
         self.assertTrue(response['result']['value'])
@@ -157,6 +223,17 @@ class TestEmailtokenController(TestController):
         self.assertTrue(response['result']['status'])
         self.assertFalse(response['result']['value'], "Challenge should have timed out")
 
+    def test_otp_not_reused(self):
+
+        """
+        check if otp isn't reused
+        """
+
+        __, otp1 = self._trigger_challenge()
+        __, otp2 = self._trigger_challenge()
+
+        self.assertNotEqual(otp1, otp2, "OTP counter not working properly")
+
     def test_blocking(self):
         """
         Test that no new e-mails are sent out during EmailBlockingTimeout
@@ -191,19 +268,18 @@ class TestEmailtokenController(TestController):
 
     def test_smtplib_exceptions(self):
         """
-        Verify that SMTPRecipientsRefused exception is caught and no challenge is created.
-
-        We assume that this works for other smtplib exceptions as well, because from LinOTPs point
-        of view they behave in the same way.
+        Verify that SMTPRecipientsRefused exception is caught and no
+        challenge is created. We assume that this works for other smtplib
+        exceptions as well, because from LinOTPs point of view they behave in
+        the same way.
         """
+
         # Get existing challenges (to verify later that no new ones were added)
-        existing_challenges = {}
-        try:
-            response_string = self.make_admin_request('checkstatus', {'user': 'root'})
-            response = response_string.json
-            existing_challenges = response['result']['value']['values'][self.token_serial]['challenges']
-        except KeyError:
-            pass  # No challenges exist for this token
+        response_string = self.make_admin_request('checkstatus',
+                                                      {'user': 'root'})
+        response = response_string.json
+        values = response.get('result').get('value').get('values', {})
+        existing_challenges = values.get(self.token_serial, {}).get('challenges', {})
 
         # Trigger SMTPRecipientsRefused exception when sendmail is called
         exception_to_raise = smtplib.SMTPRecipientsRefused(
@@ -218,14 +294,16 @@ class TestEmailtokenController(TestController):
         self.mock_smtp_instance.sendmail.side_effect = exception_to_raise
         response_string = self.app.get(url(controller='validate', action='check'),
                                        params={'user': 'root', 'pass': self.pin})
-        response = response_string.json
-        expected_error = "error sending e-mail %r" % exception_to_raise
-        self.assertEqual(expected_error, response['detail']['message'], "Error message does not match")
+        # response = response_string.json
+        # expected_error = "error sending e-mail %r" % exception_to_raise
+        # self.assertEqual(expected_error, response['detail']['message'], "Error message does not match")
+        self.assertTrue('"value": false' in response_string, response_string)
 
         # Get new challenges
         response_string = self.make_admin_request('checkstatus', {'user': 'root'})
         response = response_string.json
-        new_challenges = response['result']['value']['values'][self.token_serial]['challenges']
+        values = response['result']['value']['values']
+        new_challenges = values.get(self.token_serial, {}).get('challenges', {})
 
         # Verify that no challenge was created (the exception should have prevented it)
         self.assertTrue(existing_challenges == new_challenges,
