@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 #
 #    LinOTP - the open source solution for two factor authentication
-#    Copyright (C) 2010 - 2016 LSE Leading Security Experts GmbH
+#    Copyright (C) 2010 - 2019 KeyIdentity GmbH
 #
 #    This file is part of LinOTP server.
 #
@@ -19,9 +19,9 @@
 #    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #
 #
-#    E-mail: linotp@lsexperts.de
+#    E-mail: linotp@keyidentity.com
 #    Contact: www.linotp.org
-#    Support: www.lsexperts.de
+#    Support: www.keyidentity.com
 #
 
 
@@ -215,7 +215,7 @@ class LDAP(object):
 
         attrs = {}
         if dn is None:
-            raise Exception('missing requred dn')
+            raise Exception('missing required dn')
 
         if objectclass is None:
             attrs['objectclass'] = ['top', 'person', 'organizationalPerson', 'inetOrgPerson', 'uidObject']
@@ -476,7 +476,7 @@ class TestLDAP(TestController):
 
         if self.appconf.has_key('linotp.ldapTestServerIp.' + hostname):
             self.ldapurl = self.appconf.get('linotp.ldapTestServerIp.' + hostname)
-        elif self.appconf.has_key('<include>') == True:
+        elif self.appconf.has_key('<include>') is True:
             try:
                 filename = self.appconf.get('<include>')
                 cfgParse = ConfigParser.ConfigParser()
@@ -605,7 +605,7 @@ class TestLDAP(TestController):
             if "Can't contact LDAP server" in msg:
                 ldapServerConnected = False
 
-        if ldapServerConnected == False:
+        if ldapServerConnected is False:
             self.fail("No LDAP connection")
 
 
@@ -726,7 +726,3 @@ class TestLDAP(TestController):
 
             self.delOtpPinPolicy()
         return
-
-
-
-
