@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 #
 #    LinOTP - the open source solution for two factor authentication
-#    Copyright (C) 2010 - 2016 LSE Leading Security Experts GmbH
+#    Copyright (C) 2010 - 2019 KeyIdentity GmbH
 #
 #    This file is part of LinOTP server.
 #
@@ -19,9 +19,9 @@
 #    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #
 #
-#    E-mail: linotp@lsexperts.de
+#    E-mail: linotp@keyidentity.com
 #    Contact: www.linotp.org
-#    Support: www.lsexperts.de
+#    Support: www.keyidentity.com
 #
 
 
@@ -32,12 +32,13 @@ Test the otp_pin_random policy
 from collections import deque
 from copy import deepcopy
 
+from linotp.tests import TestController
+
 import logging
 log = logging.getLogger(__name__)
 
-from linotp.tests import TestController
 
-from distutils.version import LooseVersion
+
 
 class TestRandompinController(TestController):
     """
@@ -66,6 +67,7 @@ class TestRandompinController(TestController):
         self.create_common_realms()
         self.token_for_deletion = set()
         self.policies_for_deletion = set()
+
         return
 
     def tearDown(self):
@@ -118,9 +120,9 @@ class TestRandompinController(TestController):
         """
         Same as 'test_simple_enroll' but with assign after enroll
 
-        Verify the behaviour is the same if the token is first enrolled and then
-        assigned to a user, instead of directly enrolling for the user as in
-        test_simple_enroll.
+        Verify the behaviour is the same if the token is first enrolled and
+        then assigned to a user, instead of directly enrolling for the user
+        as in test_simple_enroll.
         """
         # Enroll token
         user = u'aἰσχύλος'  # realm myDefRealm
@@ -169,19 +171,16 @@ class TestRandompinController(TestController):
         """
         User logs into selfservice and sets PIN then authenticates with PIN+OTP
 
-        After enrolling the PIN is unknown and the token can't be used. The user
-        can log into the selfservice and set a PIN for his token. Then he can
-        authenticate with PIN+OTP.
+        After enrolling the PIN is unknown and the token can't be used. The
+        user can log into the selfservice and set a PIN for his token. Then
+        he can authenticate with PIN+OTP.
 
         This test will fail with WebTest 1.2.1 (Debian Squeeze) because of a
         bug that caused cookies to be quoted twice. The bug is fixed in 1.2.2.
-        https://github.com/Pylons/webtest/commit/8471db1c2dc505c633bca2d39d5713dba0c51a42
+        https://github.com/Pylons/webtest/
+                            commit/8471db1c2dc505c633bca2d39d5713dba0c51a42
         """
 
-        # selfservice authentication does a redirect
-        if self._version_['pylons'] <= LooseVersion('0.10'):
-            self.skipTest("Pylons lower 0.10 does not support redirect!")
-
         self._create_randompin_policy('myDefRealm')
         self._create_selfservice_policy('myDefRealm')
 
@@ -211,17 +210,14 @@ class TestRandompinController(TestController):
 
     def test_admin_setpin(self):
         """
-        Admin can set PIN, even after the user set it in selfservice
+        Admin can set the PIN, even after the user has set it in selfservice
 
         This test will fail with WebTest 1.2.1 (Debian Squeeze) because of a
         bug that caused cookies to be quoted twice. The bug is fixed in 1.2.2.
-        https://github.com/Pylons/webtest/commit/8471db1c2dc505c633bca2d39d5713dba0c51a42
+        https://github.com/Pylons/webtest/
+                                commit/8471db1c2dc505c633bca2d39d5713dba0c51a42
         """
 
-        # selfservice authentication does a redirect
-        if self._version_['pylons'] <= LooseVersion('0.10'):
-            self.skipTest("Pylons lower 0.10 does not support redirect!")
-
         self._create_randompin_policy('myDefRealm')
         self._create_selfservice_policy('myDefRealm')
 
@@ -268,17 +264,14 @@ class TestRandompinController(TestController):
         """
         Verify PIN is overwritten when assigning token to a different user
 
-        Test both the case where the user is in the same realm (where the policy
-        is defined) and in another realm without opt_pin_random policy.
+        Test both the case where the user is in the same realm (where the
+        policy is defined) and in another realm without opt_pin_random policy.
 
         This test will fail with WebTest 1.2.1 (Debian Squeeze) because of a
         bug that caused cookies to be quoted twice. The bug is fixed in 1.2.2.
-        https://github.com/Pylons/webtest/commit/8471db1c2dc505c633bca2d39d5713dba0c51a42
+        https://github.com/Pylons/webtest/
+                                commit/8471db1c2dc505c633bca2d39d5713dba0c51a42
         """
-        # selfservice authentication does a redirect
-        if self._version_['pylons'] <= LooseVersion('0.10'):
-            self.skipTest("Pylons lower 0.10 does not support redirect!")
-
         self._create_randompin_policy('myDefRealm')
         self._create_selfservice_policy('myDefRealm')
 
@@ -309,7 +302,7 @@ class TestRandompinController(TestController):
         self._assign(token['serial'], new_user)
 
         # authenticate fails because old PIN is no longer valid (i.e. was
-        # overwritten with a random value during assign)
+        # overwritten with a random value during assignment)
         self._validate(
             new_user,
             pin + token['otps'].popleft(),
@@ -370,9 +363,7 @@ class TestRandompinController(TestController):
                 )
         return
 
-
-    # -------- Private helper methods --------
-
+    # -------- Private helper methods ----- --
     def _create_randompin_policy(self, realm):
         """
         Creates an otp_pin_random policy for 'realm'. Schedules the policy for
@@ -448,11 +439,12 @@ class TestRandompinController(TestController):
 
     def _validate(self, user, pwd, expected='success', err_msg=None):
         """
-        Makes a validate/check request and verifies the response is as 'expected'
+        runs a validate/check request and verifies the response is as 'expected'
 
         :param user: Username or username@realm
         :param pwd: Password (e.g. PIN+OTP)
-        :param expected: One of 'success', 'value-false', 'status-false' or 'both-false'
+        :param expected: One of 'success', 'value-false', 'status-false' or
+                        'both-false'
         :param err_msg: An error message to display if assert fails
         :return: The content (JSON object)
         """
@@ -490,7 +482,8 @@ class TestRandompinController(TestController):
             err_msg=err_msg,
             )
 
-    def _validate_base(self, params, action='check', expected='success', err_msg=None):
+    def _validate_base(self, params, action='check', expected='success',
+                       err_msg=None):
         """
         Base method for /validate/<action> requests
 
@@ -500,7 +493,7 @@ class TestRandompinController(TestController):
         :param params: Request parameters
         :param expected: One of 'success', 'value-false', 'status-false' or
             'both-false'
-        :param err_msg: An error message to display if assert fails
+        :param err_msg: An error message to display if the assert fails
         :return: The content (JSON object)
         """
         response = self.make_validate_request(action, params=params)
@@ -554,52 +547,20 @@ class TestRandompinController(TestController):
         :param serial: Token serial
         :param pin: The PIN to be set
         """
-        params = {
-            'login': user.encode('utf-8'),
-            'password': pwd.encode('utf-8'),
-            'defaultRealm': 'myDefRealm',
-            'realm': '',
-            'realmbox': False,
-            }
-        response = self.make_request('account', 'dologin', params=params)
-        err_msg = "Unexpected response %r" % response
-        self.assertEqual(302, response.status_int, err_msg)
-        self.assertEqual('/', response.headers['location'])
-        self.assertRegexpMatches(
-            response.headers['Set-Cookie'],
-            r"^linotp_selfservice=.*",
-            err_msg,
-            )
-
-        session = self.app.cookies['linotp_selfservice']
-        session = session.strip('"')
-        self.assertGreater(len(session), 0, err_msg)
         params = {
             'serial': serial,
-            'session': session,
             'userpin': pin,
             }
-        cookies = {
-            'linotp_selfservice': '"%s"' % session,
-            }
-        response = self.make_request(
-            'userservice',
-            'setpin',
-            params=params,
-            cookies=cookies,
-            method='POST'
-            )
-
-        session_info = "cookie %r : session %r" % (cookies, session)
-        try:
-            content = TestController.get_json_body(response)
-        except ValueError as err:
-            log.error("%r: %s", err, session_info)
-            raise Exception(err)
+        login = user.encode('utf-8')
+        password = pwd.encode('utf-8')
+        response = self.make_userservice_request('setpin', params,
+                                                 auth_user=(login, password))
 
+        content = TestController.get_json_body(response)
         self.assertTrue(content['result']['status'])
-        expected = {"set userpin": 1}
-
+        expected = {
+            "set userpin": 1
+            }
         self.assertDictEqual(expected, content['result']['value'])
         return
 
