@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 #
 #    LinOTP - the open source solution for two factor authentication
-#    Copyright (C) 2010 - 2016 LSE Leading Security Experts GmbH
+#    Copyright (C) 2010 - 2019 KeyIdentity GmbH
 #
 #    This file is part of LinOTP server.
 #
@@ -19,9 +19,9 @@
 #    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #
 #
-#    E-mail: linotp@lsexperts.de
+#    E-mail: linotp@keyidentity.com
 #    Contact: www.linotp.org
-#    Support: www.lsexperts.de
+#    Support: www.keyidentity.com
 #
 
 
@@ -29,6 +29,7 @@
 """
 
 import logging
+import json
 
 from linotp.tests import TestController, url
 
@@ -57,10 +58,10 @@ class TestSelfserviceController(TestController):
                                         'action' : policy,
                                         'selftest_admin' : 'superadmin'
                                         })
-        print response
         assert '"status": true' in response
         assert '"setPolicy self01": {' in response
 
+        return response
 
     def deleteToken(self, serial):
         response = self.app.get(url(controller='admin', action='remove'),
@@ -75,13 +76,17 @@ class TestSelfserviceController(TestController):
         '''
         self.createPolicy("history")
 
-        response = self.app.get(url(controller='userservice', action='history'),
-                                params={'selftest_user': 'root@myDefRealm'})
+        auth_user = {
+             'login': 'passthru_user1@myDefRealm',
+             'password': 'geheim1'}
+
+        response = self.make_userselfservice_request('history',
+                                auth_user=auth_user)
         print response
         assert '"rows": [' in response
 
-        response = self.app.get(url(controller='selfservice', action='history'),
-                                params={'selftest_user':'root@myDefRealm'})
+        response = self.make_selfservice_request('history',
+                                auth_user=auth_user)
         print response
         assert 'view_audit_selfservice' in response
 
@@ -89,15 +94,20 @@ class TestSelfserviceController(TestController):
         '''
         Selfservice: Testing user reset
         '''
-        response = self.app.get(url(controller='userservice', action='reset'),
-                                params={'selftest_user': 'root@myDefRealm'})
+
+        auth_user = {
+             'login': 'passthru_user1@myDefRealm',
+             'password': 'geheim1'}
+
+        response = self.make_userselfservice_request('reset',
+                                auth_user=auth_user)
         print response
         assert '"status": false' in response
         assert '"code": -311' in response
 
         self.createPolicy("reset")
-        response = self.app.get(url(controller='userservice', action='reset'),
-                                params={'selftest_user': 'root@myDefRealm'})
+        response = self.make_userselfservice_request('reset',
+                                auth_user=auth_user)
         print response
         assert 'Missing parameter: ' in response
         assert '"code": 905' in response
@@ -105,7 +115,7 @@ class TestSelfserviceController(TestController):
         response = self.app.get(url(controller='admin', action='init'),
                                 params={'serial':'reset01',
                                         'type': 'spass',
-                                        'user': 'root',
+                                        'user': 'passthru_user1@myDefRealm',
                                         'pin': "secret"
                                         })
         print response
@@ -113,26 +123,26 @@ class TestSelfserviceController(TestController):
 
         for i in "12345678901234567890":
             response = self.app.get(url(controller='validate', action='check'),
-                                    params={'user': 'root',
+                                    params={'user': 'passthru_user1@myDefRealm',
                                             'pass': 'wrongpass'})
             print response
             assert '"value": false' in response
 
-        response = self.app.get(url(controller='userservice', action='reset'),
-                                params={'selftest_user': 'root@myDefRealm',
-                                        'serial': 'reset01'})
+        response = self.make_userselfservice_request('reset',
+                                auth_user=auth_user,
+                                params={'serial': 'reset01'})
         print response
         assert '"status": true' in response
         assert '"reset Failcounter": 1' in response
 
         response = self.app.get(url(controller='validate', action='check'),
-                                params={'user': 'root',
+                                params={'user': 'passthru_user1@myDefRealm',
                                         'pass': 'secret'})
         print response
         assert '"value": true' in response
 
-        response = self.app.get(url(controller='selfservice', action='reset'),
-                                params={'selftest_user':'root@myDefRealm'})
+        response = self.make_selfservice_request('reset',
+                                auth_user=auth_user)
         print response
         assert "<div id='resetform'>" in response
 
@@ -140,15 +150,20 @@ class TestSelfserviceController(TestController):
         '''
         Selfservice: Testing user resync
         '''
-        response = self.app.get(url(controller='userservice', action='resync'),
-                                params={'selftest_user': 'root@myDefRealm'})
+
+        auth_user = {
+             'login': 'passthru_user1@myDefRealm',
+             'password': 'geheim1'}
+
+        response = self.make_userselfservice_request('resync',
+                                auth_user=auth_user)
         print response
         assert '"status": false' in response
         assert '"code": -311' in response
 
         self.createPolicy("resync")
-        response = self.app.get(url(controller='userservice', action='resync'),
-                                params={'selftest_user': 'root@myDefRealm'})
+        response = self.make_userselfservice_request('resync',
+                                auth_user=auth_user)
         print response
         assert 'Missing parameter' in response
         assert '"code": 905' in response
@@ -156,33 +171,33 @@ class TestSelfserviceController(TestController):
         response = self.app.get(url(controller='admin', action='init'),
                                 params={'serial':'token01',
                                         'type': 'hmac',
-                                        'user': 'root',
+                                        'user': 'passthru_user1@myDefRealm',
                                         'pin': "secret",
                                         'otpkey': '6161e082d736d3d9d67bc1d4711ff1a81af26160'
                                         })
         print response
         assert '"status": true' in response
 
-        response = self.app.get(url(controller='userservice', action='resync'),
-                                params={'selftest_user': 'root@myDefRealm',
-                                        'serial': 'XXXX',
+        response = self.make_userselfservice_request('resync',
+                                auth_user=auth_user,
+                                params={'serial': 'XXXX',
                                         "otp1": "359864",
                                         "otp2": "348448" })
         print response
         assert '"status": false' in response
         assert 'no token found!' in response
 
-        response = self.app.get(url(controller='userservice', action='resync'),
-                                params={'selftest_user': 'root@myDefRealm',
-                                        'serial': 'token01',
+        response = self.make_userselfservice_request('resync',
+                                auth_user=auth_user,
+                                params={'serial': 'token01',
                                         "otp1": "885497",
                                         "otp2": "696793" })
         print response
         assert '"status": true' in response
         assert '"resync Token": true' in response
 
-        response = self.app.get(url(controller='selfservice', action='resync'),
-                                params={'selftest_user':'root@myDefRealm'})
+        response = self.make_selfservice_request('resync',
+                                auth_user=auth_user)
         print response
         assert "<div id='resyncform'>" in response
 
@@ -193,17 +208,21 @@ class TestSelfserviceController(TestController):
         Selfservice: setting mOTP PIN
         '''
 
-        response = self.app.get(url(controller='userservice', action='setmpin'),
-                                params={'selftest_user': 'root@myDefRealm',
-                                        'serial': 'XXXX',
+        auth_user = {
+             'login': 'passthru_user1@myDefRealm',
+             'password': 'geheim1'}
+
+        response = self.make_userselfservice_request('setmpin',
+                                auth_user=auth_user,
+                                params={'serial': 'XXXX',
                                         'pin': '1234'})
         print response
         assert '"status": false' in response
         assert '"message": "ERR410: The policy settings do not allow you to issue this request!"' in response
 
         self.createPolicy("setMOTPPIN")
-        response = self.app.get(url(controller='userservice', action='setmpin'),
-                                params={'selftest_user': 'root@myDefRealm'})
+        response = self.make_userselfservice_request('setmpin',
+                                auth_user=auth_user)
         print response
         assert 'Missing parameter: \'pin\'' in response
         assert '"code": 905' in response
@@ -212,23 +231,23 @@ class TestSelfserviceController(TestController):
         response = self.app.get(url(controller='admin', action='init'),
                                 params={'serial':'token01',
                                         'type': 'hmac',
-                                        'user': 'root',
+                                        'user': 'passthru_user1@myDefRealm',
                                         'pin': "secret",
                                         'otpkey': '6161e082d736d3d9d67bc1d4711ff1a81af26160'
                                         })
         print response
         assert '"status": true' in response
 
-        response = self.app.get(url(controller='userservice', action='setmpin'),
-                                params={'selftest_user': 'root@myDefRealm',
-                                        'serial': 'token01',
+        response = self.make_userselfservice_request('setmpin',
+                                auth_user=auth_user,
+                                params={'serial': 'token01',
                                         'pin': '1234'})
         print response
         assert '"status": true' in response
         assert '"set userpin": 1' in response
 
-        response = self.app.get(url(controller='selfservice', action='setmpin'),
-                                params={'selftest_user':'root@myDefRealm'})
+        response = self.make_selfservice_request('setmpin',
+                                auth_user=auth_user)
         print response
         assert "<div id='passwordform'>" in response
 
@@ -240,50 +259,54 @@ class TestSelfserviceController(TestController):
         response = self.app.get(url(controller='admin', action='init'),
                                 params={'serial':'spass01',
                                         'type': 'spass',
-                                        'user': 'root',
+                                        'user': 'passthru_user1@myDefRealm',
                                         })
         print response
         assert '"status": true' in response
 
-        response = self.app.get(url(controller='userservice', action='setpin'),
-                                params={'selftest_user': 'root@myDefRealm',
-                                        'serial': 'spass01',
+        auth_user = {
+             'login': 'passthru_user1@myDefRealm',
+             'password': 'geheim1'}
+
+        response = self.make_userselfservice_request('setpin',
+                                auth_user=auth_user,
+                                params={'serial': 'spass01',
                                         'pin': '1234'})
         print response
         assert '"status": false' in response
         assert '"message": "ERR410: The policy settings do not allow you to issue this request!"' in response
 
         self.createPolicy("setOTPPIN")
-        response = self.app.get(url(controller='userservice', action='setpin'),
-                                params={'selftest_user': 'root@myDefRealm'})
+        response = self.make_userselfservice_request('setpin',
+                                auth_user=auth_user)
         print response
         assert 'Missing parameter: \'userpin\'' in response
         assert '"code": 905' in response
 
 
-        response = self.app.get(url(controller='userservice', action='setpin'),
-                                params={'selftest_user': 'root@myDefRealm',
-                                        'serial': 'spass01',
+        response = self.make_userselfservice_request('setpin',
+                                auth_user=auth_user,
+                                params={'serial': 'spass01',
                                         'userpin': 'secretPin'})
         print response
         assert '"status": true' in response
         assert '"set userpin": 1' in response
 
         response = self.app.get(url(controller='validate', action='check'),
-                                params={'user': 'root@myDefRealm',
+                                params={'user': 'passthru_user1@myDefRealm',
                                         'pass': 'secretPin'})
         print response
         assert '"status": true' in response
         assert '"value": true' in response
 
-        response = self.app.get(url(controller='selfservice', action='setpin'),
-                                params={'selftest_user':'root@myDefRealm'})
+        response = self.make_selfservice_request('setpin',
+                                auth_user=auth_user)
         print response
         assert "<div id='passwordform'>" in response
 
         # testing the index and the list of the tokens
-        response = self.app.get(url(controller='selfservice', action='index'),
-                                params={'selftest_user': 'root@myDefRealm'})
+        response = self.make_selfservice_request('index',
+                                auth_user=auth_user)
 
         print "%r" % response
 
@@ -293,9 +316,13 @@ class TestSelfserviceController(TestController):
         '''
         self.deleteToken('token01')
 
-        response = self.app.get(url(controller='userservice', action='getSerialByOtp'),
-                                params={'selftest_user': 'root@myDefRealm',
-                                        'type': 'hmac',
+        auth_user = {
+             'login': 'passthru_user1@myDefRealm',
+             'password': 'geheim1'}
+
+        response = self.make_userselfservice_request('getSerialByOtp',
+                                auth_user=auth_user,
+                                params={'type': 'hmac',
                                         'otp': '885497'})
         print response
         assert '"status": false' in response
@@ -310,9 +337,9 @@ class TestSelfserviceController(TestController):
         assert '"status": true' in response
 
         self.createPolicy("getserial")
-        response = self.app.get(url(controller='userservice', action='getSerialByOtp'),
-                                params={'selftest_user': 'root@myDefRealm',
-                                        'type': 'hmac',
+        response = self.make_userselfservice_request('getSerialByOtp',
+                                auth_user=auth_user,
+                                params={'type': 'hmac',
                                         'otp': '459812'})
         print response
         # The token is not found, as it is not in the realm of the user
@@ -325,9 +352,9 @@ class TestSelfserviceController(TestController):
         assert '"value": 1' in response
 
         # NOw the token is found
-        response = self.app.get(url(controller='userservice', action='getSerialByOtp'),
-                                params={'selftest_user': 'root@myDefRealm',
-                                        'type': 'hmac',
+        response = self.make_userselfservice_request('getSerialByOtp',
+                                auth_user=auth_user,
+                                params={'type': 'hmac',
                                         'otp': '459812'})
         print response
         assert '"serial": "token01"' in response
@@ -355,41 +382,46 @@ class TestSelfserviceController(TestController):
         assert '"value": 1' in response
 
         # Now try to assign
-        response = self.app.get(url(controller='userservice', action='assign'),
-                                params={'selftest_user': 'root@myDefRealm',
-                                        'serial': 'token01'})
+
+        auth_user = {
+             'login': 'passthru_user1@myDefRealm',
+             'password': 'geheim1'}
+
+        response = self.make_userselfservice_request('assign',
+                            auth_user=auth_user, params={'serial': 'token01'})
+
         print response
         assert '"message": "ERR410: ' in response
 
         self.createPolicy("assign")
-        response = self.app.get(url(controller='userservice', action='assign'),
-                                params={'selftest_user': 'root@myDefRealm',
-                                        'serial': 'token01'})
+        response = self.make_userselfservice_request('assign',
+                                auth_user=auth_user,
+                                params={'serial': 'token01'})
         print response
         assert '"assign token": true' in response
 
         # unassign
-        response = self.app.get(url(controller='userservice', action='unassign'),
-                                params={'selftest_user': 'root@myDefRealm',
-                                        'serial': 'token01'})
+        response = self.make_userselfservice_request('unassign',
+                                auth_user=auth_user,
+                                params={'serial': 'token01'})
         print response
         assert '"message": "ERR410: The policy settings do not allow you to issue this request!",' in response
 
         self.createPolicy("unassign")
-        response = self.app.get(url(controller='userservice', action='unassign'),
-                                params={'selftest_user': 'root@myDefRealm',
-                                        'serial': 'token01'})
+        response = self.make_userselfservice_request('unassign',
+                                auth_user=auth_user,
+                                params={'serial': 'token01'})
         print response
         assert '"unassign token": true' in response
 
         # UI
-        response = self.app.get(url(controller='selfservice', action='assign'),
-                                params={'selftest_user': 'root@myDefRealm'})
+        response = self.make_selfservice_request('assign',
+                                auth_user=auth_user)
         print response
         assert "<div id='assignform'>" in response
 
-        response = self.app.get(url(controller='selfservice', action='unassign'),
-                                params={'selftest_user': 'root@myDefRealm'})
+        response = self.make_selfservice_request('unassign',
+                                auth_user=auth_user)
         print response
         assert "<div id='unassignform'>" in response
 
@@ -404,27 +436,31 @@ class TestSelfserviceController(TestController):
                                 params={'serial':'token01',
                                         'type': 'hmac',
                                         'otpkey': 'c4a3923c8d97e03af6a12fa40264c54b8429cf0d',
-                                        'user': 'root'
+                                        'user': 'passthru_user1@myDefRealm'
                                         })
         print response
         assert '"status": true' in response
 
-        response = self.app.get(url(controller='userservice', action='delete'),
-                                params={'serial': 'token01',
-                                        'selftest_user': 'root@myDefRealm'})
+        auth_user = {
+             'login': 'passthru_user1@myDefRealm',
+             'password': 'geheim1'}
+
+        response = self.make_userselfservice_request('delete',
+                                auth_user=auth_user,
+                                params={'serial': 'token01'})
         print response
         assert '"message": "ERR410: The policy settings do not allow you to issue this request!"' in response
 
         self.createPolicy("delete")
-        response = self.app.get(url(controller='userservice', action='delete'),
-                                params={'serial': 'token01',
-                                        'selftest_user': 'root@myDefRealm'})
+        response = self.make_userselfservice_request('delete',
+                                auth_user=auth_user,
+                                params={'serial': 'token01'})
         print response
         assert '"delete token": 1' in response
 
         # UI
-        response = self.app.get(url(controller='selfservice', action='delete'),
-                                params={'selftest_user': 'root@myDefRealm'})
+        response = self.make_selfservice_request('delete',
+                                auth_user=auth_user)
         print response
         assert "<div id='deleteform'>" in response
 
@@ -438,22 +474,27 @@ class TestSelfserviceController(TestController):
                                 params={'serial':'token01',
                                         'type': 'hmac',
                                         'otpkey': 'c4a3923c8d97e03af6a12fa40264c54b8429cf0d',
-                                        'user': 'root'
+                                        'user': 'passthru_user1@myDefRealm'
                                         })
         print response
         assert '"status": true' in response
 
         # disable
-        response = self.app.get(url(controller='userservice', action='disable'),
-                                params={'serial': 'token01',
-                                        'selftest_user': 'root@myDefRealm'})
+
+        auth_user = {
+             'login': 'passthru_user1@myDefRealm',
+             'password': 'geheim1'}
+
+        response = self.make_userselfservice_request('disable',
+                                auth_user=auth_user,
+                                params={'serial': 'token01'})
         print response
         assert '"message": "ERR410: The policy settings do not allow you to issue this request!",' in response
 
         self.createPolicy("disable")
-        response = self.app.get(url(controller='userservice', action='disable'),
-                                params={'serial': 'token01',
-                                        'selftest_user': 'root@myDefRealm'})
+        response = self.make_userselfservice_request('disable',
+                                auth_user=auth_user,
+                                params={'serial': 'token01'})
         print response
         assert '"disable token": 1' in response
 
@@ -465,16 +506,16 @@ class TestSelfserviceController(TestController):
 
         # now enable again
 
-        response = self.app.get(url(controller='userservice', action='enable'),
-                                params={'serial': 'token01',
-                                        'selftest_user': 'root@myDefRealm'})
+        response = self.make_userselfservice_request('enable',
+                                auth_user=auth_user,
+                                params={'serial': 'token01'})
         print response
         assert '"message": "ERR410: The policy settings do not allow you to issue this request!"' in response
 
         self.createPolicy("enable")
-        response = self.app.get(url(controller='userservice', action='enable'),
-                                params={'serial': 'token01',
-                                        'selftest_user': 'root@myDefRealm'})
+        response = self.make_userselfservice_request('enable',
+                                auth_user=auth_user,
+                                params={'serial': 'token01'})
         print response
         assert '"enable token": 1' in response
 
@@ -485,13 +526,13 @@ class TestSelfserviceController(TestController):
         assert '"LinOtp.Isactive": true' in response
 
         # UI
-        response = self.app.get(url(controller='selfservice', action='disable'),
-                                params={'selftest_user': 'root@myDefRealm'})
+        response = self.make_selfservice_request('disable',
+                                auth_user=auth_user)
         print response
         assert "<div id='disableform'>" in response
 
-        response = self.app.get(url(controller='selfservice', action='enable'),
-                                params={'selftest_user': 'root@myDefRealm'})
+        response = self.make_selfservice_request('enable',
+                                auth_user=auth_user)
         print response
         assert "<div id='enableform'>" in response
 
@@ -501,22 +542,26 @@ class TestSelfserviceController(TestController):
         '''
         self.deleteToken('token01')
 
-        response = self.app.get(url(controller='userservice', action='enroll'),
+        auth_user = {
+             'login': 'passthru_user1@myDefRealm',
+             'password': 'geheim1'}
+
+        response = self.make_userselfservice_request('enroll',
+                                auth_user=auth_user,
                                 params={'serial':'token01',
                                         'type': 'hmac',
-                                        'otpkey': 'c4a3923c8d97e03af6a12fa40264c54b8429cf0d',
-                                        'selftest_user': 'root@myDefRealm'
+                                        'otpkey': 'c4a3923c8d97e03af6a12fa40264c54b8429cf0d'
                                         })
         print response
         assert '"message": "ERR410: The policy settings do not allow you to issue this request!"' in response
 
         self.createPolicy('enrollHMAC')
 
-        response = self.app.get(url(controller='userservice', action='enroll'),
+        response = self.make_userselfservice_request('enroll',
+                                auth_user=auth_user,
                                 params={'serial':'token01',
                                         'type': 'hmac',
-                                        'otpkey': 'c4a3923c8d97e03af6a12fa40264c54b8429cf0d',
-                                        'selftest_user': 'root@myDefRealm'
+                                        'otpkey': 'c4a3923c8d97e03af6a12fa40264c54b8429cf0d'
                                         })
         print response
         assert '"status": true' in response
@@ -527,46 +572,105 @@ class TestSelfserviceController(TestController):
         assert '"LinOtp.TokenSerialnumber": "token01",' in response
         assert '"LinOtp.Isactive": true' in response
 
+    def test_enroll_onetime_spass(self):
+        '''
+        selfservice: testing enrollment of a onetime spass token as normal user
+        '''
+        self.deleteToken('token01')
+
+        ''' Verify that the relevant policy is required '''
+
+        auth_user = {
+             'login': 'passthru_user1@myDefRealm',
+             'password': 'geheim1'}
+
+        response = self.make_userselfservice_request('enroll',
+                                auth_user=auth_user,
+                                params={'serial':'token01',
+                                        'type': 'spass',
+                                        'pin': '!token0secret!'
+                                        })
+        print response
+        assert '"message": "ERR410: The policy settings do not allow you to issue this request!"' in response
+
+        ''' Verify that a spass token is properly created '''
+        self.createPolicy('enrollSPASS')
+
+        response = self.make_userselfservice_request('enroll',
+                                auth_user=auth_user,
+                                params={'serial':'token01',
+                                        'type': 'spass',
+                                        'onetime': 'true',
+                                        'pin': '!token0secret!'
+                                        })
+        print response
+        assert '"status": true' in response
+
+        response = self.app.get(url(controller='admin', action='show'),
+                                params={'serial': 'token01'})
+        print response
+        assert '"LinOtp.TokenSerialnumber": "token01",' in response
+        assert '"LinOtp.Isactive": true' in response
+
+        ''' Verify this spass works once... '''
+        parameters = {
+                      "user"     : 'passthru_user1@myDefRealm',
+                      "pass"     : '!token0secret!',
+                      }
+        response = self.app.get(url(controller='validate', action='check'), params=parameters)
+        assert '"status": true' in response
+        assert '"value": true'  in response
+
+        ''' ... and exactly once '''
+        response = self.app.get(url(controller='validate', action='check'), params=parameters)
+        assert '"status": true' in response
+        assert '"value": false'  in response
 
     def test_webprovision(self):
         '''
         selfservice: testing user webprovision
         '''
         self.deleteToken('token01')
-        response = self.app.get(url(controller='userservice', action='webprovision'),
+
+        auth_user = {
+             'login': 'passthru_user1@myDefRealm',
+             'password': 'geheim1'}
+
+        response = self.make_userselfservice_request('webprovision',
+                                auth_user=auth_user,
                                 params={'serial':'token01',
-                                        'type': 'hmac',
-                                        'selftest_user': 'root@myDefRealm'
-                                        })
+                                        'type': 'hmac'})
+
         self.assertTrue('"message": "valid types are \'oathtoken\' and \'googleauthenticator\' and \'googleauthenticator_time\'. You provided hmac",' in response,response)
 
-        response = self.app.get(url(controller='userservice', action='webprovision'),
+        response = self.make_userselfservice_request('webprovision',
+                                auth_user=auth_user,
                                 params={'serial':'token01',
-                                        'type': 'googleauthenticator',
-                                        'selftest_user': 'root@myDefRealm'
+                                        'type': 'googleauthenticator'
                                         })
 
         self.assertTrue('"message": "ERR410: The policy settings do not allow you to issue this request!"' in response,response)
 
         self.createPolicy('webprovisionGOOGLE')
 
-        response = self.app.get(url(controller='userservice', action='webprovision'),
+        response = self.make_userselfservice_request('webprovision',
+                                auth_user=auth_user,
                                 params={'prefix':'LSGO',
-                                        'type': 'googleauthenticator',
-                                        'selftest_user': 'root@myDefRealm'
+                                        'type': 'googleauthenticator'
                                         })
-        self.assertTrue('"url": "otpauth://hotp/LSGO' in response, response)
+        self.assertTrue('"url": "otpauth://hotp/LinOTP:LSGO' in response,
+                        response)
 
         # test
         response = self.app.get(url(controller='admin', action='show'),
-                                params={'user': 'root'})
+                                params={'user': 'passthru_user1@myDefRealm'})
         self.assertTrue('"LinOtp.TokenSerialnumber": "LSGO' in response,response)
         self.assertTrue('"LinOtp.Isactive": true' in response, response)
 
         # UI
 
-        response = self.app.get(url(controller='selfservice', action='webprovisiongoogletoken'),
-                                params={'selftest_user': 'root@myDefRealm'})
+        response = self.make_selfservice_request('webprovisiongoogletoken',
+                                auth_user=auth_user)
         self.assertTrue("googletokenform" in response.body, response)
 
         return
@@ -584,3 +688,60 @@ class TestSelfserviceController(TestController):
         TODO selfservice: testing getting multiple otps
         '''
         pass
+
+    def test_privilege_escalation_fix(self):
+
+        """
+        Check if logged in users can not see token data
+        of another user through /userservice/context
+
+        refers LINOTP-702
+        """
+
+        auth_user = {
+             'login': 'passthru_user1@myDefRealm',
+             'password': 'geheim1'}
+
+        response = self.make_userselfservice_request('context',
+                                auth_user=auth_user,
+                                params={'user': 'hans'})
+
+        response_dict = json.loads(response.body)
+
+        user = response_dict['user']
+        self.assertEqual(user, 'passthru_user1')
+
+    def test_setdescription(self):
+        '''
+        selfservice: testing set token description as normal user
+        '''
+
+        user = 'passthru_user1@myDefRealm'
+        selfservice_user = {
+            'login': user,
+            'password': 'geheim1'
+        }
+
+        serial = 'set_description_token'
+        params = {
+            'type': 'hmac',
+            'serial': serial,
+            'genkey': 1,
+            'user': user
+            }
+
+        response = self.make_admin_request('init', params=params)
+        assert "googleurl" in response, response
+
+        params={
+            'serial': serial,
+            'description': 'my super token'
+        }
+
+        response = self.make_userselfservice_request(
+            'setdescription', params=params, auth_user=selfservice_user)
+
+        assert '"set description": 1' in response
+
+        response = self.make_admin_request('show', params={'serial': serial})
+        assert 'my super token' in response
