@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 #
 #    LinOTP - the open source solution for two factor authentication
-#    Copyright (C) 2010 - 2016 LSE Leading Security Experts GmbH
+#    Copyright (C) 2010 - 2019 KeyIdentity GmbH
 #
 #    This file is part of LinOTP server.
 #
@@ -19,35 +19,35 @@
 #    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #
 #
-#    E-mail: linotp@lsexperts.de
+#    E-mail: linotp@keyidentity.com
 #    Contact: www.linotp.org
-#    Support: www.lsexperts.de
+#    Support: www.keyidentity.com
 #
 
 
 """ """
 
-import datetime
 import binascii
+import datetime
+from hashlib import sha1, sha256, sha512
 import hmac
+import logging
+import random
 import struct
 import time
-import random
+import traceback
+
+from freezegun import freeze_time
+
+from linotp.lib.crypto import geturandom
+from linotp.tests import TestController, url
 
 try:
     import json
 except ImportError:
     import simplejson as json
 
-import traceback
 
-from hashlib import sha1, sha256, sha512
-
-from linotp.lib.crypt   import geturandom
-from linotp.tests import TestController, url
-
-
-import logging
 log = logging.getLogger(__name__)
 
 
@@ -290,7 +290,9 @@ class TestTotpController(TestController):
         if typ is None:
             typ = 'totp'
 
-        param = { 'user': user, 'pin':pin, 'serial': serial, 'type':typ, 'timeStep':timeStep, 'otplen' : otplen, 'hashlib':hashlib}
+        param = {'user': user, 'pin':pin, 'serial': serial,
+                 'type':typ, 'timeStep':timeStep,
+                 'otplen' : otplen, 'hashlib':hashlib}
 
         if key is not None:
             param['otpkey'] = key
@@ -384,26 +386,32 @@ class TestTotpController(TestController):
 
         otpSet = set()
 
-        for i in range(1, 5):
-            offset = i * step
-            (otp, counter) = t1.getOtp(offset=offset)
-            tt = t1.getTimeFromCounter(counter)
-            log.debug("tokentime: %r" % tt)
+        # Freeze time to the current system time
+        with freeze_time(datetime.datetime.now()) as frozen_datetime:
+            for i in range(1, 5):
+                offset = i * step
+                (otp, counter) = t1.getOtp(offset=offset)
+                tt = t1.getTimeFromCounter(counter)
+                log.debug("tokentime: %r" % tt)
 
-            res = self.checkOtp(user, otp)
+                res = self.checkOtp(user, otp)
 
-            if otp not in otpSet:
-                assert '"value": true' in res.body
-                resInfo = self.getTokenInfo(tserial)
-                tInfo = json.loads(resInfo.get('result').get('value').get('data')[0].get('LinOtp.TokenInfo'))
-                tShift = tInfo.get('timeShift')
-                assert tShift <= offset and tShift >= offset - step
-            else:
-                assert '"value": false' in res.body
+                if otp not in otpSet:
+                    assert '"value": true' in res.body
+                    resInfo = self.getTokenInfo(tserial)
+                    tInfo = json.loads(resInfo.get('result').get('value').get('data')[0].get('LinOtp.TokenInfo'))
+                    tShift = tInfo.get('timeShift')
+                    assert int(tShift) <= offset + step
+                    assert int(tShift) >= offset - step
+                else:
+                    assert '"value": false' in res.body
 
-            otpSet.add(otp)
-            time.sleep(step / 2)
-            log.debug('res')
+                otpSet.add(otp)
+
+                # Jump to the future
+                frozen_datetime.tick(delta=datetime.timedelta(seconds=step/2))
+
+                log.debug('res')
 
         return
 
@@ -428,27 +436,32 @@ class TestTotpController(TestController):
 
         otpSet = set()
 
-        for i in range(1):
-            offset = i * step * -1
-            (otp, counter) = t1.getOtp(offset=offset)
-            tt = t1.getTimeFromCounter(counter)
-            log.debug("tokentime: %r" % tt)
+        # Freeze time to the current system time
+        with freeze_time(datetime.datetime.now()) as frozen_time:
+            for i in range(1):
+                offset = i * step * -1
+                (otp, counter) = t1.getOtp(offset=offset)
+                tt = t1.getTimeFromCounter(counter)
+                log.debug("tokentime: %r" % tt)
 
-            res = self.checkOtp(user, otp)
+                res = self.checkOtp(user, otp)
 
-            if otp not in otpSet:
-                if '"value": true' not in res.body:
-                    assert '"value": true' in res.body
-                resInfo = self.getTokenInfo(tserial)
-                tInfo = json.loads(resInfo.get('result').get('value').get('data')[0].get('LinOtp.TokenInfo'))
-                tShift = tInfo.get('timeShift')
-                assert tShift <= offset and tShift >= offset - step
-            else:
-                assert '"value": false' in res.body
+                if otp not in otpSet:
+                    if '"value": true' not in res.body:
+                        assert '"value": true' in res.body
+                    resInfo = self.getTokenInfo(tserial)
+                    tInfo = json.loads(resInfo.get('result').get('value').get('data')[0].get('LinOtp.TokenInfo'))
+                    tShift = tInfo.get('timeShift')
+                    assert tShift <= offset + step and tShift >= offset - step
+                else:
+                    assert '"value": false' in res.body
 
-            otpSet.add(otp)
-            time.sleep(step)
-            log.debug('res')
+                otpSet.add(otp)
+
+                # Jump to the future
+                frozen_time.tick(delta=datetime.timedelta(seconds=step))
+
+                log.debug('res')
 
         return
 
@@ -459,38 +472,212 @@ class TestTotpController(TestController):
         user = 'root'
         step = 30
 
-        t1 = TotpToken(timestep=step)
-        key = t1.getKey().encode('hex')
-        step = t1.getTimeStep()
+        # Freeze time to the current system time
+        with freeze_time(datetime.datetime.now()) as frozen_time:
+            t1 = TotpToken(timestep=step)
+            key = t1.getKey().encode('hex')
+            step = t1.getTimeStep()
 
-        tserial = self.addToken(user=user, otplen=t1.digits, typ='totp', key=key, timeStep=step)
+            tserial = self.addToken(user=user, otplen=t1.digits,
+                                    typ='totp', key=key, timeStep=step)
 
-        self.serials.append(tserial)
+            self.serials.append(tserial)
 
-        (otp, counter) = t1.getOtp()
-        tt = t1.getTimeFromCounter(counter)
-        log.debug("tokentime: %r" % tt)
+            (otp, counter) = t1.getOtp()
+            tt = t1.getTimeFromCounter(counter)
+            log.debug("tokentime: %r" % tt)
 
-        res = self.checkOtp(user, otp)
-        assert '"value": true' in res.body
+            res = self.checkOtp(user, otp)
+            assert '"value": true' in res.body
 
-        ''' reusing the otp again will fail'''
-        res = self.checkOtp(user, otp)
-        assert '"value": false' in res.body
+            ''' reusing the otp again will fail'''
+            res = self.checkOtp(user, otp)
+            assert '"value": false' in res.body
 
-        time.sleep(step)
+            # Jump to the future
+            frozen_time.tick(delta=datetime.timedelta(seconds=step))
 
+            ''' after a while, we could do a check again'''
+            (otp, counter) = t1.getOtp()
+            tt = t1.getTimeFromCounter(counter)
+            log.debug("tokentime: %r" % tt)
 
-        ''' after a while, we could do a check again'''
-        (otp, counter) = t1.getOtp()
-        tt = t1.getTimeFromCounter(counter)
-        log.debug("tokentime: %r" % tt)
-
-        res = self.checkOtp(user, otp)
-        assert '"value": true' in res.body
+            res = self.checkOtp(user, otp)
+            assert '"value": true' in res.body
 
         return
 
+    def test_resync_no_replay(self):
+        '''
+            totp test: verify that auto resync does not succeed with reused (sync) OTPs
+
+            We will use the same OTP twice. Once for starting the sync
+            and then to complete it. Both of those should not yield a
+            valid authentication. The user must provide two consecutive
+            OTPs to finish the sync. The second OTP must be within a
+            small timeframe after the first.
+        '''
+        user = 'root'
+        step = 30
+
+        params = {
+            'AutoResyncTimeout': '240',
+            'AutoResync': True
+        }
+
+        response = self.make_system_request('setConfig', params=params)
+        assert 'false' not in response.body
+
+        for offset in range(10*step, 20*step, step/2):
+            # Freeze time to the current system time
+            with freeze_time(datetime.datetime.now()) as frozen_time:
+                t1 = TotpToken(timestep=step)
+                key = t1.getKey().encode('hex')
+                step = t1.getTimeStep()
+
+                tserial = self.addToken(user=user, otplen=t1.digits,
+                                        typ='totp', key=key, timeStep=step)
+
+                self.serials.append(tserial)
+
+                (otp, counter) = t1.getOtp()
+                _tt1 = t1.getTimeFromCounter(counter)
+
+                res = self.checkOtp(user, otp)
+                assert '"value": true' in res.body
+
+                # replay doesn't work
+                res = self.checkOtp(user, otp)
+                assert '"value": false' in res.body
+
+                # advance to a future time where the old otp is no longer valid
+                frozen_time.tick(delta=datetime.timedelta(seconds=offset))
+
+                # start resync
+                res = self.checkOtp(user, otp)
+                assert '"value": false' in res.body, "%s: %s" %(offset,res.body)
+
+                # finish resync
+                res = self.checkOtp(user, otp)
+                assert '"value": true' not in res.body, offset
+
+    def test_resync_non_consecutive(self):
+        '''
+            totp test: verify that auto resync does not succeed with non-consecutive OTPs
+        '''
+        user = 'root'
+        timeWindow = 180
+        params = {
+            'AutoResyncTimeout': '240',
+            'AutoResync': True
+        }
+
+        response = self.make_system_request('setConfig', params=params)
+        assert 'false' not in response.body
+
+        # Freeze time to the current system time
+        with freeze_time(datetime.datetime.now()) as frozen_time:
+            t1 = TotpToken()
+            key = t1.getKey().encode('hex')
+            step = t1.getTimeStep()
+
+            tserial = self.addToken(user=user, otplen=t1.digits,
+                                    typ='totp', key=key, timeStep=step)
+
+            self.serials.append(tserial)
+
+            (otp, counter) = t1.getOtp()
+
+            res = self.checkOtp(user, otp)
+            assert '"value": true' in res.body
+
+            # advance to a future time where the old otp is no longer valid
+            frozen_time.tick(delta=datetime.timedelta(seconds=timeWindow))
+            res = self.checkOtp(user, otp)
+            assert '"value": false' in res.body
+
+            # skip enough OTPs to leave the current window
+            counter += 2 * timeWindow
+
+            # get the first token
+            (first_otp, _) = t1.getOtp(counter=counter)
+            # get the second token
+            (second_otp, counter) = t1.getOtp(counter=counter+step)
+
+            # start resync with 2nd otp
+            res = self.checkOtp(user, second_otp)
+            assert '"value": false' in res.body
+
+            # provide the first OTP for the resync, it should fail
+            res = self.checkOtp(user, first_otp)
+            assert '"value": true' not in res.body
+
+    def test_resync_consecutive(self):
+        '''
+            totp test: verify that auto resync does succeed with consecutive OTPs and fails if they are outside of the range
+        '''
+        user = 'root'
+        timeWindow = 180
+        syncTimeout = 240
+        step = 30
+        params = {
+            'AutoResyncTimeout': '%s' % syncTimeout,
+            'AutoResync': True
+        }
+
+        response = self.make_system_request('setConfig', params=params)
+        assert 'false' not in response.body
+
+        for offset in range(1, 5):
+            # Freeze time to the current system time
+            with freeze_time(datetime.datetime.now()) as frozen_time:
+                t1 = TotpToken(timestep=step)
+                key = t1.getKey().encode('hex')
+
+                tserial = self.addToken(user=user, otplen=t1.digits,
+                                        typ='totp', key=key, timeStep=step)
+
+                self.serials.append(tserial)
+
+                (otp, counter) = t1.getOtp()
+                res = self.checkOtp(user, otp)
+                assert '"value": true' in res.body
+
+                log.info("Successful counter: %s", counter)
+
+                # advance to a future time where the old otp is no longer valid
+                frozen_time.tick(delta=datetime.timedelta(seconds=timeWindow))
+                res = self.checkOtp(user, otp)
+                assert '"value": false' in res.body
+
+                counter_advance = (40 * timeWindow)
+                log.info("Advancing counter by %s", counter_advance)
+                # skip enough OTPs to leave the current window
+                counter = (counter * step) + counter_advance
+
+                # get the first token
+                (first_otp, first_counter) = t1.getOtp(counter=counter)
+
+                # get the second token that is offset by a few but within range
+                (second_otp, second_counter) = t1.getOtp(counter=counter+step*offset)
+
+                log.info("First OTP: %s (%s), Second OTP: %s (%s)",
+                        first_otp, first_counter,
+                        second_otp, second_counter)
+
+                # start resync with a valid OTP
+                res = self.checkOtp(user, first_otp)
+                assert '"value": false' in res.body
+
+                # provide the second otp that follows the previous one
+                res = self.checkOtp(user, second_otp)
+
+                if offset <= 3:
+                    # as long as the OTP is not out of the sync range it should be good
+                    assert '"value": true' in res.body, offset
+                else:
+                    # if we are out of the sync range the OTP should be rejected
+                    assert '"value": false' in res.body, offset
 
     def test_getotp(self):
         '''
@@ -541,25 +728,14 @@ class TestTotpController(TestController):
                 resp = json.loads(response.body)
 
                 otpres = resp.get('result').get('value').get('otp')
-
                 otp1 = otpres.get(str(counter))
-
-                ''' calculate the utc offset'''
-                uTime = otp1.get('time')
-                fTime = self.time2float(uTime)
-                ut = datetime.datetime.utcfromtimestamp(fTime)
-                ot = datetime.datetime.fromtimestamp(fTime)
-                td = (ut - ot)
-                delta = (td.microseconds * 1.0 + (td.seconds + td.days * 24 * 3600) * 10 ** 6) / 10.0 ** 6
-
-                ''' compare the time absolute values'''
-                dt = datetime.datetime.strptime(uTime, tFormat) + datetime.timedelta(seconds=delta)
-                ct = datetime.datetime.strptime(curTime, tFormat)
-                cl = ct - datetime.timedelta(seconds=step)
-
-                assert dt <= ct and cl <= dt
                 assert otp1.get('otpval') == otp
 
+                # verify: the first otp matches the unix start time + timeslice
+                if '1' in otpres:
+                    otp_one = otpres.get('1')
+                    uTime = otp_one.get('time')
+                    assert '1970-01-01 00:00:30' == uTime
 
             self.removeTokens()
 
