@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 #
 #    LinOTP - the open source solution for two factor authentication
-#    Copyright (C) 2010 - 2016 LSE Leading Security Experts GmbH
+#    Copyright (C) 2010 - 2019 KeyIdentity GmbH
 #
 #    This file is part of LinOTP server.
 #
@@ -19,9 +19,9 @@
 #    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #
 #
-#    E-mail: linotp@lsexperts.de
+#    E-mail: linotp@keyidentity.com
 #    Contact: www.linotp.org
-#    Support: www.lsexperts.de
+#    Support: www.keyidentity.com
 #
 
 
@@ -41,26 +41,55 @@ We assume port 5001 is used (default). If you want to use another port you can
 specify it with nose-testconfig (e.g. --tc=paster.port:5005).
 """
 
-import logging
 import json
+import urlparse
 
-from linotp.tests import TestController
+import httplib2
+from mock import patch
 
-log = logging.getLogger(__name__)
+from linotp.tests.functional_special import TestSpecialController
 
-DEFAULT_NOSE_CONFIG = {
-    'paster': {
-        'port': '5001',
-        }
+
+# mocking hook is startting here
+HTTP_RESPONSE_FUNC = None
+HTTP_RESPONSE = None
+
+def mocked_http_request(HttpObject, *argparams, **kwparams):
+
+    resp = 200
+    body = kwparams.get('body', '')
+    params = dict(urlparse.parse_qsl(body))
+
+    content = {
+        "version": "LinOTP MOCK",
+        "jsonrpc": "2.0",
+        "result": {
+            "status": True,
+            "value": True
+        },
+        "id": 0
     }
-try:
-    from testconfig import config as nose_config
-except ImportError as exc:
-    print "You need to install nose-testconfig. Will use default values."
-    nose_config = None
+
+    global HTTP_RESPONSE
+    if HTTP_RESPONSE:
+        status, response = HTTP_RESPONSE
+        if response:
+            content = response
+            resp = status
+        HTTP_RESPONSE = None
+
+    global HTTP_RESPONSE_FUNC
+    if HTTP_RESPONSE_FUNC:
+        test_func = HTTP_RESPONSE_FUNC
+        resp, content = test_func(params)
+        HTTP_RESPONSE_FUNC = None
+
+    return resp, json.dumps(content)
 
 
-class TestRemoteToken2(TestController):
+
+
+class TestRemoteToken2(TestSpecialController):
 
     def setUp(self):
         '''
@@ -69,17 +98,10 @@ class TestRemoteToken2(TestController):
         If the realms are deleted also the table TokenRealm gets deleted
         and we loose the information how many tokens are within a realm!
         '''
-        TestController.setUp(self)
-        self.set_config_selftest()
-        if nose_config and 'paster' in nose_config:
-            self.paster_port = nose_config['paster']['port']
-        else:
-            self.paster_port = DEFAULT_NOSE_CONFIG['paster']['port']
+        TestSpecialController.setUp(self)
         self.remote_url = 'http://127.0.0.1:%s' % self.paster_port
 
-        '''
-        Init the tests....
-        '''
+        # Init the tests....
         self.delete_all_policies()
         self.delete_all_token()
 
@@ -90,6 +112,10 @@ class TestRemoteToken2(TestController):
         self.create_realms()
         return
 
+    def tearDown(self):
+        ''' Overwrite parent tear down, which removes all realms '''
+        return
+
     def create_pin_policies(self):
 
         # set the otppin=2 (no pin) for the 'nopin' realm
@@ -340,16 +366,13 @@ class TestRemoteToken2(TestController):
         self.assertTrue('"value": true' in response, response)
         return serial
 
-    def tearDown(self):
-        ''' Overwrite parent tear down, which removes all realms '''
-        return
-
-    ### define Admins
-
+    @patch.object(httplib2.Http, 'request', mocked_http_request)
     def test_check_unassigned_tokens(self):
         """
         RT2: unassigned remote token and target yubikey (check_s)
         """
+        global HTTP_RESPONSE_FUNC
+
         (y_serial, r_serial) = self.create_tokens()
 
         # check otps on yubikey
@@ -362,16 +385,34 @@ class TestRemoteToken2(TestController):
         # check otps on remote
         otp = self.yubi_valid_otps[1]
 
+        def check_func1(params):
+            resp = 200
+            value = params.get('pass') == otp
+            content = {
+                "version": "LinOTP MOCK",
+                "jsonrpc": "2.0",
+                "result": {
+                    "status": True,
+                    "value": value
+                },
+                "id": 0
+            }
+            return resp, content
+        HTTP_RESPONSE_FUNC = check_func1
+
         params = {'serial': r_serial, 'pass': otp}
         response = self.make_validate_request(action='check_s', params=params)
         self.assertIn('"value": true', response, response)
 
         return
 
+    @patch.object(httplib2.Http, 'request', mocked_http_request)
     def test_check_assigned_tokens(self):
         """
         RT2: assigned remote token without pin and target yubikey
         """
+        global HTTP_RESPONSE_FUNC
+
         (y_serial, r_serial) = self.create_tokens()
 
         params = {}
@@ -396,16 +437,34 @@ class TestRemoteToken2(TestController):
 
         otp = self.yubi_valid_otps[3]
 
+        def check_func1(params):
+            resp = 200
+            value = params.get('pass') == otp
+            content = {
+                "version": "LinOTP MOCK",
+                "jsonrpc": "2.0",
+                "result": {
+                    "status": True,
+                    "value": value
+                },
+                "id": 0
+            }
+            return resp, content
+        HTTP_RESPONSE_FUNC = check_func1
+
         params = {'user': 'passthru_user1@withpin', 'pass': otp}
         response = self.make_validate_request(action='check', params=params)
         self.assertIn('"value": true', response, response)
 
         return
 
+    @patch.object(httplib2.Http, 'request', mocked_http_request)
     def test_check_tokens_with_pin(self):
         """
         RT2: remote token with local pin and target yubikey
         """
+        global HTTP_RESPONSE_FUNC
+
         (y_serial, r_serial) = self.create_tokens()
 
         params = {}
@@ -436,7 +495,23 @@ class TestRemoteToken2(TestController):
         val = resp.get('result', {}).get('value', {})
         self.assertTrue(val['set pin'] == 1, response)
 
-        passw = "local%s" % self.yubi_valid_otps[5]
+        otp = self.yubi_valid_otps[5]
+        passw = "local%s" % otp
+
+        def check_func1(params):
+            resp = 200
+            value = params.get('pass') == otp
+            content = {
+                "version": "LinOTP MOCK",
+                "jsonrpc": "2.0",
+                "result": {
+                    "status": True,
+                    "value": value
+                },
+                "id": 0
+            }
+            return resp, content
+        HTTP_RESPONSE_FUNC = check_func1
 
         params = {'user': 'passthru_user1@withpin', 'pass': passw}
         response = self.make_validate_request(action='check', params=params)
@@ -444,10 +519,12 @@ class TestRemoteToken2(TestController):
 
         return
 
+    @patch.object(httplib2.Http, 'request', mocked_http_request)
     def test_check_tokens_with_otppin_policy(self):
         """
         RT2: remote token with otppin policy and target yubikey
         """
+        global HTTP_RESPONSE_FUNC
 
         self.create_pin_policies()
 
@@ -473,7 +550,23 @@ class TestRemoteToken2(TestController):
         response = self.make_admin_request('assign', params)
         self.assertIn('"value": true', response, response)
 
-        passw = "geheim1%s" % self.yubi_valid_otps[1]
+        otp = self.yubi_valid_otps[1]
+        passw = "geheim1%s" % otp
+
+        def check_func1(params):
+            resp = 200
+            value = params.get('pass') == otp
+            content = {
+                "version": "LinOTP MOCK",
+                "jsonrpc": "2.0",
+                "result": {
+                    "status": True,
+                    "value": value
+                },
+                "id": 0
+            }
+            return resp, content
+        HTTP_RESPONSE_FUNC = check_func1
 
         params = {'user': 'passthru_user1@withpin', 'pass': passw}
         response = self.make_validate_request(action='check', params=params)
@@ -481,13 +574,12 @@ class TestRemoteToken2(TestController):
 
         return
 
+    @patch.object(httplib2.Http, 'request', mocked_http_request)
     def test_check_tokens_with_autoassign(self):
         """
         RT2: remote token with otppin, autoassign policy and target yubikey
         """
-
-        self.skipTest("the remote server is not aware of changed config - "
-                      " s. replication config flag")
+        global HTTP_RESPONSE_FUNC
 
         self.create_pin_policies()
         self.create_autoassign_policy()
@@ -511,7 +603,23 @@ class TestRemoteToken2(TestController):
         self.assertEqual(username, 'passthru_user1', response)
 
         # check otps on remote
-        passw = "geheim1%s" % self.yubi_valid_otps[9]
+        otp = self.yubi_valid_otps[9]
+        passw = "geheim1%s" % otp
+
+        def check_func1(params):
+            resp = 200
+            value = params.get('pass') == otp
+            content = {
+                "version": "LinOTP MOCK",
+                "jsonrpc": "2.0",
+                "result": {
+                    "status": True,
+                    "value": value
+                },
+                "id": 0
+            }
+            return resp, content
+        HTTP_RESPONSE_FUNC = check_func1
 
         params = {'user': 'passthru_user1@withpin', 'pass': passw}
         response = self.make_validate_request(action='check', params=params)
@@ -528,13 +636,12 @@ class TestRemoteToken2(TestController):
 
         return
 
-    def test_check_tokens_with_autoassign_forward(self):
+    def test_00000_check_tokens_with_autoassign_forward(self):
         """
         RT2: remote token with otppin, autoassign forward policy and yubikey
         """
-
-        self.skipTest("the remote server is not aware of changed config - "
-                      " s. replication config flag")
+        self.skipTest("this test requires a real running linotp as it "
+                      "triggers an outenrollment")
 
         self.create_pin_policies()
         self.create_autoassign_policy()
@@ -543,7 +650,9 @@ class TestRemoteToken2(TestController):
         (y_serial, r_serial) = self.create_tokens()
 
         # check otps on remote
-        passw = "geheim1%s" % self.yubi_valid_otps[10]
+        otp = self.yubi_valid_otps[10]
+        passw = "geheim1%s" % otp
+
         params = {'user': 'passthru_user1@withpin', 'pass': passw}
         response = self.make_validate_request(action='check', params=params)
         self.assertIn('"value": true', response, response)
