@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 #
 #    LinOTP - the open source solution for two factor authentication
-#    Copyright (C) 2010 - 2016 LSE Leading Security Experts GmbH
+#    Copyright (C) 2010 - 2019 KeyIdentity GmbH
 #
 #    This file is part of LinOTP server.
 #
@@ -19,9 +19,9 @@
 #    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #
 #
-#    E-mail: linotp@lsexperts.de
+#    E-mail: linotp@keyidentity.com
 #    Contact: www.linotp.org
-#    Support: www.lsexperts.de
+#    Support: www.keyidentity.com
 #
 
 """
@@ -30,7 +30,7 @@
 """
 
 from unittest import TestCase
-from linotp.lib.pbkdf2 import pbkdf2
+from linotp.lib.crypto.pbkdf2 import pbkdf2
 from linotp.lib.utils import config_get
 from linotp.lib.ImportOTP.PSKC import parsePSKCdata
 from linotp.lib.ImportOTP.DPWplain import parseDPWdata
